//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA rtlinux executive declarations
//
// Project:	Yasa 2
// Author :	Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/executives/rtlinux/kernel/include/yasa_rtlinux.h,v 1.2 2003/01/22 17:02:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef	YASA_RTLINUX_INCLUDE
#define	YASA_RTLINUX_INCLUDE

//#include <yasa/yasa.h>			// do not include yasa/yasa.h here !!!
#include <rtl_time.h>
#include <posix/unistd.h>



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef INT64
 #define INT64						long long
#endif
#ifndef YASA_TIME
 #define YASA_TIME					hrtime_t
#endif
#define YASA_TIME_INFINITY			HRTIME_INFINITY


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define YASA_EXECUTIVE_NAME			"rtlinux"
#define YASA_EXECUTIVE_VERSION		"1.7"
#define YASA_DEFAULT_DEADLINE		YASA_TIME_INFINITY
#define YASA_DEFAULT_RESUMETIME		YASA_TIME_INFINITY
#define YASA_DEFAULT_REQUIREDTIME	0		// def. laxity=YASA_TIME_INFINITY=deadline-required
#define YASA_LINUX_THREAD_NAME		"Linux"
#define YASA_LINUX_THREAD_ID		0

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define YASA_MALLOC(size)			kmalloc(size, GFP_KERNEL)
#define YASA_FREE(ptr)				kfree(ptr)
#define PRINTK						printk
#define YASA_TEST_AND_SET_BIT(bit,ptr)	test_and_set_bit(bit, ptr)
#define YASA_CLEAR_BIT(bit,ptr)			clear_bit(bit, ptr)

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern int yasa_write(void *handle, const char *buffer);
#define YASA_LOGWRITE(handle, buffer)				yasa_write(handle, buffer)
#define YASA_LOGWRITE1(string, arg)					sprintf(buffer,string, arg); \
													YASA_LOGWRITE( handle, buffer )
#define YASA_LOGWRITE2(string, arg1, arg2)			sprintf(buffer,string, arg1, arg2); \
													YASA_LOGWRITE( handle, buffer )
#define YASA_LOGWRITE3(string, arg1, arg2, arg3)	sprintf(buffer,string, arg1, arg2,arg3); \
													YASA_LOGWRITE( handle, buffer )
#define YASA_LOGWRITE4(string, arg1, arg2, arg3, arg4)	sprintf(buffer,string, arg1, arg2,arg3,arg4); \
													YASA_LOGWRITE( handle, buffer )
#define YASA_LOGEND							0xff


// Min/Max Definitions
#define YASA_MIN_CPU						NR_CPUS
#define YASA_MAX_CPU						NR_CPUS
#define YASA_MIN_PRIORITY					0			// see sched_get_priority_max()
#define YASA_MAX_PRIORITY					1000000		// see sched_get_priority_min()
#define YASA_MAX_LOGGINGSIZE				131072		// see linux/mm/slab.c row: 1542 BUG()
#define YASA_MIN_TICK						10000
#define YASA_DEF_TICK						200000
//#define YASA_MAX_TICK						YASA_TIME_INFINITY-1
#define YASA_MAX_TICK						YASA_TIME_INFINITY
#define YASA_MUTEX_DEFAULT_PRIORITY			YASA_MIN_PRIORITY-1
#define YASA_MUTEX_DEFAULT_CEILING			YASA_MAX_PRIORITY+1
#define YASA_MUTEX_MAXSTACK					RTL_SEM_NSEMS_MAX

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// scheduler mappings
#define YASA_SCHEDULER						struct rtl_sched_cpu_struct
#define YASA_TASKSET(scheduler) 			(scheduler->rtl_tasks)
#define YASA_CURRENT_SCHEDULER				LOCAL_SCHED					// use macro from <rtlinux/schedulers/rtl_sched.h>
#define YASA_SCHEDULER_ATTR(sched)			( sched->yasa_schedulerattr )
#define YASA_GETCURRENTTIME(sched)			(sched->clock->gethrtime(sched->clock))
#define YASA_SCHEDULERSTART(sched)			(sched->schedulerstart)
#define YASA_SCHEDULERDATA(sched)			(sched->yasa_schedulerdata)
#define YASA_RESCHEDULE						rtl_schedule				// reschedule current cpu

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// scheduler parameter mappings
#define YASA_SCHEDPARAM						struct rtl_sched_param

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// thread mappings
#define YASA_THREAD							RTL_THREAD_STRUCT
#define YASA_ATTRIBUTES						pthread_attr_t
#define YASA_CURRENT_THREAD					RTL_CURRENT					// get current thread
#define YASA_NEXT_THREAD(thread) 			thread->next
#define YASA_RESUME_TIME(thread) 			thread->resume_time
#define YASA_NEXTACTIVATION_TIME(thread) 	thread->nextactivation_time
#define YASA_CURRENTACTIVATION_TIME(thread)	thread->currentactivation_time
#define YASA_DEADLINE(thread)				thread->sched_param.yasa_deadline
#define YASA_DEADLINE_TOLERANCE(thread)		thread->sched_param.deadlinetolerance
#define YASA_CURRENT_DEADLINE(thread)		thread->yasa_currentdeadline
#define YASA_PERIOD(thread) 				thread->period
#define YASA_COMPUTED_TIME(thread)			thread->yasa_computedtime
#define YASA_REQUIRED_TIME(thread)			thread->sched_param.yasa_requiredtime
#define YASA_PRIORITY(thread)				thread->sched_param.sched_priority
#define YASA_MAXLAXITY(thread)				thread->maxlaxity
#define YASA_EXCLUDED(thread)				thread->yasa_excluded
#define YASA_SCHEDULINGDATA(thread)			thread->yasa_schedulingdata
#define YASA_THREADTYPE(thread)				thread->threadtype
#define YASA_THREAD_EXCLUDED(thread)		thread->yasa_excluded
#define YASA_THREAD_SCHEDULER_DATA(thread)	thread->yasa_schedulerdata
#define YASA_DL_BEHAVIOUR(thread)			thread->sched_param.yasa_dl_behaviour
#define YASA_EMERGENCYTHREAD(thread)		thread->sched_param.yasa_dl_emergencythread
#define YASA_NEXTACTIVATIONVECTORPOS(thread)	thread->nextactivationvectorpos
#define YASA_CURRENTACTIVATIONVECTORPOS(thread)	thread->currentactivationvectorpos
#define YASA_RESETRESOURCEVECTOR(thread)
#define YASA_TEST_TIMER_ARMED(thread)		test_bit(RTL_THREAD_TIMERARMED, &thread->threadflags)
#define YASA_UNSET_TIMER_ARMED(thread)		clear_bit(RTL_THREAD_TIMERARMED, &thread->threadflags)
#define YASA_SIGNAL_TIMER(thread)			rtl_sigaddset (&thread->pending, RTL_SIGNAL_TIMER)
#define YASA_SCHEDULABLE(thread)			( thread->pending & ~thread->blocked )
#define YASA_CHECK_NEXT_ACTIVATION(thread)	(*now >= YASA_NEXTACTIVATION_TIME(thread) )


#define YASA_GETSCHEDULER(thread)			(&(rtl_sched[thread->cpu]))
#define YASA_THREAD_CANCEL(thread)			rtl_printf("Thread cancel is not implemented yet\n")
											//pthread_wait_np(thread) ?
#define YASA_SET_TIMER_ARMED(thread)		(thread)->threadflags|=(1<<RTL_THREAD_TIMERARMED)
#define YASA_CALCPREEMPTIONTIME(sched)		(sched->calcpreemptiontime)
#define YASA_TEST_DEADLINE_MISSED(t,now)	( ( ! ( t->deadlinetolerancemissed) ) && YASA_CURRENT_DEADLINE(t) <= *now )
#define YASA_DEADLINE_ALREADY_MISSED(thread) ( thread->deadlinemissed )
#define YASA_SET_DEADLINE_MISSED(thread)	thread->deadlinemissed=1;
#define YASA_DEADLINE_TOLERANCE_ALREADY_MISSED(thread) ( thread->deadlinetolerancemissed )
#define YASA_SET_DEADLINE_TOLERANCE_MISSED(thread)	thread->deadlinetolerancemissed=1;
#define YASA_SETTICKSCHEDULINGTIME(ptime, now, scheduler)	{	if (scheduler->tick && scheduler->tick!=YASA_MAX_TICK )	\
																	*ptime = *now + scheduler->tick;					\
																else													\
																	*ptime = YASA_MAX_TICK;								}
#define YASA_INITSCHEDULINGLOOP(scheduler, newthread)	if ( scheduler->rtl_current && scheduler->rtl_current!=&scheduler->rtl_linux_task && YASA_SCHEDULABLE(scheduler->rtl_current) ) newthread=YASA_CURRENT_THREAD;	\
													else newthread=0;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// mutex mappings
#define YASA_MUTEX							pthread_mutex_t
#define YASA_MUTEX_PRIORITY_CEILING(mutex)	(mutex->prioceiling)
#define YASA_MUTEX_OLD_PRIORITY(mutex)		(mutex->oldprio)
#define YASA_MUTEX_LOCKED(mutex)			(mutex->busy)
#ifndef YP_SUPPORT_SYNCHRONIZATION_CSP		// rtlinux supports CSP	by default
 #define YP_SUPPORT_SYNCHRONIZATION_CSP			// Ceiling Semaphore Protocol, sometimes called IPCP=Instant Priority Ceiling Protocol
#endif
#define YASA_MUTEX_WAKEUP(mutex)			rtl_wait_wakeup( &(mutex)->wait )





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef CONFIG_RTL_OLD_TIMER_BEHAVIOUR
#error The variable CONFIG_RTL_OLD_TIMER_BEHAVIOUR is set
#error It is used in original rtlinux schedulers
#error Functionality cannot be guaranteed.
#endif

#define RTL_MARK_SUSPENDED(th) 	YASAL_LOG_THREAD_SUSPEND(th, 0, &YASA_CURRENTACTIVATION_TIME(th) ); \
								rtl_sigdelset(&(th)->pending, RTL_SIGNAL_READY)


/*
extern inline int pthread_attr_setrequiredtime_np(pthread_t thread, hrtime_t time) {
	 thread->sched_param.yasa_deadline = time;
	return 0; }
extern inline int pthread_attr_setcomputationtime_np(pthread_t thread, hrtime_t time) {
	 thread->sched_param.yasa_computationtime = time;
	return 0; }
extern inline int pthread_attr_setdeadline_np(pthread_t thread, hrtime_t time)
	 thread->sched_param.yasa_deadline = yasa_deadline;
	return 0; }
extern inline int pthread_attr_setdeadline_behaviour_np()
	 thread->sched_param.yasa_deadline = yasa_deadline;
	return 0; }
extern inline int pthread_attr_setemergencythread_np()
	 thread->sched_param.yasa_deadline = yasa_deadline;
	return 0; }

extern inline int pthread_setrequiredtime_np(pthread_t thread, hrtime_t time)
extern inline int pthread_setcomputationtime_np(pthread_t thread, hrtime_t time)
extern inline int pthread_setdeadline_np(pthread_t thread, hrtime_t time)
extern inline int pthread_setdeadline_behaviour_np()
extern inline int pthread_setemergencythread_np()

extern inline int pthread_getrequiredtime_np(pthread_t thread, hrtime_t *time)
extern inline int pthread_getcomputationtime_np(pthread_t thread, hrtime_t *time)
extern inline int pthread_getdeadline_np(pthread_t thread, hrtime_t *time)
extern inline int pthread_getdeadline_behaviour_np()
extern inline int pthread_getemergencythread_np()
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
union yasa_dyn_priority
{
	YASA_TIME				deadline;
	YASA_TIME				laxity;
	YASA_TIME				requiredtime;
	YASA_TIME				shortesttime;
	int						priority;
};
typedef union yasa_dyn_priority YASA_DYN_PRIORITY;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// additional defintions for structure "pthread_mutex_t"
#define YASA_MUTEX_ATTR_EXT											\
	YASA_ID					id;						/* id & name */	\
	/* YASA_DYN_PRIORITY		dynamicceiling;			 DPCP */		\
    YASA_DYN_PRIORITY		preemptceiling;			/* SRP  */

#define YASA_MUTEX_EXT												\
	YASA_ID					id;						/* id & name */ \
	YASA_THREAD				*thread;								\
	YASA_DYN_PRIORITY		dynamicceiling;			/* DPCP */		\
    YASA_DYN_PRIORITY		preemptceiling;			/* SRP  */


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// additional defintions for structure "pthread_attr_t"
#define YASA_THREAD_ATTR_EXT										\
	YASA_ID					id;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// additional defintions for structure "pthread_attr_t"
#define YASA_THREAD_EXT																						\
	/* static members */																					\
	YASA_ID					id;							/* id & name of rtl_task if set otherwise 0 and "noname" */				\
	YASA_TIME				*activationvectors;			/* start time of asynchronous threads */			\
	YASA_TIME				maxlaxity;					/* laxity in period at time t=0 */					\
																											\
	/* dynamic members */																					\
	YASA_TIME				nextactivation_time;		/* start of THIS period						*/		\
	YASA_TIME				currentactivation_time;		/* start of THIS period						*/		\
	YASA_TIME				yasa_currentdeadline;		/* deadline of current period since bootup */		\
														/* (HRTIME_INFINITY if not set) */					\
	int						yasa_excluded;				/* is this process (excluded 0=no, otherwise yes) */ \
	YASA_TIME				yasa_schedulingdata;		/* can be used by the scheduler, e.g. ELLF, default=0 */ \
	YASA_TIME				yasa_computedtime;			/* computed time since thread start */				\
	YASA_TIME				*nextactivationvectorpos;																\
	YASA_TIME				*currentactivationvectorpos;															\
	/* struct yasa_actionvector *actionvectorpos;		next action time (semaphore)	*/					\
	struct rtl_sched_cpu_struct	*scheduler;																	\
	int						threadtype;					/* type of thread (periodic,nonperiodic...)	*/		\
	int						deadlinemissed;				/* flag, that deadline missed */					\
	int						statepending;				/* flag, that task state is chaning from non periodic to periodic */					\
	int						deadlinetolerancemissed;

//	/* YASA_TIME				*actionvectors;				start time of resource actions */
//	/* int						deadlinebehaviour;			behaviour if deadline missed */
//	/* struct yasa_thread		*emergencythread;			emergency thread */

// map the entries of the origin structure to yasa members

struct yasa_scheduler_attr;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// yasa_scheduler_cpu, extension for rtl_sched_cpu_struct
#define YASA_SCHED_CPU_STRUCT																			\
	/* member functions */																				\
	YASA_LOGFILE	logfile;				/* yasa internal log file */								\
	YASA_TIME		tick;					/* tick value of scheduler */								\
	int				yasa_version;			/* yasa version */											\
	int				cpu;					/* number of cpu */											\
	int				calcpreemptiontime;		/* calculate preemption time */								\
																										\
	/* dynamic members */																				\
	YASA_TIME		schedulerstart;			/* start time of scheduler */								\
	YASA_TIME		schedulerend;			/* end tim time of scheduler(needed by Yasa simulations) */	\
	YASA_TIME		nextschedulingtime;		/* to of next scheduling procedure */						\
	const struct yasa_scheduler_attr	*yasa_schedulerattr;		/* ptr to structure of scheduler */							\
	YASA_THREAD*	(*schedule)(struct rtl_sched_cpu_struct *scheduler, const YASA_TIME *now, YASA_TIME *ptime);	/* scheduling function */		\
	YASA_TIME		yasa_startofthread;		/* start of current thread. When did the current thread get the cpu? */ \
	YASA_TIME		yasa_startofthreadtick;	/* start of current time tick. This can be the same like */	\
	void			*yasa_schedulerdata;	/* additional data, can be used by scheduler, default=0 */



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// logfile support functions
#define YASA_LOCKLOGFILE(logfile)		{											\
											rtl_irqstate_t	flags;					\
											char			state=0;				\
											rtl_no_interrupts (flags);				\
											if ( ! logfile->locked )				\
											{										\
												state=1;							\
												logfile->locked=1;					\
											}										\
											rtl_restore_interrupts (flags);			\
											return state;							\
										}
// usleep does not work within linux kernel
#define YASA_LOCKLOGFILEWAIT(logfile)	{											\
											while (1)								\
											{										\
												if ( yasa_locklogfile( logfile ) )	\
													return 1;						\
												/*usleep(10);*/						\
											}										\
										}
#define YASA_UNLOCKLOGFILE(logfile)		{											\
											rtl_irqstate_t	flags;					\
											rtl_no_interrupts (flags);				\
											logfile->locked=0;						\
											rtl_restore_interrupts (flags);			\
										}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define YASA_PERIODIC_START				pthread_suspend_np( pthread_self() );	\
										do { pthread_wait_np();
#define YASA_PERIODIC_END				} while (1);
#define YASA_NONPERIODIC_START			pthread_suspend_np( pthread_self() );
#define YASA_NONPERIODIC_END(retvalue)	return (void*)retvalue;




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <linux/malloc.h>

#ifdef YASA_TRACE
#undef YASA_TRACE
#endif

#ifdef YASA_DEBUG
#define YASA_TRACE(arg)	rtl_printf(arg)
#else
#define YASA_TRACE(arg)
#endif



#ifdef YASA_ASSERT
#undef YASA_ASSERT
#endif

#ifdef YASA_DEBUG
#define YASA_ASSERT(arg1,arg2)	if (arg1) rtl_printf(arg2);
#else
#define YASA_ASSERT(arg1,arg2)
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif														// ifndef YASA_RTLINUX_INCLUDE
