//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA simulator resource handling (mutexs etc.) declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/01/22
// $Header: /sources/yasa/executives/simulator/sources/simulator_resources.c,v 1.3 2003/01/23 09:18:30 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>
#include <simulator_functions.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ys_wait_sleep (YASA_WAIT_QUEUE *waitqueue) //, spinlock_t *lock)
{
	struct yasa_thread		*self=YASA_CURRENT_THREAD;
	YASA_WAIT_ENTRY			*waitentry;

	waitentry=malloc( sizeof (YASA_WAIT_ENTRY) );

	if ( waitentry )
	{
		// behaviour, if thread is cancelled
		//self->abort 	= &rtl_wait_abort;
		//self->abortdata = wait;

		waitentry->waiter 	= self;
		waitentry->next 	= waitqueue->queue;
		waitqueue->queue	= waitentry;
	}
	else
		PRINTK("Error during ys_wait_sleep()\n");

	YASA_SUSPEND_THREAD(thread);
	ys_do_signal( waitentry->waiter);				// it will be scheduled next time

//	return YASA_RESCHEDULE();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int ys_wait_wakeup (YASA_WAIT_QUEUE *waitqueue)
{
	YASA_WAIT_ENTRY		*waitentry, *tmp;

	for (waitentry = waitqueue->queue; waitentry; )
	{
		YASA_WAKEUP_THREAD(waitentry->waiter);
		ys_do_signal( waitentry->waiter );
		tmp=waitentry->next;
		free(waitentry);
		waitentry=tmp;
	}
	waitqueue->queue = 0;

	// reschedule
	ys_reschedule();
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// will be called during mutex_unlock and alloc_mutexes
void ys_resetmutexstruct( YASA_MUTEX *mutex)
{
	mutex->locked=0;
	mutex->thread=0;
//	mutex->dynceiling=DPCP_MIN_PRIORITY;
//	mutex->dynceiling=YASA_TIME_INFINITY;
	mutex->oldpriority=YASA_MUTEX_DEFAULT_PRIORITY;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output: 0  = no reschedule necessary
// Output: 0 != reschedule necessary
int ys_mutex_lock(YASA_THREAD *thread, int id)
{
	YASA_MUTEX	*mutex=ys_find_mutex(thread->scheduler, id);

	if ( mutex )
	{
		#ifdef YASA_SUPPORT_LOG_MUTEX_RELOCKED
			if ( mutex->thread==thread)
				YASAL_LOG_MUTEX_RELOCKED(thread, &thread->scheduler->currenttime, &id );
		#endif

		if ( yasa_mutex_try_lock(mutex) )
		{
			// error, mutex could not be locked
			ys_wait_sleep( &mutex->waitqueue );
			YASAL_LOG_MUTEX_TRY_N_BLOCKED( thread, &thread->scheduler->currenttime, &id );
			ys_reschedule();	// return 1; reschedule
			return 1;
			//return ys_reschedule( YASA_CURRENT_THREAD );	// return 1; reschedule
		}
		else
		{
			mutex->thread = thread;
			YASAL_LOG_MUTEX_LOCKED( thread, &thread->scheduler->currenttime, &id );
			return 0;
		}
	}
	else
		printf("Mutex not defined!\n");
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// After a mutex unlock all schedulers will be rescheduled due to support
// of synchronization protocols
void ys_mutex_unlock(YASA_THREAD *thread, int id)
{
	YASA_MUTEX	*mutex=ys_find_mutex(thread->scheduler, id);
	if ( mutex )
	{
		if ( mutex->locked )
		{
			yasa_mutex_unlock( mutex );
			YASAL_LOG_MUTEX_UNLOCKED( thread, &thread->scheduler->currenttime, &id );
			ys_resetmutexstruct( mutex );
			ys_wait_wakeup(&mutex->waitqueue);		// incl. reschedule();
		}
	#ifdef YASA_SUPPORT_LOG_MUTEX_NOT_LOCKED
		else
			YASAL_LOG_MUTEX_NOT_LOCKED(thread, &thread->scheduler->currenttime, &id );
	#endif
	}
	else
		printf("Mutex not defined!\n");
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int ys_alloc_mutex( YASA_MUTEX **ptr, YASA_MUTEX_ATTR *mutexattr )
{
	YASA_MUTEX					*current;
	YASA_ERROR					retvalue=0;
	YASA_LOG_ENTRY_MUTEXPARAM	mutexparam;

	*ptr=0;

	while ( mutexattr && !retvalue)
	{
		current=malloc( sizeof (YASA_MUTEX ) );
		current->next=*ptr;
		*ptr=current;
		if ( current )
		{
			current->protocol 		= mutexattr ? mutexattr->protocol : YASASP_NOTHING;
			current->priorityceiling= mutexattr->priorityceiling;
			current->preemptceiling = mutexattr->preemptceiling;
			current->waitqueue.queue= 0;
			ys_resetmutexstruct( current );
			yasa_initdynamicpriority(&current->dynamicceiling);

			if ( mutexattr )
			{
				retvalue=yasa_id_copy(&current->id, &mutexattr->id);
				YASA_MUTEX_PRIORITY_CEILING(current) = mutexattr->priorityceiling;
			}
			else
			{
				retvalue=yasa_id_init(&(current->id) );
				YASA_MUTEX_PRIORITY_CEILING(current) = YASA_MUTEX_DEFAULT_CEILING;
			}
			yasa_id_copy( & (mutexparam.id), &(mutexattr->id) );
			mutexparam.protocol=current->protocol;
			YASAL_LOG_MUTEX_CREATED( YASA_CURRENT_SCHEDULER, 0, &mutexparam );
		}
		else
			retvalue=YASAE_OUT_OF_MEMORY;
		mutexattr=mutexattr->next;
	}

	// initialize global values
	//PCP_SystemCeiling=YASA_MIN_PRIORITY;
	//SRP_StackCount=-1;							// no entry in stack
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ys_free_mutex( YASA_MUTEX **ptr )
{
	YASA_MUTEX	*next;

	next=(*ptr);
	while ( next )
	{
		*ptr=next->next;
		free(next);
		next=*ptr;
	};
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_MUTEX* ys_find_mutex( YASA_SCHEDULER *scheduler, int id )
{
	YASA_MUTEX	*mutex;

	mutex=firstmutex;
	while ( mutex && yasa_id_get_id(&mutex->id) !=id )
		mutex=mutex->next;
	return mutex;
}
