//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA simulator resource handling (semaphores etc.) declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/01/22
// $Header: /sources/yasa/executives/simulator/sources/simulator_scheduling.c,v 1.4 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>
#include <simulator_functions.h>


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a macro to allow an optimized scheduling simulation with long time differences
//! Not fully tested yet. If you have problems, please uncheck it.
//#define USE_OPTIMIZATION		1


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// internal functions
void			ys_do_signal(YASA_THREAD *thread);
void			ys_thread_suspend(YASA_THREAD *thread, YASA_TIME *now);
int INLINE		ys_allocresources(YASA_THREAD *thread);
int INLINE		ys_evaluatecomputing(YASA_SCHEDULER *scheduler);
void INLINE		ys_thread_wait(YASA_THREAD *thread);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int ys_scheduling(YASA_SCHEDULER *schedulers)
{
	int			retvalue=YASAE_NO_ERROR;
	YASA_TIME	time=0;
	//YASA_TIME	curtime;
	//YASA_TIME	nexttime;
	YASA_THREAD	*thread=0;
//	int			reschedule;
#if defined(USE_OPTIMIZATION) && (YP_MAX_CPU==1)
	YASA_TIME	timedif;
	YASA_THREAD	*currentthread;
#endif



	// simulation loop
//	for (curtime=0; curtime<=schedulers[0].schedulerend; curtime+=1)
	while ( schedulers[0].currenttime<=schedulers[0].schedulerend )
	{
		//nexttime=YASA_TIME_INFINITY;
		//next=YP_TICK;
		for (cpu=0; cpu<YP_MAX_CPU; cpu++)
		{
			time=schedulers[cpu].currenttime;
			//tmp.cpu=cpu;
			//schedulers[cpu].currenttime=time=curtime;

/*					if ( time==9000 || time==100000)
					{
						int a;
						a=1;
					}
*/

			// alloc resources, evaluate last computing(deadline matches)
//			ys_evaluatecomputing(&schedulers[cpu]);

/*			if ( time >= schedulers[cpu].nextschedulingtime )
			{
				do
				{
					reschedule=0;
*/
			ys_evaluatecomputing(&schedulers[cpu]);
			while ( time >= schedulers[cpu].nextschedulingtime )
			{

					// +++++++ simulate scheduling +++++++++
					YASAL_LOG_COUNT_SCHEDULER_CALLS(&schedulers[cpu]);
					YASAL_LOG_SCHEDULER_CALLED(&schedulers[cpu], &time, &schedulers[cpu].nextschedulingtime );
					thread=schedulers[cpu].schedule( &schedulers[cpu], &time, &(schedulers[cpu].nextschedulingtime));

#ifdef YP_SCHEDULINGTIME
					schedulers[cpu].currenttime=time+=YP_SCHEDULINGTIME;
					if ( schedulers[cpu].nextschedulingtime!=YASA_TIME_INFINITY )
						schedulers[cpu].nextschedulingtime+=YP_SCHEDULINGTIME;
#endif
					YASAL_LOG_SCHEDULER_FINISHED(&schedulers[cpu], &time, &schedulers[cpu].nextschedulingtime );
					if ( thread!= schedulers[cpu].currentthread )
					{
						YASAL_LOG_SCHEDULER_SWITCH(&schedulers[cpu], &time, 0 );
					}
//					if ( thread!= schedulers[cpu].currentthread || time==YP_SCHEDULINGTIME )		// thread changed ?
					{
						schedulers[cpu].currentthread=thread;
						if ( thread )
						{
							if ( thread->computedtime )
							{
								YASAL_LOG_THREAD_CONT(thread, &time, &thread->currentactivation_time );
							}
							else
							{
								YASAL_LOG_THREAD_START(thread, &time, &thread->currentactivation_time );
							}
							if ( thread->pending & ~(1 << YS_SIGNAL_READY) )		// are there other signals than READY ?
								ys_do_signal(thread);

						}
						#ifdef YASA_SUPPORT_LOG_IDLE_THREAD
						else
							YASAL_LOG_IDLE_THREAD(&schedulers[cpu], &time, 0 );
						#endif
					}
/*					if ( thread!= schedulers[cpu].currentthread || time==YP_SCHEDULINGTIME )		// thread changed ?
					{
						schedulers[cpu].currentthread=thread;
						if ( thread )
						{
							if ( thread->computedtime )
							{
								YASAL_LOG_THREAD_CONT(thread, &time, &thread->currentactivation_time );
							}
							else
							{
								YASAL_LOG_THREAD_START(thread, &time, &thread->currentactivation_time );
							}
							if ( thread->pending & ~(1 << YS_SIGNAL_READY) )		// are there other signals than READY ?
								ys_do_signal(thread);

						}
						#ifdef YASA_SUPPORT_LOG_IDLE_THREAD
						else
							YASAL_LOG_IDLE_THREAD(&schedulers[cpu], &time, 0 );
						#endif
					}
*/
					// alloc resources, evaluate last computing(deadline matches)
					if ( thread )
//						reschedule=ys_evaluatecomputing(&schedulers[cpu]);
						ys_evaluatecomputing(&schedulers[cpu]);

//				} while (reschedule);
			}


			// simulate computing every time tick
#if !defined(USE_OPTIMIZATION) || (YP_MAX_CPU!=1)
			if ( schedulers[cpu].currentthread )
			{
				//reschedule=ys_compute(thread);
				schedulers[cpu].currentthread->computedtime++;
			}
			time=++schedulers[cpu].currenttime;
#else
			// find next time and avoid ++1 to currenttime
			if (    ((currentthread=schedulers[cpu].currentthread) && currentthread->resourcevectorpos)
				 && ( schedulers[cpu].currenttime+currentthread->resourcevectorpos->time-currentthread->computedtime < schedulers[cpu].nextschedulingtime )
				 && ( currentthread->resourcevectorpos->time!=YASA_TIME_INFINITY ) ) //currentthread->computedtime<currentthread->sched_param.requiredtime )
			{
//				++schedulers[cpu].currenttime;
				schedulers[cpu].currenttime+=currentthread->resourcevectorpos->time-schedulers[cpu].currentthread->computedtime;
				currentthread->computedtime+=(currentthread->resourcevectorpos->time-currentthread->computedtime);
			}
			else
			{
				timedif=schedulers[cpu].nextschedulingtime-schedulers[cpu].currenttime;
				if ( timedif > schedulers[cpu].tick )
					timedif=schedulers[cpu].tick;

				//schedulers[cpu].currenttime+=schedulers[cpu].tick;
				schedulers[cpu].currenttime+=timedif;
				if ( currentthread )
				{
					//currentthread->computedtime+=schedulers[cpu].tick;
					currentthread->computedtime+=timedif;
					// is thread finished during tick ?
					if ( currentthread->computedtime > currentthread->sched_param.requiredtime )
					{
						schedulers[cpu].currenttime-=currentthread->computedtime-currentthread->sched_param.requiredtime;
						currentthread->computedtime=currentthread->sched_param.requiredtime;
//						schedulers[cpu].schedule( &schedulers[cpu], &time, &(schedulers[cpu].nextschedulingtime));
					}

				}
				else
				{
					if ( schedulers[cpu].currenttime> schedulers[cpu].nextschedulingtime )
						schedulers[cpu].currenttime = schedulers[cpu].nextschedulingtime;
				}
				if ( schedulers[cpu].currenttime >= schedulers[cpu].schedulerend )
				{
					if ( currentthread )
						currentthread->computedtime-=schedulers[cpu].currenttime-schedulers[cpu].schedulerend;
					schedulers[cpu].currenttime=schedulers[cpu].schedulerend+1;
				}
			}

#endif
		}
	}


	//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	for (cpu=0; cpu<YP_MAX_CPU; cpu++)
	{
		if ( schedulers[cpu].schedulerend+1 == schedulers[cpu].currenttime )
			schedulers[cpu].currenttime--;
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Compute a thread
// Output: TRUE=reschedule
int INLINE ys_allocresources(YASA_THREAD *thread)
{
	int reschedule;

	reschedule=0;
	if ( thread->resourcevectorpos )
	{
		while ( !reschedule && thread->resourcevectorpos->time <= thread->computedtime)
		{
			switch (thread->resourcevectorpos->action)
			{
			case YASAA_MUTEX_LOCK:			reschedule|=ys_mutex_lock(thread, (int)thread->resourcevectorpos->data );
											if ( reschedule )
												return reschedule;			// do not inc resourcevectorpos
											break;
			case YASAA_MUTEX_UNLOCK:		ys_mutex_unlock(thread, (int)thread->resourcevectorpos->data );
											reschedule=1;
											break;
			}
			thread->resourcevectorpos++;
		}
	}
	return reschedule;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output: true=reschedule
int INLINE ys_evaluatecomputing(YASA_SCHEDULER *scheduler)
{
#ifdef YASA_SUPPORT_LOG_TIME_TO_COMPUTE			// param=time to compute at the moment the scheduler detected a deadline miss
//	YASA_TIME		timetocompute=0;
#endif
	int				reschedule=0;

	if ( scheduler->currentthread)
	{

		// handle resources (alloc mutexes etc.)
		reschedule=ys_allocresources( scheduler->currentthread );
		if ( !reschedule )
/*		{
			scheduler->nextschedulingtime=scheduler->currenttime;
		}
		else
*/		{
			// is task finished in last TICK?
			if ( scheduler->currentthread->computedtime >= YASA_REQUIRED_TIME( scheduler->currentthread ) )
			{
				if ( scheduler->currenttime <= scheduler->currentthread->currentdeadline )
				{
					YASAL_LOG_DEADLINE_MATCH(	scheduler->currentthread,
												&scheduler->currenttime,
												&YASA_CURRENT_DEADLINE(scheduler->currentthread) );
				}
				else
				{
					// thread in last tick finished, but deadline missed

					yasa_deadlinemissed( scheduler->currentthread, &scheduler->currenttime );

/*					// task is finished, but deadline is missed
					// deadline behaviour ?
					if ( ! YASA_DEADLINE_ALREADY_MISSED(scheduler->currentthread) )
					{
						YASAL_LOG_DEADLINE_MISS(	scheduler->currentthread,
													&scheduler->currenttime,
													&YASA_CURRENTACTIVATION_TIME(scheduler->currentthread) );
						YASAL_LOG_DEADLINE_MISS_DETECTED(	scheduler->currentthread,
														&YASA_CURRENT_DEADLINE(scheduler->currentthread),
														&scheduler->currenttime );
						YASAL_LOG_TIME_TO_COMPUTE		(	scheduler->currentthread,
														&YASA_CURRENT_DEADLINE(scheduler->currentthread),
														&timetocompute );
					}
*/				}
				ys_reschedule(); // scheduler->currentthread ) ;
				ys_thread_wait( scheduler->currentthread );
				//scheduler->currentthread=0;
				scheduler->nextschedulingtime=scheduler->currenttime;
				reschedule=1;
			}
		}
	}
	return reschedule;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output: 1=The current cpu has to be rescheduled
//int ys_reschedule(YASA_THREAD *thread)
void ys_reschedule()
{
//	thread->scheduler->nextschedulingtime=thread->scheduler->currenttime;
	YASA_CURRENT_SCHEDULER->nextschedulingtime=YASA_CURRENT_SCHEDULER->currenttime;
//	return 1;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ys_do_signal(YASA_THREAD *thread)
{
	// is this thread suspended ?
	if ( thread->pending & (1<<YS_SIGNAL_SUSPEND) )
	{
		// delete READY & SUSPEND Signal
		thread->pending&= ~ (1<<YS_SIGNAL_READY | 1<<YS_SIGNAL_SUSPEND);
/*		if ( thread->resume_time )
		{
			thread->threadflags|=1<<YS_TIMER_ARMED;
			yasa_setcurrentactivationtime(thread);
		}
*/	}

	if ( thread->pending & (1<<YS_SIGNAL_WAKEUP) )
	{
		// delete wake up signal, mark as ready
		thread->pending= ( 1<<YS_SIGNAL_READY) | ( thread->pending & ( ~ (1<<YS_SIGNAL_WAKEUP ) ) );
	}


	// was a new timer signal set ?
	// this is done if a new 'scheduling times' was set
	if ( thread->pending & (1<<YS_SIGNAL_TIMER) )
	{
		// delete TIMER Signal
		thread->pending= (thread->pending & ~(1<<YS_SIGNAL_TIMER)) |
						 (1<<YS_SIGNAL_READY);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// this function will suspend the given thread. The thread
// will no more be scheduled until it will be resumed
void ys_thread_suspend(YASA_THREAD *thread, YASA_TIME *now)
{
	// this task should be suspended now
	YASAL_LOG_THREAD_SUSPEND(thread, now, &thread->currentactivation_time );

	// is this an emergency thread ?
	/*if ( thread->emergencycallingthread )
	{
		thread->resume_time=YASA_TIME_INFINITY;		// block emergency thread
	}*/

	thread->computedtime=0;
	thread->currentdeadline=YASA_DEFAULT_DEADLINE;
	//thread->currentactivation_time=0;
	thread->excluded=0;
	//thread->threadflags&=~(1<<YS_DEADLINE_MISSED);
	thread->deadlinemissed=0;
	thread->deadlinetolerancemissed=0;
	thread->pending|=1<<YS_SIGNAL_SUSPEND;			// suspend thread
	ys_do_signal(thread);							// do it

	// emergency threads should not be started automatically
	if ( thread->emergencycallingthread )
	{
		thread->threadflags&= ~(1<<YS_TIMER_ARMED);	// this thread is not armed
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// this function simulates a wait() function in RT operating systems. It
// will be called, if the thread is finished !
void INLINE ys_thread_wait(YASA_THREAD *thread)
{

	// is thread finished? If yes a new activation time must be set
	//if ( thread->resume_time )
	if ( thread->nextactivation_time )
	{
		thread->threadflags|=1<<YS_TIMER_ARMED;
	}
	ys_thread_suspend(thread, &thread->scheduler->currenttime);
	yasa_setcurrentactivationtime(thread);
	thread->resume_time=thread->currentactivation_time;
	thread->scheduler->currentthread=0;
}
