//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA additional logging definitions
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/include/yasa/yasa_logging.h,v 1.2 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_LOGGING_INCLUDE
#define	YASA_LOGGING_INCLUDE

#include <yasa/yasa.h>



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Logging types
#define YASALT_UNKNOWN						"Unknown              "
#define YASALT_THREAD_CREATED				"Thread_created       "
#define YASALT_THREAD_ACTIVATE				"Thread_activate      "
#define YASALT_THREAD_EXIT					"Thread_exiting       "
#define YASALT_THREAD_START					"Thread_started       "
#define YASALT_THREAD_CONT					"Thread_continues     "
#define YASALT_THREAD_SUSPEND				"Thread_suspended     "
#define YASALT_THREAD_TOLERANCE_START		"Thread_tol_started   "
#define YASALT_THREAD_TOLERANCE_CONT		"Thread_tol_continues "
#define YASALT_THREAD_TOLERANCE_SUSPEND		"Thread_tol_suspended "
#define YASALT_THREAD_SKIPPED				"Thread_skipped       "
#define YASALT_THREAD_TYPE					"Thread_type_changed  "
#define YASALT_THREAD_EXCLUDED				"Thread_excluded      "
#define YASALT_DEADLINE_MATCH				"Deadline_match       "
#define YASALT_DEADLINE_MISS				"Deadline_miss        "
#define YASALT_DEADLINE_MISS_DETECTED		"Miss_detected        "
#define YASALT_DEADLINE_TOLERANCE_MISS		"Tolerence_miss       "
#define YASALT_TIME_TO_COMPUTE				"Time to compute      "
#define YASALT_MUTEX_CREATED				"Mutex_created        "
#define YASALT_MUTEX_DESTROYED				"Mutex_destroyed      "
#define YASALT_MUTEX_LOCKED					"Mutex_locked         "
#define YASALT_MUTEX_TRY_N_BLOCKED			"Mutex_try_n_blocked  "
#define YASALT_MUTEX_UNLOCKED				"Mutex_unlocked       "
#define YASALT_MUTEX_RELOCKED				"Mutex_relocked       "
#define YASALT_MUTEX_NOT_LOCKED				"Mutex_not_locked     "
#define YASALT_MUTEX_PRIORITY_CHANGED		"Mutex_Prio_changed   "
#define YASALT_MUTEX_SYSTEM_CEILING			"Systemceiling        "
#define YASALT_MESSAGE						"Message              "
#define YASALT_EMERGENCY_THREAD				"Emergency_thread     "
#define YASALT_SCHEDULER_CHANGED			"Scheduler_changed    "
#define YASALT_SCHEDULER_CALLED				"Scheduler_called     "
#define YASALT_SCHEDULER_FINISHED			"Scheduler_finished   "
#define YASALT_SCHEDULER_SWITCH				"Thread_switch        "
#define YASALT_SCHEDULER_SYNCHRONIZED_START	"Synchronized start   "
#define YASALT_IDLE_THREAD					"Idle_thread          "
#define YASALT_JITTER						"Jitter               "
#if YASAL_MAX!=35
#error There are inconsistences handling logging types!
#endif




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// task types for log entry YASAL_TASK_TYPE
#define YASALTT_NON_PERIODIC			0
#define YASALTT_PERIODIC				1
#define YASALTT_PENDING_PERIODIC		2	// task type is currently changing but not finshed
											// It will finish, if the task type YASALTT_PERIODIC is announced

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define LOGBUFFERSIZE					255



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct
{
	unsigned char	type;
	unsigned char	length;
	int				threadid;
	YASA_TIME		time;
} YASA_LOG_ENTRY;

typedef struct
{
	YASA_ID			id;
	int				protocol;
} YASA_LOG_ENTRY_MUTEXPARAM;

typedef struct
{
	YASA_ID			id;
} YASA_LOG_ENTRY_THREADPARAM;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// YASA-Logfile enabled
#ifdef YP_ENABLE_LOGGING
 int  yasa_initlogging(YASA_LOGFILE *logfile, size_t size);
 void yasa_stoplogging(YASA_LOGFILE *logfile);
 int  yasa_printlogfile(YASA_LOGFILE *logfile, const YASA_TIME *schedulerstart, int cpu, void* handle);
 void yasa_logscheduler(YASA_SCHEDULER *sched, int id, char type, const YASA_TIME *time, const void *param);
 void yasa_logentry(YASA_THREAD *thread, char type, const YASA_TIME *time, const void *param);
 void yasa_logfileconstructor(YASA_SCHEDULER *sched, YASA_LOGFILE *logfile);
 char yasa_locklogfilewait(YASA_LOGFILE *logfile);
 char yasa_locklogfile(YASA_LOGFILE *logfile);
 void yasa_unlocklogfile(YASA_LOGFILE *logfile);
 void yasa_resetlogging(YASA_LOGFILE *logfile);
#else // YASA-Logfile not enabled
 #define yasa_initlogging(a, b)
 #define yasa_stoplogging(a)
 #define yasa_printlogfile(a, b, c, d)
 #define yasa_logscheduler(a, b, c, d, e)
 #define yasa_logentry(a, b, d, c)
 #define yasa_logfileconstructor(a,b)
 #define yasa_locklogfilewait(a)
 #define yasa_locklogfile(a)
 #define yasa_unlocklogfile(a)
 #define yasa_resetlogging(a)
#endif															// YASA_ENABLE_LOGGING


// define supported/needed logging functions
#ifdef YASA_SUPPORT_LOG_THREAD_CREATED	// a new task was created
 #define YASAL_LOG_THREAD_CREATED(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_YASASUBSYSTEM, YASAL_THREAD_CREATED, time, param)
#else
 #define YASAL_LOG_THREAD_CREATED(a,b,c)
#endif

// define supported/needed logging functions
#ifdef YASA_SUPPORT_LOG_THREAD_ACTIVATE	// when should the task start (make a flash), param=required time?
 #define YASAL_LOG_THREAD_ACTIVATE(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_ACTIVATE, time, param)
#else
 #define YASAL_LOG_THREAD_ACTIVATE(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_EXIT		// param: *retvalue
 #define YASAL_LOG_THREAD_EXIT(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_EXIT, time, param)
#else
 #define YASAL_LOG_THREAD_EXIT(a,b,c)
#endif

/*
#ifdef YASA_SUPPORT_LOG_THREAD_CANCEL	// time: activation time, param: deadline
 #define YASAL_LOG_THREAD_CANCEL(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_CANCEL, time, param)
#else
 #define YASAL_LOG_THREAD_CANCEL(a,b,c)
#endif
*/

#ifdef YASA_SUPPORT_LOG_THREAD_START		// start of computing (make opened bracket)
 #ifdef YASA_SUPPORT_LOG_THREAD_TOLERANCE_START
  #define YASAL_LOG_THREAD_START(thread, time, param)	{																	\
	  													if ( YASA_CURRENT_DEADLINE(thread) > *time )						\
															yasa_logentry(thread, YASAL_THREAD_START, time, param);			\
														else																\
															yasa_logentry(thread, YASAL_THREAD_TOLERANCE_START, time, param);\
														}
 #else
  #define YASAL_LOG_THREAD_START(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_TOLERANCE_START, time, param);
 #endif
#else
 #define YASAL_LOG_THREAD_START(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_CONT		// task is continuing, param: activation time
 #ifdef YASA_SUPPORT_LOG_THREAD_TOLERANCE_START
  #define YASAL_LOG_THREAD_CONT(thread, time, param)	{																	\
	  													if ( YASA_CURRENT_DEADLINE(thread) > *time )						\
															yasa_logentry(thread, YASAL_THREAD_CONT, time, param);			\
														else																\
															yasa_logentry(thread, YASAL_THREAD_TOLERANCE_CONT, time, param);\
														}
 #else
  #define YASAL_LOG_THREAD_CONT(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_CONT, time, param)
 #endif
#else
 #define YASAL_LOG_THREAD_CONT(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_SUSPEND		// task finished (make closed bracket), param: activation time
 #define YASAL_LOG_THREAD_SUSPEND(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_SUSPEND, time, param)
#else
 #define YASAL_LOG_THREAD_SUSPEND(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_SKIPPED		// time: activation time, param: deadline
 #define YASAL_LOG_THREAD_SKIPPED(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_SKIPPED, time, param)
#else
 #define YASAL_LOG_THREAD_SKIPPED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_TYPE
 #define YASAL_LOG_THREAD_TYPE(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_TYPE, time, param)
#else
 #define YASAL_LOG_THREAD_TYPE(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_EXCLUDED
 #define YASAL_LOG_THREAD_EXCLUDED(thread, time, param)	yasa_logentry(thread, YASAL_THREAD_EXCLUDED, time, param)
#else
 #define YASAL_LOG_THREAD_EXCLUDED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_DEADLINE_MATCH		// upward arrow (black), time: now, param: current deadline // param=activation time
 #define YASAL_LOG_DEADLINE_MATCH(thread, time, param)	yasa_logentry(thread, YASAL_DEADLINE_MATCH, time, param)
#else
 #define YASAL_LOG_DEADLINE_MATCH(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_DEADLINE_MISS		// make upward arrow (red), time: now, param: current deadline, param=activation time
 #define YASAL_LOG_DEADLINE_MISS(thread, time, param)	yasa_logentry(thread, YASAL_DEADLINE_MISS, time, param)
#else
 #define YASAL_LOG_DEADLINE_MISS(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_DEADLINE_MISS_DETECTED		// param=time when deadline miss was detected
 #define YASAL_LOG_DEADLINE_MISS_DETECTED(thread, time, param)	yasa_logentry(thread, YASAL_DEADLINE_MISS_DETECTED, time, param)
#else
 #define YASAL_LOG_DEADLINE_MISS_DETECTED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_DEADLINE_TOLERANCE_MISS		// param: activation time
 #define YASAL_LOG_DEADLINE_TOLERANCE_MISS(thread, time, param)	yasa_logentry(thread, YASAL_DEADLINE_TOLERANCE_MISS, time, param)
#else
 #define YASAL_LOG_DEADLINE_TOLERANCE_MISS(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_TIME_TO_COMPUTE				// param=time to compute at the moment the scheduler detected a deadline miss
 #define YASAL_LOG_TIME_TO_COMPUTE(thread, time, param)	yasa_logentry(thread, YASAL_TIME_TO_COMPUTE, time, param)
#else
 #define YASAL_LOG_TIME_TO_COMPUTE(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_CREATED	// mutex was created
 #define YASAL_LOG_MUTEX_CREATED(scheduler, time, param) yasa_logscheduler(scheduler, YASA_YASASUBSYSTEM, YASAL_MUTEX_CREATED, time, param)
#else
 #define YASAL_LOG_MUTEX_CREATED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_DESTROYED	// mutex was destroyed
 #define YASAL_LOG_MUTEX_DESTROYED(thread, time, param)	yasa_logentry(thread, YASAL_MUTEX_DESTROYED, time, param)
#else
 #define YASAL_LOG_MUTEX_DESTROYED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_LOCKED	// mutex was locked successfully
 #define YASAL_LOG_MUTEX_LOCKED(thread, time, param)	yasa_logentry(thread, YASAL_MUTEX_LOCKED, time, param)
#else
 #define YASAL_LOG_MUTEX_LOCKED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_TRY_N_BLOCKED	// mutex wanted to lock a mutex and was blocked due to already locked mutex
 #define YASAL_LOG_MUTEX_TRY_N_BLOCKED(thread, time, param)	yasa_logentry(thread, YASAL_MUTEX_TRY_N_BLOCKED, time, param)
#else
 #define YASAL_LOG_MUTEX_TRY_N_BLOCKED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_UNLOCKED	// mutex was unlocked
 #define YASAL_LOG_MUTEX_UNLOCKED(thread, time, param)	yasa_logentry(thread, YASAL_MUTEX_UNLOCKED, time, param)
#else
 #define YASAL_LOG_MUTEX_UNLOCKED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_RELOCKED	// mutex was relocked
 #define YASAL_LOG_MUTEX_RELOCKED(thread, time, param)	yasa_logentry(thread, YASAL_MUTEX_RELOCKED, time, param)
#else
 #define YASAL_LOG_MUTEX_RELOCKED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_NOT_LOCKED // mutex was not locked
 #define YASAL_LOG_MUTEX_NOT_LOCKED(thread, time, param)	yasa_logentry(thread, YASAL_MUTEX_NOT_LOCKED, time, param)
#else
 #define YASAL_LOG_MUTEX_NOT_LOCKED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_PRIORITY_CHANGED // during mutex lock the priority of the thread is changed
 #define YASAL_LOG_MUTEX_PRIORITY_CHANGED(thread, time, oldprio, newprio)	if ( *oldprio!=*newprio)	\
																				yasa_logentry(thread, YASAL_MUTEX_PRIORITY_CHANGED, time, newprio)
#else
 #define YASAL_LOG_MUTEX_PRIORITY_CHANGED(a,b,c,d)
#endif

#ifdef YASA_SUPPORT_LOG_MUTEX_SYSTEM_CEILING // system ceiling changed
 #define YASAL_LOG_MUTEX_SYSTEM_CEILING(thread, time, oldceil, newceil)	if ( *oldceil!= *newceil) \
																			yasa_logentry(thread, YASAL_MUTEX_SYSTEM_CEILING, time, newceil)
#else
 #define YASAL_LOG_MUTEX_SYSTEM_CEILING(a,b,c,d)
#endif

#ifdef YASA_SUPPORT_LOG_MESSAGE				// make message flag
 #define YASAL_LOG_MESSAGE(thread, time, param)	yasa_logentry(thread, YASAL_MESSAGE, time, param)
#else
 #define YASAL_LOG_MESSAGE(a,b,c)
#endif
#ifdef YASA_SUPPORT_LOG_MESSAGE				// make message flag
 #define YASAL_LOG_SCHEDULER_MESSAGE(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_SCHEDULER_ID, YASAL_MESSAGE, time, param)
#else
 #define YASAL_LOG_SCHEDULER_MESSAGE(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_EMERGENCY_THREAD	// emergency call
 #define YASAL_LOG_EMERGENCY_THREAD(thread, time, param)	yasa_logentry(thread, YASAL_EMERGENCY_THREAD, time, param)
#else
 #define YASAL_LOG_EMERGENCY_THREAD(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_SCHEDULER_CHANGED	// the scheduler was changed
 #define YASAL_LOG_SCHEDULER_CHANGED(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_SCHEDULER_ID, YASAL_SCHEDULER_CHANGED, time, param)
#else
 #define YASAL_LOG_SCHEDULER_CHANGED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_SCHEDULER_CALLED	// the scheduler was called
 #define YASAL_LOG_SCHEDULER_CALLED(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_SCHEDULER_ID, YASAL_SCHEDULER_CALLED, time, param)
#else
 #define YASAL_LOG_SCHEDULER_CALLED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_SCHEDULER_FINISHED	// param=time when the scheduler should be called again
 #define YASAL_LOG_SCHEDULER_FINISHED(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_SCHEDULER_ID, YASAL_SCHEDULER_FINISHED, time, param)
#else
 #define YASAL_LOG_SCHEDULER_FINISHED(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_SCHEDULER_SWITCH	// param=
 #define YASAL_LOG_SCHEDULER_SWITCH(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_SCHEDULER_ID, YASAL_SCHEDULER_SWITCH, time, param)
#else
 #define YASAL_LOG_SCHEDULER_SWITCH(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_SCHEDULER_SYNCHRONIZED_START	// param=
 #define YASAL_LOG_SCHEDULER_SYNCHRONIZED_START(scheduler, id, time, param)	yasa_logscheduler(scheduler, id, YASAL_SCHEDULER_SYNCHRONIZED_START, time, param)
#else
 #define YASAL_LOG_SCHEDULER_SYNCHRONIZED_START(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_IDLE_THREAD
 #define YASAL_LOG_IDLE_THREAD(scheduler, time, param)	yasa_logscheduler(scheduler, YASA_IDLE_ID, YASAL_IDLE_THREAD, time, param)
#else
 #define YASAL_LOG_IDLE_THREAD(a,b,c)
#endif

#ifdef YASA_SUPPORT_LOG_COUNT_SCHEDULER_CALLS		// count scheduler callc
 #define YASAL_LOG_COUNT_SCHEDULER_CALLS(scheduler)	if ( yasa_locklogfile(&(scheduler)->logfile) )		\
													{	(scheduler)->logfile.schedulercounts++;			\
														yasa_unlocklogfile( &(scheduler)->logfile );	\
													}
#else
 #define YASAL_LOG_COUNT_SCHEDULER_CALLS(scheduler)
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif															// ifndef YASA_LOGGING_INCLUDE
