//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA handling of dynamically synchronization protocols (SRP, DPCP etc.)
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/15
// $Header: /sources/yasa/include/yasa/yasa_resourcesdynamic.h,v 1.2 2003/01/23 09:18:31 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASA_RESOURCES_DYNAMIC_INCLUDE
#define	YASA_RESOURCES_DYNAMIC_INCLUDE

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>

// dynamically synchronization protocols need much more program space so they will
// be compiled and linked only if they are needed
#if defined(YP_SUPPORT_SYNCHRONIZATION_DPCP) || defined(YP_SUPPORT_SYNCHRONIZATION_SRP)
 #define YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION				// This projects needs additional schedulings comparisons
#endif

#ifdef YP_SUPPORT_SYNCHRONIZATION_SRP
 #define YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION_SCHEDULING
#endif


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Supported dynamically priorities, must be set by the scheduler or will be set to YASA_SYNC_PRIORITY
#define YASA_SYNC_PRIORITY		0						// default, static priority used, p(0) < (p(3)
#define YASA_SYNC_DEADLINE		1						// earliest deadline first
#define YASA_SYNC_LAXITY		2						// shortest laxity first
#define YASA_SYNC_REQUIREDTIME	3						// highest execution time first
#define YASA_SYNC_SHORTESTTIME	4						// shortest required time first


#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This structure contains information about the dynamically priorities used in
// the schedulers. It is set during startup with the flags YASA_SYNC_PRIORITY
// or during initialization of the schedulers.
// Note: It makes no sense to use different priority types in the schedulers!
// Otherwise errors will occur!
struct yasa_sync_info
{
	int	prioritytype;		// YASA_SYNC_xxx
};
typedef struct yasa_sync_info	YASA_SYNC_INFO;
extern YASA_SYNC_INFO			yasa_syncstruct;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definitions to support Stack Resource Policy (SRP)
#define YASA_SRP_MAXSTACK	YASA_MUTEX_MAXSTACK
typedef struct
{
	YASA_DYN_PRIORITY		preemptceiling;
	YASA_THREAD				*thread;
} YASA_SRP_STACK;

extern YASA_SRP_STACK		SRP_Stack[];
extern int 					SRP_StackCount;






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Definitions to support Dynamic Priority Ceiling Protocol (DPCP)
typedef struct
{
	YASA_DYN_PRIORITY		dynamicceiling;	// current dynamically ceiling priority
	int						stackpos;		// position in YASA_PCP_STACK, until threads must be waked up after mutex unlock
	YASA_THREAD				*thread;		// owner of this ceiling
} YASA_DPCP_SYSTEM_CEILING_STACK;
//#define YASA_DYNPRIORITY(thread)		(thread->currentdeadline)
#define YASA_DPCP_MAXSTACK				YASA_MUTEX_MAXSTACK
#define YASA_DPCP_STACK					YASA_PCP_STACK
extern YASA_DYN_PRIORITY				DPCP_MIN_PRIORITY;
extern YASA_DPCP_STACK					DPCP_Stack[];
extern int 								DPCP_StackCount;
extern YASA_DPCP_SYSTEM_CEILING_STACK	DPCP_SystemCeilingStack[];
extern int 								DPCP_SystemCeilingCount;








//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// function declarations
//#define yasa_initschedulingcycle(newthread)	newthread = SRP_Stack[SRP_StackCount].thread && YASA_SCHEDULABLE(SRP_Stack[SRP_StackCount].thread) ? SRP_Stack[SRP_StackCount].thread : 0
#ifdef __CYGWIN__	// necessary to prevent cygwin linker bug
 YASA_THREAD *cygwin_initschedulingcycle(void);
 #define yasa_initschedulingcycle(newthread)		newthread=cygwin_initschedulingcycle();
#else				// save a call
 #define yasa_initschedulingcycle(newthread)		newthread = SRP_Stack[SRP_StackCount].thread && YASA_SCHEDULABLE(SRP_Stack[SRP_StackCount].thread) ? SRP_Stack[SRP_StackCount].thread : 0
#endif

void	yasa_initsyncstruct(void);
int		yasa_setdynamicprioritytype( int type );
int		yasa_cmpdynamicpriority( void *dprio );
int		yasa_cmpdynamicpriority2( YASA_DYN_PRIORITY *a, YASA_DYN_PRIORITY *b);
int		yasa_equaldynamicpriority( YASA_DYN_PRIORITY *a, YASA_DYN_PRIORITY *b);
void	yasa_logceiling(YASA_DYN_PRIORITY *o, YASA_DYN_PRIORITY *n);
void	yasa_copydynamicpriority( YASA_THREAD *thread, YASA_DYN_PRIORITY *dprio );
int		yasa_dpcp_equaldynamicpriority( YASA_DYN_PRIORITY *a, YASA_DYN_PRIORITY *b);
void	yasa_dpcp_copydynamicpriority( YASA_THREAD *thread, YASA_DYN_PRIORITY *dprio );
int		yasa_dpcp_cmpdynamicpriority( YASA_DYN_PRIORITY *ceiling );
void	yasa_initdynamicpriority(YASA_DYN_PRIORITY *dprio);
//void yasa_dpcp_correctdynamicpriority( YASA_THREAD *srcthread, YASA_DYN_PRIORITY *destdprio, YASA_MUTEX *mutex );




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#else															// ifdef YASA_SUPPORT_DYN_SYNCHRONIZATION

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// if no dynamically synchronization protocols are defined
#define yasa_initdynamicpriority(dprio)
#define yasa_setdynamicprioritytype(type)	0;
#define yasa_initschedulingcycle(newthread)	newthread=0
//#define yasa_initpreemptceiling(a,b)
//void	yasa_initsyncstruct()				{}
void	yasa_initsyncstruct(void);
#define yasa_setdynamicpriority(a)			return 0;

#endif															// ifdef YASA_SUPPORT_DYN_SYNCHRONIZATION
#endif															// ifndef YASA_RESOURCES_DYNAMIC_INCLUDE
