//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA enhanced least laxity first implementation
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/schedulers/ellf/ellf.c,v 1.7 2003/03/28 14:21:43 hilde Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Features: Support for Stack Resource Protocol (SRP)
//
// V1.1			laxity calculation and deadline check changed
//				deadline check improved "<=" instead of "<"
//				YASA_LAXITY(thread) established
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "ellf.h"

#define YASA_LAXITY		YASA_SCHEDULINGDATA

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This scheduler is using the thread member 'schedulerdata'
YASA_THREAD* ellf_schedule(YASA_SCHEDULER *scheduler, const YASA_TIME *now, YASA_TIME *ptime)
{
	YASA_THREAD		*thread;
	YASA_THREAD		*newthread;
	YASA_TIME		newthreadlaxity;
	YASA_TIME		threadlaxity;
	int				reschedule;

	do
	{
		yasa_initschedulingcycle(newthread);
		newthreadlaxity=YASA_TIME_INFINITY;
		reschedule=0;
		for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
		{
			if ( YASA_CHECK_NEXT_ACTIVATION(thread) )
				yasa_setnextactivationtime(thread, now);

			// Should the thread be (re)started in future?
			// thread->threadflags&TIMER_ARMED
			// The thread should be restarted. Did we reach the resume time?
			if ( YASA_TEST_TIMER_ARMED(thread) && *now >= YASA_RESUME_TIME(thread) )
			{
				// timer is no more armed. thread is ready to compute
				// thread->threadflags&=~TIMER_ARMED
				YASA_UNSET_TIMER_ARMED(thread);

				// thread can be choosen in this scheduling !
				// thread->pending will 'TRUE'
				YASA_SIGNAL_TIMER(thread);

				// set next(!) resume time
				yasa_setdeadline(thread);			// set deadline of current thread
			}

			// find highest priority runnable task
			if (  YASA_SCHEDULABLE(thread) )
			{
				// scheduling algorithm
				YASA_LAXITY(thread) = ( YASA_CURRENT_DEADLINE(thread) - *now)
							-( YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread) );
				// if deadline missed, there can be started an emergency thread
				// rescheduling neccessary !!!
//				if ( (  YASA_TEST_DEADLINE_MISSED(thread,now)						) ||
//					 (	YASA_LAXITY(thread) < 0 									) )
//					reschedule|=yasa_deadlinemissed(thread, now);

				/*	 ||
					 ( YASA_LAXITY(thread) == 0 && !(YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread) ) ) )
					reschedule|=yasa_deadlinemissed(thread, now);
				*/

/* old one <V1.1 !
				// if deadline missed, there can be started an emergency thread
				// rescheduling neccessary !!!
				if ( YASA_TEST_DEADLINE_MISSED(thread,now) || YASA_LAXITY(thread) < 0 )
					reschedule|=yasa_deadlinemissed(thread, now);

				// scheduling algorithm
				YASA_LAXITY(thread) = ( YASA_CURRENT_DEADLINE(thread) - *now)
							-( YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread) );
*/				//if ( thread->schedulerdata < newthreadlaxity && ( YASA_COMPUTED_TIME(thread) < YASA_REQUIRED_TIME(thread) ) )
				if ( !newthread || YASA_LAXITY(thread) < newthreadlaxity ||
					 ( YASA_LAXITY(thread) == newthreadlaxity &&
						 YASA_CURRENT_DEADLINE(thread) < YASA_CURRENT_DEADLINE(newthread)
					 ) /*|| //variant that does not preempt current thread in case of laxity *and* deadline tie
					 ( YASA_LAXITY(thread) == newthreadlaxity &&
						 YASA_CURRENT_DEADLINE(thread) == YASA_CURRENT_DEADLINE(newthread) &&
						 	thread == YASA_CURRENT_THREAD
					 )*/
				   )
				{
#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION_SCHEDULING
					if ( (yasa_cmpdynamicpriority( &YASA_MAXLAXITY(thread) ) ) )
#endif
					{
						newthreadlaxity=YASA_LAXITY(thread);
						newthread=thread;
					}
				}
			}
		}
	} while (reschedule);



	// check if there are more than one task with the same smallest laxity
	if ( newthread )
	{
		// check if the chosen thread is excluded
		if ( YASA_EXCLUDED(newthread) && YASA_CURRENT_THREAD &&
									YASA_SCHEDULABLE(YASA_CURRENT_THREAD) )
			newthread=YASA_CURRENT_THREAD;
		else
		{
			for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
			{
				if ( (newthread!=thread) && YASA_SCHEDULABLE(thread) && ( YASA_LAXITY(thread) == YASA_LAXITY(newthread) ) )
				// changed to correct SRP Scheduling
				//if ( (newthread!=thread) && YASA_SCHEDULABLE(thread) && ( YASA_LAXITY(thread) <= YASA_LAXITY(newthread) ) )
				{

#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION_SCHEDULING
					if ( (yasa_cmpdynamicpriority( &YASA_MAXLAXITY(thread) ) ) )
					{
#endif

#ifdef YASA_SUPPORT_LOG_THREAD_EXCLUDED
						if ( ! YASA_EXCLUDED(thread) )
						{
							YASA_EXCLUDED(thread)=1;
							YASAL_LOG_THREAD_EXCLUDED(thread, now, &YASA_EXCLUDED(thread) );
						}
#else
						YASA_EXCLUDED(thread)=1;
#endif
					}
					else
					{
#ifdef YASA_SUPPORT_LOG_THREAD_EXCLUDED
						if ( YASA_EXCLUDED(thread) )
						{
							YASA_EXCLUDED(thread)=0;
							YASAL_LOG_THREAD_EXCLUDED(thread, now, &YASA_EXCLUDED(thread) );
						}
#else
						YASA_EXCLUDED(thread)=0;
#endif
#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION_SCHEDULING
					}
#endif
				}
			}
		}
	}

	/////////////////////////////////////////////////////////////////////////////////////////
	// check wether there are threads with a laxity of 0 that are not the new thread, i.e.
	// that will miss their deadlines

	for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
	{
		if ( YASA_SCHEDULABLE(thread) && (YASA_LAXITY(thread) <= 0) && (newthread != thread) )
		{
#ifdef YASA_SUPPORT_LOG_THREAD_EXCLUDED
			if ( YASA_EXCLUDED(thread) )
			{
				YASA_EXCLUDED(thread)=0;
				YASAL_LOG_THREAD_EXCLUDED(thread, now, &YASA_EXCLUDED(thread) );
			}
#else
			YASA_EXCLUDED(thread)=0;
#endif
		
			yasa_deadlinemissed(thread, now);
		}
		
	}



	/////////////////////////////////////////////////////////////////////////////////////////
	// determine preemption time. Please pay attention to:
	// 1. should the tick or the preemption time be used
	// 2. what happens if newthread=0 ?
	// 3. what happens with non computing threads having deadlines until *ptime ?
	// 4. You cannot be sure to get valid values if the project does not support this scheduler
	//    Example: In priority based schedulers the priority should be analyzed only?
	if ( YASA_CALCPREEMPTIONTIME(scheduler) )
	{
		*ptime=newthreadlaxity=YASA_TIME_INFINITY;
		for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
		{
			if ( YASA_TEST_TIMER_ARMED(thread) )
			{
				if ( *ptime>YASA_RESUME_TIME(thread) )
				{
					// Here could be calculated the laxity of thread and newthread at resume time
					// and could be compared but I think that is will cost more time than a new scheduling!?
					threadlaxity	=( YASA_DEADLINE(thread) - (YASA_RESUME_TIME(thread)-*now) )
									-( YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread) ) ;

					if ( newthread )
						newthreadlaxity	=( YASA_CURRENT_DEADLINE(newthread) - YASA_RESUME_TIME(thread) )
										-( YASA_REQUIRED_TIME(newthread) - (YASA_RESUME_TIME(thread)-*now+YASA_COMPUTED_TIME(newthread) ) );

					if( threadlaxity < newthreadlaxity )
					{
						*ptime=YASA_RESUME_TIME(thread);
					}
				}
			}
		}
	}
	else
		YASA_SETTICKSCHEDULINGTIME(ptime, now, scheduler)
		//*ptime = scheduler->tick;

	return newthread;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int ellf_init(YASA_SCHEDULER *scheduler)
{
	return yasa_setdynamicprioritytype(YASA_SYNC_LAXITY);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// static definitions
const YASA_SCHEDULER_ATTR ellf_attr=
{
	0,						// next
	{ 0, "ys_ellf" },		// id & name
	ELLF_VERSION,			// scheduler version
	0,						// userdata
	ellf_init,				// init()
	0,						// cleanup()
	ellf_schedule,			// schedule()
	0,						// findpreemptor()
};
