//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA least laxity first implementation
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/schedulers/llf/llf.c,v 1.3 2003/01/23 09:18:33 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Features: Support for Stack Resource Protocol (SRP)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "llf.h"


////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_THREAD* llf_schedule(YASA_SCHEDULER *scheduler, const YASA_TIME *now, YASA_TIME *ptime)
{
	YASA_THREAD		*thread;
	YASA_THREAD		*newthread;
	YASA_TIME		newthreadlaxity;
	YASA_TIME		threadlaxity;
	int				reschedule;

	do
	{
		yasa_initschedulingcycle(newthread);
		newthreadlaxity=YASA_TIME_INFINITY;
		reschedule=0;
		for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
		{
			if ( YASA_CHECK_NEXT_ACTIVATION(thread) )
				yasa_setnextactivationtime(thread, now);

			// Should the thread be (re)started in future?
			// thread->threadflags&TIMER_ARMED
			// The thread should be restarted. Did we reach the resume time?
			if ( YASA_TEST_TIMER_ARMED(thread) && *now >= YASA_RESUME_TIME(thread) )
			{
				// timer is no more armed. thread is ready to compute
				// thread->threadflags&=~TIMER_ARMED
				YASA_UNSET_TIMER_ARMED(thread);

				// thread can be choosen in this scheduling !
				// thread->pending will 'TRUE'
				YASA_SIGNAL_TIMER(thread);

				// set next(!) resume time
				yasa_setdeadline(thread);			// set deadline of current thread
			}

			// find highest priority runnable task
			if (  YASA_SCHEDULABLE(thread) )
			{
				// scheduling algorithm
				threadlaxity=( YASA_CURRENT_DEADLINE(thread) - *now)
							-( YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread) );

				// if deadline missed, there can be started an emergency thread
				// rescheduling neccessary !!!
				if ( YASA_TEST_DEADLINE_MISSED(thread,now) || threadlaxity<0 )
					reschedule|=yasa_deadlinemissed(thread, now);

				if ( threadlaxity < newthreadlaxity
					 ||  ( threadlaxity == newthreadlaxity &&
							 YASA_CURRENT_DEADLINE(thread) < YASA_CURRENT_DEADLINE(newthread)
						 )
					)
				{
#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION_SCHEDULING
					if ( yasa_cmpdynamicpriority( &YASA_MAXLAXITY(thread) ) )
#endif
					{
						newthreadlaxity=threadlaxity;
						newthread=thread;
					}
				}
			}
		}
	} while (reschedule);


	/////////////////////////////////////////////////////////////////////////////////////////
	// determine preemption time. Please pay attention to:
	// 1. should the tick or the preemption time be used
	// 2. what happens if newthread=0 ?
	// 3. what happens with non computing threads having deadlines until *ptime ?
	// 4. You cannot be sure to get valid values if the project does not support this scheduler
	//    Example: In priority based schedulers the priority should be analyzed only?
	if ( YASA_CALCPREEMPTIONTIME(scheduler) )
	{
		*ptime=newthreadlaxity=YASA_TIME_INFINITY;
		for (thread = YASA_TASKSET(scheduler); thread; thread = YASA_NEXT_THREAD(thread) )
		{
			if ( YASA_TEST_TIMER_ARMED(thread) )
			{
				if ( *ptime>YASA_RESUME_TIME(thread) )
				{
					// Here could be calculated the laxity of thread and newthread at resume time
					// and could be compared but I think that is will cost more time than a new scheduling!?
					threadlaxity	=( YASA_DEADLINE(thread) - (YASA_RESUME_TIME(thread)-*now) )
									-( YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread) ) ;

					if ( newthread )
						newthreadlaxity	=( YASA_CURRENT_DEADLINE(newthread) - YASA_RESUME_TIME(thread) )
										-( YASA_REQUIRED_TIME(newthread) - (YASA_RESUME_TIME(thread)-*now+YASA_COMPUTED_TIME(newthread) ) );

					if( threadlaxity < newthreadlaxity )
					{
						*ptime=YASA_RESUME_TIME(thread);
					}
				}
			}
		}
	}
	else
		YASA_SETTICKSCHEDULINGTIME(ptime, now, scheduler)
		//*ptime = *now + scheduler->tick;

	return newthread;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int llf_init(YASA_SCHEDULER *scheduler)
{
	return yasa_setdynamicprioritytype(YASA_SYNC_LAXITY);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// static definitions
const YASA_SCHEDULER_ATTR llf_attr=
{
	0,						// next
	{ 0, "ys_llf" },		// id & name
	LLF_VERSION,			// scheduler version
	0,						// userdata
	llf_init,				// init()
	0,						// cleanup()
	llf_schedule,			// schedule()
	0,						// findpreemptor()
};
