//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA handling of dynamically synchronization protocols (SRP, DPCP etc.)
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/15
// $Header: /sources/yasa/yasa/yasa_resourcesdynamic.c,v 1.2 2003/01/23 09:18:33 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>

#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Synchronization structure
YASA_SYNC_INFO					yasa_syncstruct;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// definitions for DPCP
YASA_DYN_PRIORITY				DPCP_MIN_PRIORITY;
YASA_DPCP_STACK					DPCP_Stack[YASA_DPCP_MAXSTACK];
int 							DPCP_StackCount=0;
YASA_DPCP_SYSTEM_CEILING_STACK	DPCP_SystemCeilingStack[YASA_DPCP_MAXSTACK]={ { { YASA_TIME_INFINITY},0,0 } };
int 							DPCP_SystemCeilingCount=0;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// definitions for SRP
YASA_SRP_STACK					SRP_Stack[YASA_SRP_MAXSTACK];
int 							SRP_StackCount=0;



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize the synchronization structure which contains information about the priority type of the scheduler
void yasa_initsyncstruct()
{
	yasa_syncstruct.prioritytype=YASA_SYNC_PRIORITY;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifdef __CYGWIN__
YASA_THREAD* cygwin_initschedulingcycle()
{
	return SRP_Stack[SRP_StackCount].thread && YASA_SCHEDULABLE(SRP_Stack[SRP_StackCount].thread) ? SRP_Stack[SRP_StackCount].thread : 0;
}
#endif

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function will be called by the scheduler during initialization to
// set type of priority. It is not allowed to use different priority
// types in one systems.
// Input: type sync_priority, sync_deadline etc.
// Output: Error or NULL
int yasa_setdynamicprioritytype( int type )
{
	if ( yasa_syncstruct.prioritytype && yasa_syncstruct.prioritytype!=type)
	{
		PRINTK("Error! Only one synchronization type allowed.\n");
		PRINTK("Please do not use different schedulers if you\n");
		PRINTK("want to use a dynamically synchronization\n");
		PRINTK("protocol!\n");
		return -1;
	}
	//------------------------------------ SRP & DPCP ----------------------------------
	yasa_syncstruct.prioritytype=type;
	SRP_StackCount=0;
	SRP_Stack[0].thread=0;
	switch ( type )
	{
	case YASA_SYNC_PRIORITY:	DPCP_MIN_PRIORITY.priority=YASA_MUTEX_DEFAULT_PRIORITY;
								SRP_Stack[0].preemptceiling.priority=YASA_MUTEX_DEFAULT_PRIORITY;
								break;
	case YASA_SYNC_DEADLINE:	DPCP_MIN_PRIORITY.deadline=YASA_TIME_INFINITY;
								SRP_Stack[0].preemptceiling.deadline=YASA_TIME_INFINITY;
								break;
	case YASA_SYNC_LAXITY:		DPCP_MIN_PRIORITY.laxity=YASA_TIME_INFINITY;
								SRP_Stack[0].preemptceiling.laxity=YASA_TIME_INFINITY;
								break;
	case YASA_SYNC_REQUIREDTIME:DPCP_MIN_PRIORITY.requiredtime=0;
								SRP_Stack[0].preemptceiling.requiredtime=0;
								break;
	case YASA_SYNC_SHORTESTTIME:DPCP_MIN_PRIORITY.shortesttime=YASA_TIME_INFINITY;
								SRP_Stack[0].preemptceiling.shortesttime=YASA_TIME_INFINITY;
								break;
	default:					PRINTK("Dynamically priority %d not supported!\n", type);
								return 1;
	}
	//------------------------------------ SRP & DPCP ----------------------------------
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function initializes the mutex preemptceiling (SRP) according to the defined priority type
// will be called during initialization of mutex
// Attention: attr can be 0
void yasa_initdynamicpriority(YASA_DYN_PRIORITY *dprio)
{
	//------------------------------------ SRP & DPCP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
	{
	case YASA_SYNC_PRIORITY:	dprio->priority=YASA_MUTEX_DEFAULT_PRIORITY;
								break;
	case YASA_SYNC_DEADLINE:	dprio->deadline=YASA_TIME_INFINITY;
								break;
	case YASA_SYNC_LAXITY:		dprio->laxity=YASA_TIME_INFINITY;
								break;
	case YASA_SYNC_REQUIREDTIME:dprio->requiredtime=0;
								break;
	case YASA_SYNC_SHORTESTTIME:dprio->shortesttime=YASA_TIME_INFINITY;
								break;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
								break;
	}
	//------------------------------------ SRP & DPCP ----------------------------------
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Compare given "priority" with ceiling of used synchronization protocol
// (will be called from the scheduler)
// Output: True=do not block the thread
//		  False=do block the thread
int yasa_cmpdynamicpriority( void *dprio )
{
	//------------------------------------ SRP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
	{
	case YASA_SYNC_PRIORITY:	if 	( !SRP_StackCount || ( *(int*)dprio > SRP_Stack[SRP_StackCount].preemptceiling.priority ) )
									return 1;
								return 0;
	case YASA_SYNC_DEADLINE:	if 	( !SRP_StackCount || ( *(YASA_TIME*)dprio < SRP_Stack[SRP_StackCount].preemptceiling.deadline ) )
									return 1;
								return 0;
	case YASA_SYNC_LAXITY:		if 	( !SRP_StackCount || ( *(YASA_TIME*)dprio < SRP_Stack[SRP_StackCount].preemptceiling.laxity) )
									return 1;
								return 0;
	case YASA_SYNC_REQUIREDTIME:if 	( !SRP_StackCount || ( *(YASA_TIME*)dprio > SRP_Stack[SRP_StackCount].preemptceiling.requiredtime) )
									return 1;
								return 0;
	case YASA_SYNC_SHORTESTTIME:if 	( !SRP_StackCount || ( *(YASA_TIME*)dprio < SRP_Stack[SRP_StackCount].preemptceiling.shortesttime) )
									return 1;
								return 0;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
	}
	//------------------------------------ SRP ----------------------------------

	return 1;	// unknown type -> ignore
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Output: True=prio(a) < prio(b)
//		  False=do block the thread
int yasa_cmpdynamicpriority2( YASA_DYN_PRIORITY *a, YASA_DYN_PRIORITY *b)
{
	//------------------------------------ SRP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
	{
	case YASA_SYNC_PRIORITY:	if 	( a->priority >= b->priority )
									return 1;
								return 0;
	case YASA_SYNC_DEADLINE:	if 	( a->deadline <= b->deadline )
									return 1;
								return 0;
	case YASA_SYNC_LAXITY:		if 	( a->laxity <= b->laxity )
									return 1;
								return 0;
	case YASA_SYNC_REQUIREDTIME:if 	( a->requiredtime >= b->requiredtime )
									return 1;
								return 0;
	case YASA_SYNC_SHORTESTTIME:if 	( a->shortesttime <= b->shortesttime )
									return 1;
								return 0;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
	}
	//------------------------------------ SRP ----------------------------------
	return 1;	// unknown type -> ignore
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// will be called
// Output: True=prio(a) < prio(b)
//		  False=do block the thread
int yasa_dpcp_equaldynamicpriority( YASA_DYN_PRIORITY *a, YASA_DYN_PRIORITY *b)
{
	//------------------------------------ DPCP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
	{
	case YASA_SYNC_PRIORITY:	if 	( a->priority == b->priority )
									return 1;
								return 0;
	case YASA_SYNC_DEADLINE:	if 	( a->deadline == b->deadline )
									return 1;
								return 0;
	case YASA_SYNC_LAXITY:		if 	( a->laxity == b->laxity )
									return 1;
								return 0;
	case YASA_SYNC_REQUIREDTIME:if 	( a->requiredtime == b->requiredtime )
									return 1;
								return 0;
	case YASA_SYNC_SHORTESTTIME:if 	( a->shortesttime == b->shortesttime )
									return 1;
								return 0;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
	}
	//------------------------------------ DPCP ----------------------------------
	return 1;	// unknown type -> ignore
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Copy
void yasa_dpcp_copydynamicpriority( YASA_THREAD *srcthread, YASA_DYN_PRIORITY *destdprio )
{
	//------------------------------------ DPCP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
	{
	case YASA_SYNC_PRIORITY:	destdprio->priority=YASA_PRIORITY(srcthread);
								break;
	case YASA_SYNC_DEADLINE:	destdprio->deadline=YASA_CURRENT_DEADLINE(srcthread);
								break;
	case YASA_SYNC_LAXITY:		destdprio->laxity=YASA_MAXLAXITY(srcthread);
								break;
	case YASA_SYNC_REQUIREDTIME:destdprio->requiredtime=YASA_REQUIRED_TIME(srcthread);
								break;
	case YASA_SYNC_SHORTESTTIME:destdprio->shortesttime=YASA_REQUIRED_TIME(srcthread);
								break;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
								break;
	}
	//------------------------------------ DPCP ----------------------------------
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Correct dynamic priority. The ceiling priority is the the priority at time t0 of the most priority
// task which is using or will be using the semaphore
/*
void yasa_dpcp_correctdynamicpriority( YASA_THREAD *srcthread, YASA_DYN_PRIORITY *destdprio, YASA_MUTEX *mutex )
{
	//------------------------------------ DPCP ----------------------------------
	YASA_THREAD					*thread;
	struct yasa_resourcevector	*vector;
	for (thread = YASA_TASKSET( YASA_CURRENT_SCHEDULER ); thread; thread = YASA_NEXT_THREAD(thread) )
	{
		// is the task using the current mutex ?
		vector=thread->resourcevectorpos;
		while ( vector && vector->data)
		{
			if ( (vector->action == YASAA_MUTEX_UNLOCK) && (int)vector->data==mutex->id.id )
			{
				switch (yasa_syncstruct.prioritytype)
				{
				case YASA_SYNC_PRIORITY:	if ( destdprio->priority < YASA_PRIORITY(srcthread) )
												destdprio->priority=YASA_PRIORITY(srcthread);
											break;
				case YASA_SYNC_DEADLINE:	if ( destdprio->deadline > YASA_CURRENT_DEADLINE(srcthread) )
											destdprio->deadline = YASA_CURRENT_DEADLINE(srcthread);
											break;
				case YASA_SYNC_LAXITY:		if ( destdprio->laxity > YASA_MAXLAXITY(srcthread) )
												destdprio->laxity=YASA_MAXLAXITY(srcthread);
											break;
				case YASA_SYNC_REQUIREDTIME:if ( destdprio->requiredtime < YASA_REQUIRED_TIME(srcthread) )
												destdprio->requiredtime=YASA_REQUIRED_TIME(srcthread);
											break;
				case YASA_SYNC_SHORTESTTIME:if ( destdprio->shortesttime > YASA_REQUIRED_TIME(srcthread) )
												destdprio->shortesttime=YASA_REQUIRED_TIME(srcthread);
											break;
				default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
											break;
				}
				break;
			}
			vector++;
		}
	}
	//------------------------------------ DPCP ----------------------------------
}
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Compare given "priority" with ceiling of used synchronization protocol
// (will be called from the scheduler)
// Output: True=do not block the thread
//		  False=do block the thread
int yasa_dpcp_cmpdynamicpriority( YASA_DYN_PRIORITY *ceiling )
{
	//------------------------------------ DPCP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
	{
	case YASA_SYNC_PRIORITY:	if 	( YASA_PRIORITY(YASA_CURRENT_THREAD) > ceiling->priority )
									return 1;
								return 0;
	case YASA_SYNC_DEADLINE:	if 	( YASA_CURRENT_DEADLINE(YASA_CURRENT_THREAD) < ceiling->deadline )
									return 1;
								return 0;
	case YASA_SYNC_LAXITY:		if 	( YASA_MAXLAXITY(YASA_CURRENT_THREAD) < ceiling->laxity )
									return 1;
								return 0;
	case YASA_SYNC_REQUIREDTIME:if 	( YASA_REQUIRED_TIME(YASA_CURRENT_THREAD) > ceiling->requiredtime )
									return 1;
								return 0;
	case YASA_SYNC_SHORTESTTIME:if 	( YASA_REQUIRED_TIME(YASA_CURRENT_THREAD) < ceiling->requiredtime )
									return 1;
								return 0;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
	}
	//------------------------------------ DPCP ----------------------------------

	return 1;	// unknown type -> ignore
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yasa_logceiling(YASA_DYN_PRIORITY *o, YASA_DYN_PRIORITY *n)
{
	//------------------------------------ SRP & DPCP ----------------------------------
	switch (yasa_syncstruct.prioritytype)
		{
	case YASA_SYNC_PRIORITY:	YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0,
																&( o->priority ) ,
																&( n->priority ) );
								break;
	case YASA_SYNC_DEADLINE:	YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0,
																&( o->deadline ) ,
																&( n->deadline ) );
								break;
	case YASA_SYNC_LAXITY:		YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0,
																&( o->laxity ) ,
																&( n->laxity ) );
								break;
	case YASA_SYNC_REQUIREDTIME:YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0,
																&( o->requiredtime ) ,
																&( n->requiredtime ) );
								break;
	case YASA_SYNC_SHORTESTTIME:YASAL_LOG_MUTEX_SYSTEM_CEILING( YASA_CURRENT_THREAD, 0,
																&( o->shortesttime ) ,
																&( n->shortesttime ) );
								break;
	default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
								break;
	}
	//------------------------------------ SRP & DPCP ----------------------------------
}


#else


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a dummy function used if no dynamically synchronization protocols used
void	yasa_initsyncstruct(void)
{
}



#endif												// ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION (see top of source)





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void yasa_calculateceilings(YASA_MUTEX_ATTR *mutexattr, YASA_THREAD_ATTR *firstthreadattr)
{
	int					priority;
	YASA_TIME			requiredtime;
	YASA_TIME			shortesttime;
	YASA_TIME			deadline;
	YASA_TIME			laxity;
	YASA_THREAD_ATTR	*threadattr;
	YASA_RESOURCEVECTOR	*vector;
	YASA_MUTEX_ATTR		*firstmutexattr=mutexattr;
	YASA_MUTEX_ATTR		*mutexattr2=mutexattr;

	while ( mutexattr )
	{
		priority=YASA_MUTEX_DEFAULT_PRIORITY;
		deadline=YASA_TIME_INFINITY;
		shortesttime=YASA_TIME_INFINITY;
		requiredtime=0;
		laxity=YASA_TIME_INFINITY;
		for (threadattr = firstthreadattr; threadattr; threadattr = threadattr->next )
		{
			vector=threadattr->resourcevectors;
			if ( vector )
			{
				while ( vector->data )
				{
					if ( vector->action==YASAA_MUTEX_LOCK && (int)(vector->data)== yasa_id_get_id(&mutexattr->id) )
					{
#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION
						switch ( yasa_syncstruct.prioritytype )
						{
						// find smallest deadline
						case YASA_SYNC_PRIORITY:	// will be done always (see below)
													break;
						case YASA_SYNC_DEADLINE:	if ( threadattr->deadline < deadline )
														deadline = threadattr->deadline;
													break;
						case YASA_SYNC_LAXITY:		if ( threadattr->deadline-threadattr->requiredtime < laxity )
														laxity = threadattr->deadline-threadattr->requiredtime;
													break;
						case YASA_SYNC_REQUIREDTIME:if ( threadattr->requiredtime > requiredtime )
														requiredtime = threadattr->requiredtime;
													break;
						case YASA_SYNC_SHORTESTTIME:if ( threadattr->requiredtime < shortesttime )
														requiredtime = threadattr->requiredtime;
													break;
						default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
													break;
						}
#endif
						// find highest priority
						if ( threadattr->priority > priority )
							priority=threadattr->priority;
					}
					vector++;
				};
			}
		}

		// fill ceilings in mutex
		mutexattr2=firstmutexattr;
		while ( mutexattr2 && ( yasa_id_get_id(&mutexattr2->id) != yasa_id_get_id(&mutexattr->id ) ) )
			mutexattr2=mutexattr2->next;
		if ( mutexattr2 )
		{
#ifdef YASA_SUPPORT_DYNAMICALLY_SYNCHRONIZATION
					// set ceiling only if it is set to zero!
			switch ( yasa_syncstruct.prioritytype )
			{
				case YASA_SYNC_PRIORITY:	if ( ! mutexattr2->preemptceiling.priority )
												mutexattr2->preemptceiling.priority=priority;
											break;
				case YASA_SYNC_DEADLINE:	if ( ! mutexattr2->preemptceiling.deadline )
												mutexattr2->preemptceiling.deadline=deadline;
											break;
				case YASA_SYNC_LAXITY:		if ( ! mutexattr2->preemptceiling.laxity )
												mutexattr2->preemptceiling.laxity=laxity;
											break;
				case YASA_SYNC_REQUIREDTIME:if ( ! mutexattr2->preemptceiling.requiredtime )
												mutexattr2->preemptceiling.requiredtime=requiredtime;
											break;
				case YASA_SYNC_SHORTESTTIME:if ( ! mutexattr2->preemptceiling.shortesttime )
												mutexattr2->preemptceiling.shortesttime=shortesttime;
											break;
				default:					PRINTK("Dynamically priority %d not supported!\n", yasa_syncstruct.prioritytype);
											break;
			}
#endif
			if ( ! mutexattr2->priorityceiling )
				mutexattr2->priorityceiling=priority;
		}
		else
			PRINTK("Error during parsing mutex attr!\n");
		mutexattr=mutexattr->next;
	}
}
