//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA scheduling definitions
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2001/11/26
// $Header: /sources/yasa/yasa/yasa_scheduling.c,v 1.2 2003/01/23 09:18:33 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasa/yasa.h>


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// will be called once per period/activation during scheduling
INLINE void yasa_setdeadline( YASA_THREAD *thread )
{
	if ( YASA_DEADLINE(thread) )
	{
		switch ( YASA_DEADLINE(thread) )
		{
			default:
					YASA_CURRENT_DEADLINE(thread) = YASA_CURRENTACTIVATION_TIME(thread) + YASA_DEADLINE(thread);
					//rtl_printf("default deadline: %9ld%09ld\n", YASA_DEADLINE(thread)/1000000000, YASA_DEADLINE(thread)%1000000000 );
					break;
			case YASA_TIME_INFINITY:
					YASA_CURRENT_DEADLINE(thread) = YASA_DEFAULT_DEADLINE;
					//rtl_printf("infinity deadline\n");
					break;
		}
	}
	else
		YASA_CURRENT_DEADLINE(thread) = YASA_DEFAULT_DEADLINE;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////
// will be called if a thread is gone in suspended mode to prepare next start usually wait()
// activation time is the time which represents the start time (activation) of the current thread !
// If you use DL_SKIPOVER it is the start time of all non skipped threads
////////////////////////////////////////////////////////////////////////////////////////////////////////////
INLINE void yasa_setcurrentactivationtime( YASA_THREAD *thread )
{
	YASA_TIME	timetmp;

	YASA_RESETRESOURCEVECTOR(thread);
	switch ( YASA_THREADTYPE(thread) )
	{
	case YASA_THREAD_PERIODIC:
		switch ( YASA_DL_BEHAVIOUR(thread) )
		{
		case YASA_DL_CANCEL_THREAD:
		case YASA_DL_SKIPOVER:		YASA_CURRENTACTIVATION_TIME(thread) += YASA_PERIOD(thread);
									while ( YASA_CURRENTACTIVATION_TIME(thread) < YASA_NEXTACTIVATION_TIME(thread) )
									{
										timetmp=YASA_CURRENTACTIVATION_TIME(thread) + YASA_DEADLINE(thread);
										// This thread is skipped. It cannot miss its deadline !
										//YASAL_LOG_DEADLINE_MISS(thread, &timetmp, &(YASA_CURRENTACTIVATION_TIME(thread)) );
										YASAL_LOG_THREAD_SKIPPED(thread, &YASA_CURRENTACTIVATION_TIME(thread), &timetmp);
										YASA_CURRENTACTIVATION_TIME(thread) += YASA_PERIOD(thread);
									};
									break;

		case YASA_DL_EMERGENCYTHREAD:
									//PRINTK("EMCall not implemented yet!\n");

		case YASA_DL_IGNORE:		YASA_CURRENTACTIVATION_TIME(thread) += YASA_PERIOD(thread);
									break;
		}
		break;


	case YASA_THREAD_ASYNCHRONOUS:
		if ( YASA_CURRENTACTIVATIONVECTORPOS(thread) )
		{
			switch ( YASA_DL_BEHAVIOUR(thread) )
			{
			case YASA_DL_CANCEL_THREAD:
			case YASA_DL_SKIPOVER:		YASA_CURRENTACTIVATION_TIME(thread) = *thread->currentactivationvectorpos;
										if ( YASA_CURRENTACTIVATION_TIME(thread) != YASA_TIME_INFINITY)
										{
											thread->currentactivationvectorpos++;
											while ( YASA_CURRENTACTIVATION_TIME(thread) < YASA_NEXTACTIVATION_TIME(thread))
											{
												timetmp=YASA_CURRENTACTIVATION_TIME(thread) + YASA_DEADLINE(thread);
												// This thread is skipped. It cannot miss its deadline !
												//YASAL_LOG_DEADLINE_MISSED(thread, &timetmp, &(YASA_CURRENTACTIVATION_TIME(thread)) );
												YASAL_LOG_THREAD_SKIPPED(thread, &YASA_CURRENTACTIVATION_TIME(thread), &timetmp);
												YASA_CURRENTACTIVATION_TIME(thread) = *thread->currentactivationvectorpos;
												if ( YASA_CURRENTACTIVATION_TIME(thread) != YASA_TIME_INFINITY)
												{
													thread->currentactivationvectorpos++;
												}
											};
										}
										break;
			case YASA_DL_EMERGENCYTHREAD:
										//PRINTK("EMCall not implemented yet!\n");

			case YASA_DL_IGNORE:		YASA_CURRENTACTIVATION_TIME(thread) = *thread->currentactivationvectorpos;
										if ( YASA_CURRENTACTIVATION_TIME(thread) != YASA_TIME_INFINITY)
										{
											thread->currentactivationvectorpos++;
										}
										break;
			}
			break;
		} // threads with no action vector have the same activation time like
		  // next activation time (e.g. Emergency threads)

	case YASA_THREAD_NONPERIODIC:
	default:						YASA_CURRENTACTIVATION_TIME(thread)=YASA_NEXTACTIVATION_TIME(thread);
									break;
	}
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function will be called everytime a thread should be startet even if it cannot !
INLINE void yasa_setnextactivationtime( YASA_THREAD *thread, const YASA_TIME *now)
{
	YASAL_LOG_THREAD_ACTIVATE(thread, now, &YASA_NEXTACTIVATION_TIME(thread) );
	switch ( YASA_THREADTYPE(thread) )
	{
	case YASA_THREAD_PERIODIC:	//YASAL_LOG_THREAD_ACTIVATE(thread, now, &YASA_NEXTACTIVATION_TIME(thread) );
								YASA_NEXTACTIVATION_TIME(thread) += YASA_PERIOD(thread);
								break;

	case YASA_THREAD_ASYNCHRONOUS:
								//YASAL_LOG_THREAD_ACTIVATE(thread, now, &YASA_NEXTACTIVATION_TIME(thread) );
								if ( thread->nextactivationvectorpos )
								{
									YASA_NEXTACTIVATION_TIME(thread) = *thread->nextactivationvectorpos;
									if ( YASA_NEXTACTIVATION_TIME(thread) != YASA_TIME_INFINITY)
									{
										thread->nextactivationvectorpos++;
									}
									break;
								} // threads without a resume vector have resume time infinity !!!

	case YASA_THREAD_NONPERIODIC:
	default:					YASA_NEXTACTIVATION_TIME(thread) = YASA_TIME_INFINITY;
								break;
	}
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////
// During scheduling it was determined that the deadline of the specified thread was missed.
// What should be done now?
// 1.) We sign a deadline miss (LOGFILE)
// 2.) do action defined in YASA_DL_BEHAVIOUR
INLINE int yasa_deadlinemissed(YASA_THREAD *thread, const YASA_TIME *now)
{
#ifdef YASA_SUPPORT_LOG_TIME_TO_COMPUTE			// param=time to compute at the moment the scheduler detected a deadline miss
	YASA_TIME		timetocompute;
#endif
	int		reschedule;

	reschedule=0;

/*	if ( *now==39000)
	{
		int a;
		a=2;
	}
*/
	if ( ! YASA_DEADLINE_ALREADY_MISSED(thread) )
	{
//		YASAL_LOG_DEADLINE_MISS_DETECTED( thread, &YASA_CURRENT_DEADLINE(thread), now );
		YASAL_LOG_DEADLINE_MISS_DETECTED( thread, now, &YASA_CURRENT_DEADLINE(thread) );
#ifdef YASA_SUPPORT_LOG_TIME_TO_COMPUTE
		timetocompute=YASA_REQUIRED_TIME(thread) - YASA_COMPUTED_TIME(thread);
//		YASAL_LOG_TIME_TO_COMPUTE( thread, &YASA_CURRENT_DEADLINE(thread), &timetocompute );
		YASAL_LOG_TIME_TO_COMPUTE( thread, now, &timetocompute );
#endif
		YASAL_LOG_DEADLINE_MISS( thread, &YASA_CURRENT_DEADLINE(thread), &(YASA_CURRENTACTIVATION_TIME(thread)) );
		YASA_SET_DEADLINE_MISSED(thread);
	}


		if ( !YASA_DEADLINE_TOLERANCE_ALREADY_MISSED(thread) )
		{
			if ( YASA_DEADLINE_TOLERANCE(thread) )
			{
				if ( *now < YASA_CURRENT_DEADLINE(thread) + YASA_DEADLINE_TOLERANCE(thread) )
					return 0;					// tolerance not reached, no reschedule
#ifdef YASA_SUPPORT_LOG_DEADLINE_TOLERANCE_MISS
				else
					YASAL_LOG_DEADLINE_TOLERANCE_MISS( thread, now, &YASA_CURRENTACTIVATION_TIME(thread) );
#endif
			}
			YASA_SET_DEADLINE_TOLERANCE_MISSED(thread);

			switch ( YASA_DL_BEHAVIOUR(thread) )
			{
				case YASA_DL_EMERGENCYTHREAD:if ( YASA_EMERGENCYTHREAD(thread) )
											{
												YASA_NEXTACTIVATION_TIME( YASA_EMERGENCYTHREAD(thread) )=*now;
												yasa_setcurrentactivationtime( YASA_EMERGENCYTHREAD(thread) );
//												YASAL_LOG_EMERGENCY_THREAD( YASA_EMERGENCYTHREAD(thread), &YASA_CURRENTACTIVATION_TIME( YASA_EMERGENCYTHREAD(thread) ), &thread->id.id );
												YASAL_LOG_EMERGENCY_THREAD( YASA_EMERGENCYTHREAD(thread), &YASA_CURRENTACTIVATION_TIME( YASA_EMERGENCYTHREAD(thread) ), (void*)yasa_id_get_idptr(&thread->id) );
												YASA_RESUME_TIME(YASA_EMERGENCYTHREAD(thread)) = YASA_CURRENTACTIVATION_TIME(YASA_EMERGENCYTHREAD(thread));
												YASA_SET_TIMER_ARMED( YASA_EMERGENCYTHREAD(thread) );
											//	YASA_EMERGENCYTHREAD(thread)->threadflags|=(1<<YS_TIMER_ARMED);
												#ifdef YASA_SUPPORT_LOG_TASK_EXCLUDED
												if ( YASA_THREAD_EXCLUDED(thread) )
													YASAL_LOG_TASK_EXCLUDED(thread, now, 0 );
												#endif
												YASA_THREAD_CANCEL(thread);
											//ys_thread_wait(thread);
												reschedule=1;
	/*											thread->emergencythread->activation_time=*now;
												YASAL_LOG_THREAD_ACTIVATE( thread->emergencythread, &thread->emergencythread->activation_time, &YASA_REQUIRED_TIME(thread) );
												thread->emergencythread->pending|=(1<<YS_SIGNAL_TIMER);
												yasa_setdeadline(thread->emergencythread);
												ys_thread_wait(thread);
												reschedule=1;
	*/										}
											break;
		/*		case YASA_DL_TOLERANCE:		if ( now < YASA_CURRENT_DEADLINE(thread) + YASA_DEADLINE_TOLERANCE(thread) )
											{
												break;
											}
											YASAL_LOG_TOLERANCE_MISS( thread, &now, &YASA_CURRENTACTIVATION_TIME(thread) );
		*/
				case YASA_DL_CANCEL_THREAD:	YASA_THREAD_CANCEL(thread);
											//ys_thread_wait(thread);
											reschedule=1;
											break;

				default:
				case YASA_DL_SKIPOVER:
				case YASA_DL_IGNORE:		break;
			}
	}
	return reschedule;
}
