//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA application object action declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/actions.h,v 1.5 2003/02/26 14:13:24 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about the actions occured by user interactions and useful in
// conjunction with yasa objects
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_ACTIONS_INCLUDE
#define	YASAGUI_ACTIONS_INCLUDE

#include <qaction.h>
#include <qpopupmenu.h>
#include "yasagui/yasagui.h"
#include "list.h"
#include "filedialog.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class QListViewItem;
class YActionHandler;
class YFrame;
class YEnvironment;
class YMDIWindow;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YActions : public QObject
{	Q_OBJECT
	enum ACTION_TYPE	{	ACTIVATE=0,
							MAX_ACTIONS };
	// Commands are useful for Toolbars and Menus where the context is changing but nothing else
	QArray<QAction*>	setactions;
protected:
	YActionHandler		&actionhandler;
	virtual int			_CreateMenu(YObject *object, QPopupMenu *menu) const;
	virtual int			_CreateTreeItems(YObject *object, QListViewItem *parent, QListViewItem *after=0) const
						{	return 0;	object=0; parent=0; after=0;	}
	virtual int			_Activate(YObject *object);
public:					YActions(YActionHandler &ah);
	enum COMMAND_TYPE	{	ZOOM_IN,
							ZOOM_OUT,
							EXECUTE,
						};
	virtual int			InitInstance();
	virtual void		UpdateActions();
	virtual int			Activate(YObject *object);
	//! A key was pressed and the widget was associated with the object
	//! If return value=true -> key event used
	virtual int			DoWindowCommand(YObject *, YActions::COMMAND_TYPE, YMDIWindow *){	return YASAE_NOT_IMPLEMENTED;	}
	virtual int			DoTreeCommand(YObject *, YActions::COMMAND_TYPE)				{	return YASAE_NOT_IMPLEMENTED;	}
	static int			CreateMenu(YObject *object, QPopupMenu *parent);
	static int			CreateTreeItems(YObject *object, QListViewItem *parent, QListViewItem *after=0);
	virtual void		NewObjectCreated(YObject *object)				{ Q_UNUSED(object);	}
	virtual void		ObjectRemoved(YObject *object)					{ Q_UNUSED(object);	}
private slots:
	int					_activate();
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Attention: Only classes containing workspace/project definitions are allowed to
//! derive from this class
class YSetActions : public YActions
{	Q_OBJECT
	enum{ ADD=0, REMOVE_ALL, MAX_ACTIONS };
	QArray<QAction*>	setactions;
protected:
	virtual int			_CreateMenu(YObject *object, QPopupMenu *menu) const;
public:					YSetActions(YActionHandler &ah) : YActions(ah), setactions(MAX_ACTIONS)			{}
	virtual int			InitInstance();
	virtual void		UpdateActions();
	virtual YObject*	Add(YObject *set);
	virtual int			RemoveAll(YObject *set) const;
	virtual void		NewObjectCreated(YObject *object);
	virtual void		ObjectRemoved(YObject *object);
public slots:
	YObject*			AddEntry();
	int					RemoveAllEntries();
signals:
	void				NewItem(YObject *object);		// There was an item created. Not not neccessary a project item !
	void				ItemRemoved(YObject *object);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YEntryActions : public YActions
{	Q_OBJECT
	enum{ RENAME=0, REMOVE, MAX_ACTIONS };
	QArray<QAction*>	entryactions;
protected:
	virtual int			_CreateMenu(YObject *object, QPopupMenu *parent) const;
public:					YEntryActions(YActionHandler &ah) : YActions(ah), entryactions(MAX_ACTIONS)		{}
	virtual int			InitInstance();
	virtual void		UpdateActions();
	virtual int			Remove(YObject *entry) const;
	virtual int			Rename(YObject *entry) const;
public slots:
	int					RenameEntry();
	int					RemoveEntry();
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProjectSetActions : public YSetActions
{	Q_OBJECT
	enum{ CLEANUP=0, MAX_ACTIONS };
	QArray<QAction*>	projectsetactions;
public:					YProjectSetActions(YActionHandler &ah) : YSetActions(ah), projectsetactions(MAX_ACTIONS) {}
	virtual int			InitInstance();
protected:
	virtual int			_Activate(YObject *object);
	virtual int			_CreateMenu(YObject *object, QPopupMenu *parent) const;
public slots:
	void				CleanupProjects();
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProjectActions : public YEntryActions
{	Q_OBJECT
	enum{ CLEANUP=0, MAX_ACTIONS };
	QArray<QAction*>	projectactions;
public:					YProjectActions(YActionHandler &ah) : YEntryActions(ah), projectactions(MAX_ACTIONS)	{}
	virtual int			Rename(YObject *entry) const;
	virtual int			Remove(YObject *entry) const;
	virtual int			InitInstance();
protected:
	virtual int			_Activate(YObject *object);
	virtual int			_CreateMenu(YObject *object, QPopupMenu *parent) const;
public slots:
	void				CleanupProject();
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a dummy class to prevent error messages during runtime
class YResultsActions : public YEntryActions
{	Q_OBJECT
public:					YResultsActions(YActionHandler &ah) : YEntryActions(ah)							{}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTimingResultActions : public YEntryActions
{	Q_OBJECT
public:					YTimingResultActions(YActionHandler &ah) : YEntryActions(ah)					{}
	virtual int			DoWindowCommand(YObject *object, YActions::COMMAND_TYPE command, YMDIWindow *window);
protected:
	virtual int			_Activate(YObject *object);
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStatisticResultActions : public YEntryActions
{	Q_OBJECT
public:					YStatisticResultActions(YActionHandler &ah) : YEntryActions(ah)					{}
protected:
	virtual int			_Activate(YObject *object);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTaskSetActions : public YSetActions
{	Q_OBJECT
public:					YTaskSetActions(YActionHandler &ah) : YSetActions(ah)
						{}
protected:
	virtual int			_Activate(YObject *object);
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTaskActions : public YEntryActions
{	Q_OBJECT
public:					YTaskActions(YActionHandler &ah) : YEntryActions(ah)
						{}
protected:
	virtual int			_Activate(YObject *object);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YMutexSetActions : public YSetActions
{	Q_OBJECT
public:					YMutexSetActions(YActionHandler &ah) : YSetActions(ah)
						{}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YMutexActions : public YEntryActions
{	Q_OBJECT
public:					YMutexActions(YActionHandler &ah) : YEntryActions(ah)
						{}
protected:
	virtual int			_Activate(YObject *object);
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YEnvironmentSetActions : public YSetActions
{	Q_OBJECT
public:					YEnvironmentSetActions(YActionHandler &ah) : YSetActions(ah)
						{}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YEnvironmentActions : public YEntryActions
{	Q_OBJECT
	enum{ EXECUTE=0, TIMINGS, STATISTICS, LOGFILE, CLEANUP,  MAX_ACTIONS };
	QArray<QAction*>	eactions;
public:					YEnvironmentActions(YActionHandler &ah) : YEntryActions(ah), eactions(MAX_ACTIONS)
						{}
	int					InitInstance();
	int					_CreateMenu(YObject *object, QPopupMenu *menu) const;
	void				UpdateActions();
	virtual int			Rename(YObject *entry) const;
	virtual int			Remove(YObject *object) const;
	virtual int			ShowLogFile(YEnvironment *object);
	virtual int			ShowTimingResults(YEnvironment *object);
	virtual int			ShowStatisticResults(YEnvironment *e);
	virtual int			DoTreeCommand(YObject *treeobject, YActions::COMMAND_TYPE command);
protected:
	virtual int			_Activate(YObject *object);
public slots:
	void				ExecuteEnvironment();
	void				CleanupEnvironment();
	void				ShowTimingResults();
	void				ShowStatisticResults();
	void				ShowLogFile();
};
//////////////////////	////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerSetActions : public YSetActions
{	Q_OBJECT
public:					YSchedulerSetActions(YActionHandler &ah) : YSetActions(ah)
						{}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerActions : public YEntryActions
{	Q_OBJECT
public:					YSchedulerActions(YActionHandler &ah) : YEntryActions(ah)
						{}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFilePathActions : public YEntryActions
{	Q_OBJECT
	QFont				font;										//! font for all file windows
public:					YFilePathActions(YActionHandler &ah) : YEntryActions(ah),
						font("Courier", 10)
						{
							font.setFixedPitch(true);
						}
	QFont&				GetFont()									{ return font; }
protected:
	virtual int			_Activate(YObject *object);
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFilePathSetActions : public YSetActions
{	Q_OBJECT
	YFileDialog			dialog;
public:					YFilePathSetActions(YActionHandler &ah, const QString &addcaption, const QString &postfix) :
						YSetActions(ah),
						dialog(".", addcaption, postfix, QFileDialog::ExistingFiles )
						{}
	virtual YObject*	Add(YObject *set);
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSourceSetActions : public YFilePathSetActions
{	Q_OBJECT
public:					YSourceSetActions(YActionHandler &ah) :
						YFilePathSetActions(ah, MSGP_ADD_SOURCE_FILE, SOURCE_FILE_FILTER)
						{}
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YHeaderSetActions : public YFilePathSetActions
{	Q_OBJECT
public:					YHeaderSetActions(YActionHandler &ah) :
						YFilePathSetActions(ah, MSGP_ADD_HEADER_FILE, HEADER_FILE_FILTER)
						{}
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_ACTIONS_INCLUDE
