//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA datatypes definitions
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/datatypes.cpp,v 1.3 2003/01/23 09:18:34 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "datatypes.h"




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function justifies the given string to a 64Bit value
//! Example: tmp= 2   -> 0000000000000002
//! Example: tmp= 20  -> 0000000000000020
//! Example: tmp= 2.0 -> 0000000000000002.0
QString YString::Justify(QString tmp, int width, QChar fillchar)
{
	int	pointindex=tmp.find(NUMBER_SEPARATOR);
	if ( pointindex==-1 )
		pointindex=tmp.length();
	return tmp.rightJustify(width+( tmp.length()-pointindex), fillchar);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Return the time as string ( like snprintf("%ldd",time) ). Only strings in English are allowed due to using
//! this strings in configs. For localization issues use YASA_TIME::GetVisibleContentString()
//\return error code
QString YTIME::GetContentString() const						// YASA_SHORT_INFINITY_TIME=-1
{
	if ( *this==YASA_TIME_INFINITY)
		return QString::number( YASA_SHORT_INFINITY_TIME );
	return QString::number( (long) GetContent() );
}
QString YTIME::GetContentString(YASA_TIME time)				// YASA_SHORT_INFINITY_TIME=-1
{
	if ( time==YASA_TIME_INFINITY)
		return QString::number( YASA_SHORT_INFINITY_TIME );
	return QString::number( (long) time );
}
QString YTIME::GetContentStringMax(YASA_TIME time)			// YASA_TIME_INFINITY is returned as YASA_TIME_INT_MAX=0x7fffffff
{
	if ( time==YASA_TIME_INFINITY)
		return QString::number( YASA_TIME_INT_MAX );
	return QString::number( (long) time );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Return the time as string ( like snprintf("%ldd",time) ). Only strings in English are allowed due to using
//! this strings in configs. For localization issues use YASA_TIME::GetVisibleContentString()
//\return error code
QString YTIME::GetSourceString(YASA_TIME time)
{
	if ( time==YASA_TIME_INFINITY)
		return "YASA_TIME_INFINITY";
	return QString::number( (unsigned long) time );
	//return QString::number( (unsigned long) time );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_TIME YTIME::ReadYasaTimeFromString(const QString &str, bool *ok)
{
	unsigned int	index=0;
	int			count=0;
	QChar		a;
	YASA_TIME	value=0;
	bool		negative=false;

	*ok=false;
	while ( str.constref(index) == ' ' )
		index++;
		//str.remove(0,1);
	while ( str.length()-index )
	{
		a=str.constref(index);
		if ( a=='-' )
			negative=true;
		else
		{
			if ( a<'0' || a>'9' || count>=20 )
				break;
			value=value*10+(a-0x30);
			count++;
		}
		//str.remove(0,1);
		index++;
//		test=str.latin1();
//		*ok=sscanf( test, "%ld", &lvalue);
//		*ok=sscanf( test, "%lld", &value);
//		*ok=sscanf(str.latin1(), "%lld", &value);
	};
	if ( ( !(str.length()-index) || a==' ' || a=='\t' || a=='\n' || a==',' || a==(char)13) && count<=20 && str.length() )
	{
		*ok=true;
		if (negative)
			value=-value;
	}

	return value;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! returns the time as visible string with explanations like "infinity"
QString YTIME::GetVisibleContentString() const
{
	if ( *this==YASA_TIME_INFINITY)
		return MSGP_YASA_TIME_INFINITY;
	//return QString::number( (long) GetContent() );
	return QString("%1 %2").arg(number( (long) GetContent() )).arg(MSGP_YASA_TIME_UNIT);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! returns the time as visible string with explanations like "infinity"
QString YTIME::GetVisibleContentString(YASA_TIME time)
{
	if ( time==YASA_TIME_INFINITY)
		return MSGP_YASA_TIME_INFINITY;
	//return QString::number( (long) time );
	return QString("%1 %2").arg(number( (long) time )).arg(MSGP_YASA_TIME_UNIT);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//\return error code
int YTIME::ReadFromString(const QString &content)
{
	unsigned int	count=0;
	INT64			b=0;
	QChar			a;
	char			c;
	unsigned int	negative=0;

	a=content[0];
	if ( a == '-' )
		negative=1;
	for ( count=negative; count<(20+negative) && content.length() > count ; count++ )
	{
		a=content[(int)count];
		if (a<'0' || a>'9')
			return YASAE_ERROR_IN_ARGUMENTS;
		c=char(a);
		if ( negative)
			b=b*10-(c-'0');
		else
			b=b*10+(c-'0');
	}
	if ( count==(20+negative) && content.length() > count )
		return YASAE_ERROR_IN_ARGUMENTS;

	SetContent(b);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YASA_TICK::SetMinTimerTick(YASA_TIME t)
{
	YASA_TIME	newt=t;

	if ( GetDerivedParent() )
	{
		// is min. tick of environment > scheduler tick ? -> increase min scheduler tick
		YASA_TICK	*tick=((YASA_TICK*)GetDerivedParent());
		if ( tick->min_tick > t )
			newt=tick->min_tick;
	}
	if ( min_tick != newt )
	{
		min_tick=newt;
		DerivedParentContentChanged();	// check current content again, maybe limits have changed
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is used to derive the content from parents
//! If one of the parent objects does not support timer tick there is no support for any child possible
bool YASA_TICK::CheckLimits(YASA_TIME &dest, const YASA_TIME &source) const
{
	if ( source < min_tick )
	{
		dest=min_tick;
		return true;
	}
	dest=source;
	return true;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_PRIORITY::ReadFromString(const QString &content)
{
	bool	ok;
	int		p;
	p=content.toInt(&ok);
	if ( ok )
	{
		SetContent(p);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_TASKTYPE::ReadFromString(const QString &content)
{
	bool	ok;
	int		t;
	t=content.toInt(&ok);
	if ( ok )
	{
		SetContent(t);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_SCHEDULERTYPE::ReadFromString(const QString &content)
{
	bool	ok;
	int		t;
	t=content.toInt(&ok);
	if ( ok )
	{
		SetContent(t);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_SERVERTYPE::ReadFromString(const QString &content)
{
	bool	ok;
	int		t;
	t=content.toInt(&ok);
	if ( ok )
	{
		SetContent(t);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_DLBEHAVIOUR::ReadFromString(const QString &content)
{
	bool	ok;
	int		b;
	b=content.toInt(&ok);
	if ( ok )
	{
		SetContent(b);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_CPU::ReadFromString(const QString &content)
{
	bool	ok;
	int		c;
	c=content.toInt(&ok);
	if ( ok )
	{
		SetContent(c);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Checks if the parent objects have set lower maximum cpus. If yes corrent the given value.
//! If the parameter is
bool YASA_CPU::CheckLimits(int &dest, const int &source) const
{
	if ( GetDerivedParent()  && (GetDerivedParent()->GetContent() < source ) )
	{
		dest=GetDerivedParent()->GetContent();
		return true;
	}
	if ( source < YG_MIN_CPU )
	{	qWarning("YASA_CPU::SetCPU() called with illegal arguments");
		dest=YG_MIN_CPU;
		return true;
	}
	if ( source > YG_MAX_CPU )
	{	qWarning("YASA_CPU::SetCPU() called with illegal arguments");
		dest=YG_MAX_CPU;
		return true;
	}
	dest=source;
	return true;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Checks if the parent objects have set lower maximum cpus. If yes corrent the given value.
//! If the parameter is
bool YASA_MAXCPU::CheckLimits(int &dest, const int &source) const
{
	if ( source == YASA_FLEXIBLE_MAX_CPU )
	{
		dest=YG_MAX_CPU;
		return true;
	}
	if ( source < YG_MIN_NUMBER_OF_CPUS )
	{	qWarning("YASA_MAXCPU::SetCPU() called with illegal arguments");
		dest=YG_MIN_NUMBER_OF_CPUS;
		return true;
	}
	if ( source > YG_MAX_CPU )
	{	qWarning("YASA_MAXCPU::SetCPU() called with illegal arguments");
		dest=YG_MAX_CPU;
		return true;
	}
	dest=source;
	return true;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_MUTEXPROTOCOL::ReadFromString(const QString &content)
{
	bool	ok;
	int		p;
	p=content.toInt(&ok);
	if ( ok )
	{
		SetContent(p);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_RESOURCEACTION::ReadFromString(const QString &content)
{
	bool	ok;
	int		p;
	p=content.toInt(&ok);
	if ( ok )
	{
		SetContent(p);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_CALCPREEMPTIONTIME::ReadFromString(const QString &content)
{
	bool	ok;
	int		p;
	p=content.toInt(&ok);
	if ( ok )
	{
		SetContent(p);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is used to derive the content from parents
//! If one of the parent objects does not support calcpreemptiontime there is no support for any child possible
bool YASA_CALCPREEMPTIONTIME::CheckLimits(bool &dest, const bool &source) const
{
/*	if ( GetDerivedParent()  && (GetDerivedParent()->GetContent() == false ) )
	{
		dest=GetDerivedParent()->GetContent();
		return true;
	}
*/	dest=source;
	return true;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// OUTPUT: error code
int YASA_TIMERTICK::ReadFromString(const QString &content)
{
	bool	ok;
	int		p;
	p=content.toInt(&ok);
	if ( ok )
	{
		SetContent(p);
		return 0;
	}
	return YASAE_ERROR_IN_ARGUMENTS;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is used to derive the content from parents
//! If one of the parent objects does not support timer tick there is no support for any child possible
bool YASA_TIMERTICK::CheckLimits(bool &dest, const bool &source) const
{
	if ( GetDerivedParent()  && (GetDerivedParent()->GetContent() == false ) )
	{
		dest=GetDerivedParent()->GetContent();
		return true;
	}
	dest=source;
	return true;
}
