//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project, environment support
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/environment.cpp,v 1.3 2003/01/23 09:18:34 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "environment.h"
#include "parser.h"
#include "plugins.h"
#include "sourcegenerator.h"
#include "application.h"
#include "output.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YEnvironmentSet::YEnvironmentSet() :
YSet(YPC_ENVIRONMENT, MSGP_DEFAULT_ENVIRONMENTNAME, YObjectArgs( MSGP_CAPTION_ENVIRONMENTS, YActionHandler::GetActions(YActionHandler::ENVIRONMENTSET_ACTIONS) ) )
{
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YEnvironment::YEnvironment(const YObjectArgs &args=YObjectArgs() ) :
YObject(args),
output( YApplication::GetOutput() ),
executive(0),
results( YResultArgs( YResultArgs::ENVIRONMENT_RESULTS) ),
calcpreemptiontime( &GetProject()->GetCalcPreemptionTime() ),
support_timertick( (bool)SUPPORT_TIMERTICK),							// default=false
timertick( &( GetProject()->GetTimerTick() ) ),
maxcpu( &GetProject()->GetMaxCPU() ),
logfilepath( YObjectArgs("Logfile", YActionHandler::GetActions(YActionHandler::FILEPATH_ACTIONS) ) ),
logfilename( YASA_LOGFILE_NAME )
{
//	connect( &maxcpu	, SIGNAL( UpdateObject() ), this, SLOT( ProjectMaxCPUChanged() ) );
	connect( &maxcpu	, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
//	connect( &calcpreemptiontime, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	connect( &timertick	, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
	schedulerset.SetParent(this);
	logfile.SetParent(this);
	results.SetParent(this);
	schedulerset.InitInstance();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read environment specific configuration data from stream and set to instance
int YEnvironment::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_EXECUTIVENAME) )
	{
		QString	name;
		int retvalue = parser.ReadContentString( name );
		if ( ! retvalue )
		{
			if ( SetExecutive(name) == 0)
					retvalue=YASAE_ERROR_IN_ARGUMENTS;
			schedulerset.ClearSet();							// clear automatically created cpu items
				return retvalue;
		}
	}
	if ( parser.CheckSectionStart(YPC_GCC_COMPILER_OPTIONS) )
		return parser.ReadContentObject( gcccompileroptions );
	if ( parser.CheckSectionStart(YPC_LOGFILE_NAME) )
		return parser.ReadContentObject( logfilename );
	if ( parser.CheckSectionStart(YPC_SCHEDULING_TIME) )
		return parser.ReadContentObject( schedulingtime );
	if ( parser.CheckSectionStart(YPC_SYNCHRONIZATION_OFFSET) )
		return parser.ReadContentObject( synchronizationoffset );
//	if ( parser.CheckSectionStart(YPC_EXECUTIVE) )
//		return parser.ReadContentObject(executivename);
/*	if ( parser.CheckSectionStart(YPC_CALC_PREEMPTIONTIME) )
		return parser.ReadContentState(calcpreemptiontime);
	if ( parser.CheckSectionStart(YPC_TIMER_TICK) )
		return parser.ReadContentObject(timertick);
*/	if ( parser.CheckSectionStart(YPC_SCHEDULER) )
		return parser.NewSectionSet(YPC_SCHEDULER, schedulerset );
/*	if ( key == YPC_NUMBER_OF_CPUS)
	{
		int number,retvalue;
		retvalue=ReadInt(number);
		if ( retvalue )
			return retvalue;
		return GetSchedulerSet().AllocEntries(number);
	}
*/
	return YObject::SetConfig(parser);
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read environment specific configuration data from instance and set to stream
int YEnvironment::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	if ( executive )
	{
		retvalue=parser.WriteContentString(YPC_EXECUTIVENAME, executive->GetName() );
			if ( retvalue )	return retvalue;
	}
	retvalue=parser.WriteContentObject(YPC_GCC_COMPILER_OPTIONS, gcccompileroptions);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_LOGFILE_NAME, logfilename);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_SCHEDULING_TIME, schedulingtime);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_SYNCHRONIZATION_OFFSET, synchronizationoffset );
		if ( retvalue )	return retvalue;
/*	retvalue=parser.WriteContentState(YPC_CALC_PREEMPTIONTIME, calcpreemptiontime);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_TIMER_TICK, timertick);
		if ( retvalue )	return retvalue;
*/	retvalue=parser.NewSectionSet(YPC_SCHEDULER, schedulerset );
		if ( retvalue )	return retvalue;
	return YObject::GetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The config files contain only the executive names. That is why we have to resolve the names here
//! to the plugins
//\return Ptr to executive plugin or NULL
/*
YExecutivePlugIn* YEnvironment::GetExecutive()
{
	if ( executive )
		return executive;
	YExecutivePlugIns	&executiveset=YExecutivePlugIns::GetExecutivePlugIns();
	for (unsigned i=0; i<executiveset.GetCount(); i++)
	{
		if ( executivename == executiveset.At(i)->GetName() )
		{
			executive=(YExecutivePlugIn*)executiveset.At(i);
			executivename=executive->GetName();
			maxcpu=executive->GetMaxCPU().GetContent();
			schedulerset.EnsureItemCount( GetMaxCPU().GetContent() );
			calcpreemptiontime.SetChildsDisabled( executive->GetSupportCalcPreemptionTime() );
			timertick.SetChildsDisabled( executive->GetSupportTimerTick() );

			//executives are read only
			//connect( executive, SIGNAL(UpdateObject(YObject *)), this, SLOT( ExecutiveUpdated(YObject *) ) );
			return executive;
		}
	}
	return 0;			// Object not found -> executive undefined
}
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A new executive name was set
YExecutivePlugIn* YEnvironment::SetExecutive(const QString &name)
{
	YExecutivePlugIns	&executiveset=YExecutivePlugIns::GetExecutivePlugIns();
	for (unsigned i=0; i<executiveset.GetCount(); i++)
	{
		if ( name == executiveset.At(i)->GetName() )
		{
			SetExecutive( (YExecutivePlugIn*)executiveset.At(i) );
			return (YExecutivePlugIn*) executiveset.At(i);
		}
	}
	executive=0;
	return 0;			// Object not found -> executive undefined
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironment::SetExecutive(YExecutivePlugIn *e)
{
	if ( e!=executive )
	{
		executive=e;
		logfilepath.SetPath( QString::null );
		if ( e )
		{
//			if ( executive->GetMaxCPU().IsValid() && executive->GetMaxCPU()!=YASA_MAXCPU::YASA_MAXCPU_FLEXIBLE )						// is maxcpu >0 ?
//			if ( executive->GetMaxCPU().IsValid() && ! executive->GetMaxCPU().isFlexible() )
			if ( executive->GetMaxCPU().IsValid() && ! executive->GetUnlimitedCPUSupport()  ||
				executive->GetMaxCPU() <= GetProject()->GetMaxCPU() )
				maxcpu=executive->GetMaxCPU().GetContent();
			else
				maxcpu.Invalid();		// will be derived from parent (project)


			schedulerset.EnsureItemCount( GetMaxCPU().GetContent() );

			// if this executives does not support cpt we set content=false and disable all childs
			if ( executive->GetSupportCalcPreemptionTime() )
				calcpreemptiontime.Invalid();							// derive settings from project
			else
				calcpreemptiontime=false;								// no support for cpt
			calcpreemptiontime.SetChildsDisabled( ! executive->GetSupportCalcPreemptionTime() );

			support_timertick = executive->GetSupportTimerTick();
			support_timertick.SetChildsDisabled( ! executive->GetSupportTimerTick() );
			//support_timertick=executive->GetSupportTimerTick();
			timertick.SetMinTimerTick( executive->GetMinTimerTick() );
			gcccompileroptions.SetParent( & executive->GetGCCCompilerOptions() );
			gcccompileroptions.Invalid();
			//if ( executive->GetSchedulingTimeSupport() )
			//	schedulingtime.SetContent( YTIME::YASA_TIME_MIN );
			//CorrectLogfilePath();
		}
		else
		{
			//logfilepath.SetPath( QString::null );
			schedulingtime.SetContent( YTIME::YASA_TIME_MIN );
			gcccompileroptions=QString::null;
			support_timertick=false;
			calcpreemptiontime.Invalid();
			maxcpu.Invalid();
			schedulerset.EnsureItemCount( GetMaxCPU().GetContent() );
			timertick.SetMinTimerTick( YTIME::YASA_TIME_MIN );
		}
		ObjectUpdated();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function returns the work directory of the environment
QString YEnvironment::GetEnvironmentDirectory(bool abspath) const
{
	QString	dest;
	dest=GetProject()->CopyProjectDir(abspath);
	YFileOperations::AddSubDirectory( dest, YFileOperations::CorrectFileName( GetName() ) );
	YFileOperations::CreateDirectory( dest );
	return dest;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironment::CorrectLogfilePath()
{
	QString		path;

	path=GetEnvironmentDirectory(true);
	YFileOperations::AddSubDirectory(path,  GetLogFileName() );
	logfilepath.SetPath( path );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YEnvironment::ExecuteEnvironment()
{
	int					retvalue=YASAE_ERROR_IN_ARGUMENTS;
	YSourceGenerator	generator( this, output );
	QString				command;
	QString				yasadir;
	QString				workdir;
	//QString				path;

	output->Message( );
	output->Message( MSGP_E_EXECUTING_PROJECT.arg( GetProject()->GetName() ) );
	CorrectLogfilePath();
	retvalue=generator.PrepareWorkProject();
	if ( ! retvalue )
	{
		output->Message( MSGP_E_STARTING_ENVIRONMENT.arg(GetName() ) );
		retvalue=YASAE_COULD_NOT_OPEN_FILE;
		yasadir=QDir().absPath();
		workdir=GetEnvironmentDirectory(true);
		if ( QDir::setCurrent( workdir ) )
		{
			retvalue=YFileOperations::RunBashScript(SOURCE_RUNIT_NAME);
			command=MSGP_E_ENVIRONMENT_FINISHED.arg( GetName() );
			output->Message( command.arg(retvalue) );
			QDir::setCurrent( yasadir );						// set YASA directory
			if ( ! retvalue )
			{
				CleanupLogFile();
				retvalue=EvaluateLogFile();
			}
			emit EnvironmentExecuted();							// the file "logfile" has changed
		}
	}
	return retvalue;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YEnvironment::CleanupEnvironment(QString absexecutivedir)
{
	QString				currentdir;

	emit EnvironmentCleanedUp();
	CleanupLogFile();
	output->Message( QString(MSGP_E_CLEANING_ENVIRONMENT.arg(GetName() )).arg(GetProject()->GetName() ) );
	if ( ! absexecutivedir.length() )
		absexecutivedir=GetEnvironmentDirectory(true);
	currentdir=QDir().absPath();
	if ( QDir::setCurrent( absexecutivedir ) )
	{
		output->Message( MSGP_E_CHANGING_DIRECTORY.arg(absexecutivedir) );
		YFileOperations::RunBashScript( SOURCE_CLEANUP_NAME );
		QDir::setCurrent( currentdir );							// set to YASA directory
	}
	output->Message( MSGP_E_REMOVING_DIRECTORY.arg( absexecutivedir ) );
	YFileOperations::RemoveDirectory( absexecutivedir );
}


