//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA environment window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/03/18
// $Header: /sources/yasa/yasagui/environmentwindow.h,v 1.4 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about environment windows
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_ENVIRONMENTWINDOW_INCLUDE
#define	YASAGUI_ENVIRONMENTWINDOW_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "mainwindow.h"
#include "widgets.h"
#include "task.h"
#include "environment.h"
#include "plugins.h"

class YSchedulerPlugIns;
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the environment configuration
class YEnvironmentFrame : public YObjectFrame
{	Q_OBJECT
	QScrollView			scrollview;
public:					YEnvironmentFrame(QWidget *parent, YFrameArgs &args) :
						YObjectFrame(args),
						scrollview(parent)
						{}
	virtual int			InitInstance();
	virtual QWidget*	GetMainWidget()							{ return &scrollview;	}
	static YFrame*		CreateEnvironmentFrame(QWidget *parent, YFrameArgs &args)
						{
							return new YEnvironmentFrame(parent, args);
						}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YExecutiveStringList : public YStringList
{	Q_OBJECT
public:					YExecutiveStringList(YExecutivePlugIns *set) : YStringList(set)	{ }
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YExecutiveWidget : public YTextComboWidget
{	Q_OBJECT
	YExecutiveStringList executivelist;
public:					YExecutiveWidget(YEnvironment *environment, QWidget *parent) :
						YTextComboWidget(environment, parent),
						executivelist( & YExecutivePlugIns::GetExecutivePlugIns() )
						{	//connect( &YExecutivePlugIns::GetExecutivePlugIns(), SIGNAL(UpdateObject(YObject *)), this, SLOT( UpdateExecutiveList(YObject *) ) );
							//connect( YExecutivePlugIns::GetExecutivePlugIns(), SIGNAL(NewItem(YObject *)), this, SLOT(UpdateExecutiveList(YObject *) ) );
							//connect( YExecutivePlugIns::GetExecutivePlugIns(), SIGNAL(ItemRemoved(YObject *)), this, SLOT(UpdateExecutiveList(YObject *) ) );
							executivelist.AddItem(0, MSGP_ENVIRONMENT_EXECUTIVE_UNDEFINED, false);
							connect( &executivelist, SIGNAL(StringListChanged()), this, SLOT(ExecutiveListChanged() ) );
							ExecutiveListChanged();
						}
public slots:
	virtual void		UpdateWidget();
	//! item in combo box of executives changed
	virtual void		UpdateValue(const QString &e);
	// Stringlist of executives was changed
	void				ExecutiveListChanged()
						{	clear();
							insertStringList(executivelist);
							setCurrentItem( executivelist.FindIndex( ((YEnvironment*)object)->GetExecutive() ) );
						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YGCCCompilerOptionsWidget : public YStringWidget
{	Q_OBJECT
public:					YGCCCompilerOptionsWidget(YEnvironment *e, QWidget *parent) :
						YStringWidget(e, parent)	{	UpdateWidget(); }
public slots:
	virtual void		UpdateWidget()
						{
							setEnabled( ((YEnvironment*)object)->GetExecutive() );
							if ( ((YEnvironment*)object)->GetGCCCompilerOptions() != text() )
								setText( ((YEnvironment*)object)->GetGCCCompilerOptions().GetContent() );
						}
	virtual void		UpdateValue( const QString &t)
						{
							if ( ((YEnvironment*)object)->GetGCCCompilerOptions() != t )
								((YEnvironment*)object)->SetGCCCompilerOptions( text() );
						}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YExecutiveFeatureListWidget : public YTextWidget
{	Q_OBJECT
	YExecutivePlugIn	*executive;
public:					YExecutiveFeatureListWidget(YEnvironment *e, QWidget *parent) : YTextWidget(e,parent),
						executive(0)
						{	setReadOnly(true);
							setText(MSGP_ENVIRONMENT_EXECUTIVE_UNDEFINED);
						}
public slots:
	virtual void		UpdateWidget()
						{
							YExecutivePlugIn	*newexecutive=((YEnvironment*)object)->GetExecutive();
							setEnabled( newexecutive);
							if ( executive != newexecutive )
							{
								executive = newexecutive;
								if ( newexecutive )
								{
									clear();
									QStringList	list=executive->GetFeatureList();
									for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
									{
										insertLine( (*it) );
									}
									if ( executive->GetDescription().length() )
									{
										insertLine("\n");
										insertLine(MSGP_EXECUTIVE_DESCRIPTION);
										insertLine(executive->GetDescription());
									}
 								}
								else
								{
									setText(MSGP_ENVIRONMENT_EXECUTIVE_UNDEFINED);
								}
							}
						}

	virtual void		UpdateValue()							{ }
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulingTimeWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YSchedulingTimeWidget(YEnvironment *e, QWidget *parent) : YYasaTimeWidget(e, parent)
						{	UpdateWidget();
							updateDisplay();	}
public slots:
	virtual void		UpdateWidget()
						{
							setEnabled( ((YEnvironment*)object)->GetExecutive() && ((YEnvironment*)object)->GetExecutive()->GetSchedulingTimeSupport() );
							if ( GetYasaTime() != ((YEnvironment*)object)->GetSchedulingTime() )
								SetYasaTime( ((YEnvironment*)object)->GetSchedulingTime());
						}
	virtual void		UpdateValue(YASA_TIME value) { ((YEnvironment*)object)->SetSchedulingTime( value );			}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v==YTIME::YASA_TIME_MIN)	return QString( MSGP_ENVIRONMENT_SCHEDULING_TIME_UNSET );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_ENVIRONMENT_SCHEDULING_TIME_UNSET )	return YTIME::YASA_TIME_MIN;
																	return text().toInt(ok);						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSynchronizationOffsetWidget : public YYasaTimeWidget
{	Q_OBJECT
public:					YSynchronizationOffsetWidget(YEnvironment *e, QWidget *parent) : YYasaTimeWidget(e, parent)
						{	UpdateWidget();
							updateDisplay();	}
public slots:
	virtual void		UpdateWidget()
						{
							setEnabled( ((YEnvironment*)object)->GetExecutive() && ((YEnvironment*)object)->GetExecutive()->GetSynchronizationOffsetSupport() );
							if ( GetYasaTime() != ((YEnvironment*)object)->GetSynchronizationOffset() )
								SetYasaTime( ((YEnvironment*)object)->GetSynchronizationOffset() );
						}
	virtual void		UpdateValue(YASA_TIME value) { ((YEnvironment*)object)->SetSynchronizationOffset(value );			}
protected:
	virtual QString		mapValueToText(int v)					{	if ( v==YTIME::YASA_TIME_MIN)	return QString( MSGP_ENVIRONMENT_SYNCHRONIZATION_OFFSET_UNSET );
																	return QString::number(v);						}
	virtual int			mapTextToValue(bool *ok)				{	if ( text()==MSGP_ENVIRONMENT_SYNCHRONIZATION_OFFSET_UNSET )	return YTIME::YASA_TIME_MIN;
																	return text().toInt(ok);						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! object is a type of YScheduler
class YSchedulerListViewItem : public YListViewItem
{	Q_OBJECT
public:					YSchedulerListViewItem(YObject *o, QListView *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{
							connect( ((YEnvironment*)o->GetSet()->GetParent() ), SIGNAL( UpdateObject() ) ,
									 this, SLOT ( EnvironmentChanged() ) );
							UpdateObject(o);					// fill all columns
						}
						YSchedulerListViewItem(YObject *o, QListViewItem *parent, QListViewItem *after=0) :
						YListViewItem( o, parent, after)
						{
							connect( ((YEnvironment*)o->GetSet()->GetParent() ), SIGNAL( UpdateObject() ) ,
									 this, SLOT ( EnvironmentChanged() ) );
							UpdateObject(o);					// fill all columns
						}
protected:QString		key(int column, bool ascending) const
						{
							switch (column)
							{
							// "CPU 10"
							case 0:		{
											int	index=GetObject()->GetSet()->FindItemIndex( GetObject() );
											return YString::Justify( QString::number(index) );
										}
							// column 3 timertick
							case 3:		return YString::Justify( ((YScheduler*)object)->GetTimerTick().GetVisibleContentString() );
							}
							return text(column);
							Q_UNUSED(ascending);
						}
public slots:
	void				UpdateObject(YObject *o);
	//! this slot is called if the environment is changed. This is neccessary to
	//! set the correct values e.g. CalcPreemptionTime
	void				EnvironmentChanged()						{	UpdateObject(object);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerListView : public YListView
{	Q_OBJECT
public:					YSchedulerListView(YSchedulerSet *s, QWidget *parent, const QString &title) :
						YListView(s,parent,title)
						{}
 virtual YListViewItem* CreateNewItem(QListViewItem *parent, YObject *object, QListViewItem *after=0)
						{	if ( parent )
								return new YSchedulerListViewItem(object,parent,after);
							else
								return new YSchedulerListViewItem(object,this,after);
						}
	virtual int			InitInstance()
						{
							addColumn(MSGP_ENVIRONMENT_SCHEDULER);
							addColumn(MSGP_ENVIRONMENT_CALCPREEMPTIONTIME);
							addColumn(MSGP_ENVIRONMENT_TIMERTICK);
							return YListView::InitInstance();
						}
public slots:
	void				UpdateObject(YObject *o)
	{
		setEnabled( ((YEnvironment*)o)->GetExecutive() );
	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerStringList : public YStringList
{	Q_OBJECT
public:					YSchedulerStringList(YSchedulerPlugIns *set) : YStringList(set)	{ }
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTickEnvironmentWidget : public YYasaTimeWidget
{	Q_OBJECT
	YListView			*schedulerlistview;
public:					YTickEnvironmentWidget(YObject *environment, QWidget *parent, YListView *schedulerlist) :
						YYasaTimeWidget(0, parent),
						schedulerlistview(schedulerlist)
						{	UpdateWidget();
							Q_UNUSED(environment);
						}
	YScheduler*			GetCurrentScheduler() const
						{
							YListViewItem	*item=(YListViewItem*)schedulerlistview->currentItem();
							YScheduler	*scheduler=0;
							if ( item )
								scheduler=(YScheduler*)item->GetObject();
							return scheduler;
						}
public slots:
	virtual void		UpdateWidget()
						{	YASA_TIME	time=YTIME::YASA_TIME_MIN;
							YScheduler	*scheduler=GetCurrentScheduler();
							if ( scheduler )
								time=scheduler->GetTimerTick();
							if ( GetYasaTime() != time )
								SetYasaTime(time);
						}
	virtual void		UpdateValue(YASA_TIME value)
						{
							YScheduler *scheduler=GetCurrentScheduler();
							if ( scheduler )
							{
								//! check if
								if ( scheduler->GetTimerTick().GetContent() != value )
								{
									if ( value )								// set real new and non derived tick
										scheduler->SetTimerTick( value );
									else										// tick timer = 0 -> derived
										scheduler->DeriveTimerTick();
								}
							}
						}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSchedulerListViewGroup : public QGroupBox, public YWidget
{	Q_OBJECT
protected:
	YScheduler			*scheduler;				//! current selected scheduler
	YListView			*listview;
	QHBox				*hbox;
	QLineEdit			*cpuinfo;
	YSchedulerStringList schedulerlist;
	QComboBox			*schedulercombo;
	QComboBox			*calcpreemptiontimecombo;
	YTickEnvironmentWidget	*timertickwidget;
public:					YSchedulerListViewGroup(YEnvironment *e, QWidget *parent) :
						QGroupBox(1,Horizontal, MSGP_ENVIRONMENT_SCHEDULERS, parent),
						YWidget( e ),
						scheduler(0),
						listview(0),
						hbox(0),
						schedulerlist( & YSchedulerPlugIns::GetSchedulerPlugIns() ),
						schedulercombo(0),
						calcpreemptiontimecombo(0)
						{
							connect( e, SIGNAL( UpdateObject() ), this, SLOT( UpdateWidget() ) );
							setAlignment(AlignHCenter);
						}
	virtual int			InitInstance();
	virtual QWidget*	GetWidget()								{ return this;		}
	int					CreateWidgets(YSet *set, QWidget *parent);
public slots:
	// Stringlist of scheduler was changed
	void				SchedulerListChanged()
						{
							schedulercombo->clear();
							schedulercombo->insertStringList(schedulerlist);
							UpdateWidgets( listview->GetCurrentItem() );
						}
	// Current item in listview changed -> update info buttons below
	virtual void		UpdateWidgets(QListViewItem *currentitem);
	virtual void		ListViewEmpty()							{ UpdateWidgets(0);	}

	//! scheduler in combo box changed
	void				SchedulerChanged(int index);

	//! calc preemption time in combo box changed
	void				CalcPreemptionTimeChanged(int index);

	virtual void		UpdateWidget()
						{
							UpdateWidgets( listview->currentItem() );
							//listview->setEnabled( ((YEnvironment*)object )->GetExecutive() );
						}

};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_ENVIRONMENTWINDOW_INCLUDE
