//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA File dialog
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/filedialog.cpp,v 1.4 2003/01/23 09:18:35 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "filedialog.h"
#include "list.h"
#include <stdlib.h>
#include "application.h"
#include "output.h"
#include <time.h>
#include <sys/stat.h>
#ifdef _MSC_VER
#include <sys/utime.h>
#else
#include <utime.h>
#endif
#include <yasagui/yasagui_customize.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YFileDialog::YFileDialog(const QString &dir, const QString &title, const QString &filter, QFileDialog::Mode mode) :
QFileDialog(dir, 0, 0, title, TRUE)
{
	QStringList	filters;
	setViewMode( QFileDialog::Detail );
	filters.append( filter );
	filters.append( MSGP_ALLFILES );
	setFilters(filters);
//	setMode(QFileDialog::ExistingFiles);
		setMode(mode);
	resize(800,600);
	setCaption(title);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//\ mode can be:	QFileDialog::AnyFile (for saving files)
//					QFileDialog::ExistingFile (for loading files)
int YFileDialog::Request(const QFileInfo &fileinfo, QFileDialog::Mode mode)
{
	setMode(mode);
	if ( fileinfo.filePath().length() )
		setSelection( fileinfo.absFilePath() );

	if ( exec() == QDialog::Accepted )
		return 0;
	return YASAE_OPERATION_ABORTED;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Input: filename should be an absolute path
//			mode=true : if filename is relative nothing is done (default)
//			    =false: if filename is relative filename points from the current dir !
int YFileOperations::MakeRelative(QString &filename, const QString &dir, bool mode)
{
	const QString separator=DIR_SEPARATOR;
	const QString parent=PARENTDIR_PREFIX;

	QFileInfo	dirinfo(dir);
	QFileInfo	fileinfo(filename);
	dirinfo.convertToAbs();
	QString		dirtmp=dirinfo.absFilePath();

	if ( fileinfo.isRelative() && mode)
		return 0;
	fileinfo.convertToAbs();
	QString		filetmp=fileinfo.absFilePath();
	unsigned int count=0, lastseparator=0;

	// find first difference between filename and dir
	if ( dirtmp.right(separator.length()) != separator )
		dirtmp.append(DIR_SEPARATOR);
	while ( (dirtmp.length() > count) && (filetmp.length() > count) )
	{
		if ( dirtmp.at(count) != filetmp.at(count) )
			break;
		if ( dirtmp.mid(count, separator.length() ) == separator )
			lastseparator=count;
		count++;
	};

	if ( lastseparator )									// same root ?
	{
		filetmp.remove(0,lastseparator);					// file before dir?
		if ( dirtmp.length()-1 > lastseparator )
		{
			filetmp.remove(0,1);							// remove separator
			dirtmp.remove(0,lastseparator+1);
			for ( count=0; count<dirtmp.length(); count++)
			{
				if ( dirtmp.mid(count, separator.length() ) == separator )
				{
					filetmp.insert(0, separator);
					filetmp.insert(0, parent);
				}
			}
		}
		else
		{
			filetmp.insert(0, CURRENTDIR_PREFIX);
			filetmp.insert(0, dirtmp.right( dirtmp.length() - count ) );
		}
		filename=filetmp;
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::AddSubDirectory(QString &path, const QString &subdir)
{
	QDir	dir(path);

	const QString separator=DIR_SEPARATOR;

	if ( path.length() && ( ! dir.isRoot() ) )
	{
		if ( path.right(separator.length() ) != separator )
		{
			path+=separator;
		}
		path+=subdir;
	}
	else
		path=subdir;
	return 0;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YFileOperations::AddDirectory(const QString &path, const QString &subdir)
{
	QString	dest(path);
	QDir	dir(dest);

	const QString separator=DIR_SEPARATOR;

	if ( dest.length() && ( ! dir.isRoot() ) )
	{
		if ( dest.right(separator.length() ) != separator )
		{
			dest+=separator;
		}
		dest+=subdir;
	}
	else
		dest=subdir;
	return dest;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::CreateDirectory(const QString &directory)
{
	QDir	dir(directory);

	if ( dir.exists() || dir.mkdir(directory) )
		return 0;
	return YASAE_CREATE_DIRECTORY;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::CopyFile(const QString &src, const QString &dest, bool overwrite)
{
#if _WINDOWS
	QString command="copy \"%1\" \"%2\"";
#else
	QString command="cp \"%1\" \"%2\"";
#endif
	if ( overwrite || ! QFile( QDir::convertSeparators(dest) ).exists() )
	{
		command=command.arg( QDir::convertSeparators(src) );
		command=command.arg( QDir::convertSeparators(dest) );
		return RunCommand( command );
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::RenameFile(const QString &src, const QString &dest, bool overwrite)
{
/*#if _WINDOWS
	QString command="ren \"%1\" \"%2\"";
#else
	QString command="mv -f \"%1\" \"%2\"";
#endif
*/
	QString command;
	if (overwrite)
		command="mv -f \"%1\" \"%2\"";
	else
		command="mv \"%1\" \"%2\"";

	if ( overwrite || ! QFile( QDir::convertSeparators(dest) ).exists() )
	{
		command=command.arg( QDir::convertSeparators(src) );
		command=command.arg( QDir::convertSeparators(dest) );
		return RunCommand( command );
	}
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::CopyDirectory(const QString &src, const QString &dest, bool subdir)
{
	QString command;
/*
#if _WINDOWS
	if ( subdir )
		command="xcopy /E /C /F /Y \"%1\" \"%2\"";
	else
		command="xcopy /C /F /Y \"%1\" \"%2\"";
#else
*/
	if ( subdir )
		command="cp -f -v -R \"%1\" \"%2\"";
	else
		command="cp -f -v \"%1\" \"%2\"";
//#endif
	command=command.arg( ConvertToUnixPath(src) );
	command=command.arg( ConvertToUnixPath(dest) );
	return RunCommand( command );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function corrects the filename to ASCII. All non ascii characters will be set to '_'
//! Example: "Das ist eine bung" -> "Das_is_eine__bung"
//! Attention: Only filenames are allowedsince separtors will be corrected too.
QString YFileOperations::CorrectFileName(const QString &name)
{
	QString	dest;
	QChar	a;

	for (unsigned int i=0 ; i<name.length(); i++)
	{
		a=name.at(i);
		if ( !(a>='a' && a<='z') && !(a>='A' && a<='Z') && !(a>='0' && a<='9') && a!='_' )
			a='_';
		dest+=a;
	}
	return dest;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function corrects the filename to ASCII. All non ascii characters will be set to '_'
//! Example: "Das ist eine bung" -> "Das_is_eine__bung"
//! Attention: Only filenames are allowedsince separtors will be corrected too.
QString YFileOperations::GetFileName(const QString &path)
{
	QFileInfo	fi(path);
	return fi.fileName();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function corrects the filename to ASCII. All non ascii characters will be set to '_'
//! Example: "Das ist eine bung" -> "Das_is_eine__bung"
//! Attention: Only filenames are allowedsince separtors will be corrected too.
QString YFileOperations::CutExtension(const QString &path)
{
	for ( int i=path.length()-1 ; i>=0 ; i--)
	{
		if ( path.at(i)==EXTENSION_SEPARATOR )
			return path.left(i);
	}
	return path;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function corrects the filename to ASCII. All non ascii characters will be set to '_'
//! Example: "Das ist eine bung" -> "Das_is_eine__bung"
//! Attention: Only filenames are allowedsince separtors will be corrected too.
int YFileOperations::AddToFileName(QString &path, const QString &addi)
{
	path+=addi;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function corrects the filename to ASCII. All non ascii characters will be set to '_'
//! Example: "Das ist eine bung" -> "Das_is_eine__bung"
//! Attention: Only filenames are allowedsince separtors will be corrected too.
int YFileOperations::AddExtension(QString &path, const QString &addi)
{
	path+=EXTENSION_SEPARATOR;
	path+=addi;
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! The given directory will be removed. If subdir=true
void YFileOperations::RemoveDirectory(const QString &name, bool subdir, bool force)
{
/*
#if _WINDOWS
	QString command="del ";
	if (subdir)
		command+="/S ";
	if (force)
		command+="/F ";
#else
*/
	QString command="rm -v ";
	if (subdir)
		command+="-r ";
	if (force)
		command+="-f ";
//#endif
	command+=ConvertToUnixPath(name);
	RunBashCommand( command );
}

/*
class YProcess : public QProcess
{
public:				YProcess()
					{
					}
	int				Run(const QString &command)
	{
		addArgument(command);
        connect( this, SIGNAL(readyReadStdout()), this, SLOT(readFromStdout()) );
		if ( start() )

	}
public slots:
	void			ReadStdOutFromProcess()
	{
		YApplication::GetOutput()->Message(
	}
	void			ReadStdErrFromProcess();
};
*/

//#include <stdio.h>
//#include "application.h"
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::RunCommand(const QString &name, bool showcommand)
{
	//FILE	*fin;
	//char	buffer[BUFSIZ];
	YOutput	*output=YApplication::GetOutput();

	if ( showcommand )
	{
		if ( output )
		{
			output->Command( QString(">%1").arg(name) );
		}
	}
//#if QT_VERSION<300

/*	fin=_popen( name.latin1() , "r" );
	fin=_popen( "dir e:\\music" , "rt" );
	if ( fin )
	{
		((YApplication*)qApp)->ProcessEvents();
		while ( fgets( buffer, BUFSIZ, fin ) )
		{
			output->Message( QString(buffer) );
		};
		_pclose(fin);
		output->Message( QString("errno %1").arg( errno) );
	}
	else
		output->Error( QString("%1 occured!").arg(errno) );
	return 0;
*/
	return system( name.latin1() );
/*
#else
	YProcess	process;
#endif
*/
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::RunBashScript(const QString &name, bool showcommand)
{
	return RunCommand( QString(BASH_SCRIPT_STARTUP).arg(name, showcommand ) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YFileOperations::RunBashCommand(const QString &name, bool showcommand)
{
	return RunCommand( QString(BASH_STARTUP).arg(name, showcommand ) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YFileOperations::ConvertToUnixPath(const QString &srcpath)
{
	QString	path=srcpath;
	QChar	a;

	if (path.length() >1 )
	{
		a=path.at(1);
		if ( a==':' )
		{
			path.remove(1,1);
			path.insert( 0, CYGWINPATH_EXTENSION);
		}
	}
	return path;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Returns the modification datetime of the used file
int YFile::GetModifiedTime(QDateTime &datetime) const
{
	QFileInfo		fileinfo( *this );
	if ( fileinfo.exists() )
	{
		datetime=fileinfo.lastModified();
		return 0;
	}
	return YASAE_COULD_NOT_OPEN_FILE;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Sets the modification datetime of the used file
int YFile::SetModifiedTime(const QDateTime &dt)
{
	// do not use QDateTime epoch( QDate( 1970, 1,1), QTime( 0,0,0) );
	// since it will be localized
	QDateTime		epoch;
#ifdef _MSC_VER
	struct _utimbuf	newtime;
#else
	struct utimbuf	newtime;
#endif

	time_t			t;

	epoch.setTime_t(0);
	t=epoch.secsTo( dt );			// seconds since epoch
#ifdef _MSC_VER
	t=t+_timezone;					// correct time zone
#else
	t=t+timezone;					// correct time zone
#endif

	newtime.actime = t;
	newtime.modtime = t;
#ifdef _MSC_VER
	_utime ( name().latin1(), &newtime );
#else
	utime ( name().latin1(), &newtime );
#endif

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Opens the file, request last modification time loads old file to stringlist, truncates the file to 0
int YSourceFile::OpenFile()
{
	int	retvalue=YASAE_COULD_NOT_OPEN_FILE;
	if ( isOpen() )
		FinalizeFile();
	if ( exists() )
	{
		GetModifiedTime( lastmodified );
		if ( open(IO_ReadWrite) )
			retvalue=0;
		contentchanged=false;	// if contents will not be changed the original date stamp will be restored
	}
	else
	{
		if ( open(IO_ReadWrite) )
			retvalue=GetModifiedTime( lastmodified );
	}

	// File successfully opened ?
	if ( ! retvalue )
	{
		// Read contents
		QString	line;
		while ( readLine(line, 256) != -1 )
		{
			if ( line.length() && ( line.at( line.length()-1 )==10 ) )
			{
//				a=line.at( line.length()-1 );
//				if ( a==10 )
					line.remove(line.length()-1,1);
			}
			if ( CheckImportant(line) )
				oldcontents.append( line );
		}

		// close file and reopen with truncate option
		close();
		if ( ! open( IO_WriteOnly | IO_Truncate) )
				retvalue=YASAE_COULD_NOT_OPEN_FILE;
	}

	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
int YSourceFile::WriteImportant(const QString &line)
{
	int	index;

	index=oldcontents.findIndex( line );
	if (  index == -1 )
		contentchanged=true;
	else
		oldcontents.remove( oldcontents.at(index) );
	return WriteUnimportant(line);
}
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSourceFile::WriteLine(const QString &line)
{
	if ( oldcontents.count() && CheckImportant(line) )
	{
		if ( oldcontents.first() == line )
		{
			oldcontents.remove( oldcontents.at(0) );
		}
		else
		{
			oldcontents.clear();
			contentchanged=true;
		}
	}

	if ( writeBlock( line.latin1(), line.length() ) == -1 )
		return YASAE_WRITE_FILE;
	if ( writeBlock( SG_NEW_LINE, 1) == -1 )
		return YASAE_WRITE_FILE;
	return 0;
}
