//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA frame code declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/frame.h,v 1.4 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Use frames to create mdi child windows. In this way you can cascade frames
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_FRAME_INCLUDE
#define	YASAGUI_FRAME_INCLUDE

#include <qwidget.h>
#include "translations.h"
#include "object.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFrame;
class YFrameArgs;
class YMDIWindow;
typedef YFrame* (*CreateYFrameCall) (QWidget *parent, YFrameArgs &args);


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFrameArgs
{
protected:
	QString				title;
	CreateYFrameCall	createcall;
	int					refcount;
public:					YFrameArgs(CreateYFrameCall call, const QString &t) :
						title(t),
						createcall(call),
						refcount(0)
						{}
	virtual				~YFrameArgs()
						{}
	const QString&		GetTitle() const							{ return title;			}
	void				SetTitle(const QString &t)					{ title=t;				}
	YFrame*				CreateFrame(QWidget *parent)
						{
							refcount++;
							return createcall(parent, *this );
						}
	static void			DeleteFrameArgs(YFrameArgs &args)
						{
							args.refcount--;
							if (!args.refcount)
								delete &args;
						}
	virtual YObject*	GetObject() const							{	return 0;			}
	//! This function is used to initialize mdi windows after creation and before calling window->show()
	//! e.g. connect to signals/slots
//	virtual void		PostCreateWindow(YMDIWindow *)				{	return;				}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFrameObjectArgs : public YFrameArgs
{	YObject				*object;
public:					YFrameObjectArgs(CreateYFrameCall call, YObject *o) :
						YFrameArgs(call, o->GetName() ),
						object(o)
						{}
	virtual YObject*	GetObject() const							{ return object;	}
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YFrame : public QObject
{	Q_OBJECT
	YFrameArgs			&frameargs;									//! args to generate this frame
public:					YFrame(YFrameArgs &args) :
						frameargs(args)
						{}
						~YFrame();
	virtual int			InitInstance()					{ return 0;	}
	virtual QWidget*	GetMainWidget() =0;
	YFrameArgs&			GetFrameArgs() const			{ return frameargs;	}
	void				SetTitle(const QString &newtitle)
						{
							frameargs.SetTitle(newtitle);			//! we do not compare due to speed issues
							TitleChanged(newtitle, frameargs.GetObject() );
						}
	//! This function is used to initialize mdi windows after creation and before calling window->show()
	//! e.g. connect to signals/slots
	virtual void		PostCreateWindow(YMDIWindow *)				{	return;				}
signals:
	void				CloseFrame();
	void				TitleChanged(const QString &newname, YObject *object);
	void				StatusMessage(const QString &message, int time_ms=0);
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YObjectFrame : public YFrame
{	Q_OBJECT
public:					YObjectFrame(YFrameArgs &args):
						YFrame(args)
						{}
	virtual int			InitInstance();
	YObject*			GetObject() const
						{ return ((YFrameObjectArgs&)GetFrameArgs()).GetObject();	}
public slots:
	void				ObjectUpdated()
						{
							SetTitle(GetObject()->GetName());
						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_FRAMES_INCLUDE
