//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA logfile parser
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/logfilestatistics.cpp,v 1.4 2003/01/23 09:18:35 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "logfilestatistics.h"
#include "logfile.h"
#include "widgets.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YYasaTimeStatisticEntry::_GetKey() const
{
	return YString::Justify( YTIME::GetContentStringMax( GetContent() ) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YIntStatisticEntry::_GetKey() const
{
	return YString::Justify( QString::number( GetContent() ) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YPriorityStatisticEntry::_GetKey() const
{
	return YString::Justify( QString::number( GetContent() ) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YPercentStatisticEntry::_GetKey() const
{
	return YString::Justify( QString::number( GetContent() ) );
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QString YVersionStatisticEntry::_GetKey() const
{
	return YString::Justify( QString::number( GetContent() ) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Calculate time values (min, max, averagea and percentage if needed. The given list is cleared
//! after using
void YLogFileStatistics::CalculateTimingValues(YEnvironmentLogFileSet &envlogfile, QList<YASA_TIME> &list, int firstproperty, int propertycount)
{
	unsigned int	i;
	YASA_TIME		min,max,sum,tmp;

	tmp=max=sum=0;
	min=YASA_TIME_INFINITY;
	for ( i=0; i<list.count() ; i++)
	{
		tmp=*list.at(i);
		if ( tmp<min ) min=tmp;
		if ( tmp>max ) max=tmp;
		sum+=tmp;
	}
	if ( tmp<min ) min=tmp;
	if ( list.count() )
	{
		if ( propertycount>0 )
		{
			if ( min!=YASA_TIME_INFINITY)
				((YYasaTimeStatisticEntry*)properties[firstproperty])->SetContent(min);		// min <something>, e.g. MIN_COMPUTATION_TIME
			else
				((YYasaTimeStatisticEntry*)properties[firstproperty])->Reset();				// min <something>, e.g. MIN_COMPUTATION_TIME
		}
		if ( propertycount>1 )
			((YYasaTimeStatisticEntry*)properties[firstproperty+1])->SetContent(max);		// max <something>, e.g. MAX_COMPUTATION_TIME
		if ( propertycount>2 )
			((YYasaTimeStatisticEntry*)properties[firstproperty+2])->SetContent(sum/list.count() );	// average something, e.g. AVERAGE_COMPUTATION_TIME
		if ( propertycount>3 )
			((YYasaTimeStatisticEntry*)properties[firstproperty+3])->SetContent(sum);		// percentage of scheduling period, e.g. COMPUTATION_TIME
		if ( propertycount>4 )
		{
			YCPULogFile	*first=(YCPULogFile*)envlogfile.At(0);
			if ( first )
			{
				((YPercentStatisticEntry*)properties[firstproperty+4])->CalcContent( first->GetStatistics().GetExecutionDuration(), sum );		// percentage of scheduling period, e.g. AVERAGE_COMPUTATION_TIME
			}
			else
				((YPercentStatisticEntry*)properties[firstproperty+4])->Reset();			// percentage of scheduling period, e.g. COMPUTATION_TIME_PERCENT
		}
	}
	list.clear();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTaskLogFileStatistics::CalculateStatistics(YEnvironmentLogFileSet &envlogfile)
{
	CalculateTimingValues( envlogfile, computationtimes	, MIN_COMPUTATION_TIME, 5);
	CalculateTimingValues( envlogfile, jittertimes		, MIN_JITTER, 3);
	CalculateTimingValues( envlogfile, reactiontimes	, MIN_REACTION_TIME, 3);
	((YIntStatisticEntry*) properties[ID])->SetContent( task->GetId() );
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMutexLogFileStatistics::CalculateStatistics(YEnvironmentLogFileSet &envlogfile)
{
	CalculateTimingValues( envlogfile, lockingtimes	, MIN_LOCKING_TIME, 3);
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YCPULogFileStatistics::CalculateStatistics(YCPULogFile &)
{
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int	YEnvironmentLogFileStatistics::CalculateStatistics(YEnvironmentLogFileSet &envlogfile)
{
	// Is this logfile valid ?
	if ( envlogfile.GetCount() )
	{
		YCPULogFile		*cpulogfile=0;
		unsigned int	i;
		int				activations=0;
		int				starts=0;
		int				suspended=0;
		int				preemptions=0;
		int				cpulosses=0;
		int				exclusions=0;
		int				skippedthreads=0;
		int				emergencythreads=0;
		int				detecteddeadlinesmisses=0;
		int				deadlinetolerancemisses=0;
		int				schedulercalls=0;
		int				taskswitches=0;
		int				taskcpuswitches=0;
		int				matcheddeadlines=0;
		int				misseddeadlines=0;
		int				logbuffersize=0;
		int				usedlogbuffersize=0;
		int				lockedmutexes=0;
		int				relockedmutexes=0;
		int				unlockedmutexes=0;
		int				blockedmutexes=0;
		int				maxpriorityceiling=0;
		int				numberofprioritychanges=0;
		YASA_TIME		idletime=0;
		YASA_TIME		schedulingtime=0;
		PERCENTTYPE		utilization=0;

		for (i=0; i<envlogfile.GetCount(); i++)
		{
			cpulogfile= (YCPULogFile*) envlogfile.At(i);
			if ( cpulogfile )
			{
				matcheddeadlines		+= cpulogfile->GetStatistics().GetMatchedDeadlines();
				misseddeadlines			+= cpulogfile->GetStatistics().GetMissedDeadlines();
				detecteddeadlinesmisses	+= cpulogfile->GetStatistics().GetDetectedDeadlineMisses();
				deadlinetolerancemisses	+= cpulogfile->GetStatistics().GetDeadlineToleranceMisses();
				schedulercalls			+= cpulogfile->GetStatistics().GetSchedulerCalls();
				taskswitches			+= cpulogfile->GetStatistics().GetTaskSwitches();
				taskcpuswitches			+= cpulogfile->GetStatistics().GetTaskCPUSwitches();
				logbuffersize			+= cpulogfile->GetStatistics().GetLogBufferSize();
				usedlogbuffersize		+= cpulogfile->GetStatistics().GetUsedLogBufferSize();
				idletime				+= cpulogfile->GetStatistics().GetIdleTime();
				schedulingtime			+= cpulogfile->GetStatistics().GetTimeOfScheduling();
				utilization				+= cpulogfile->GetStatistics().GetUtilizationPercent();
				lockedmutexes			+= cpulogfile->GetStatistics().GetMutexLocked();
				relockedmutexes			+= cpulogfile->GetStatistics().GetMutexRelocked();
				unlockedmutexes			+= cpulogfile->GetStatistics().GetMutexUnlocked();
				blockedmutexes			+= cpulogfile->GetStatistics().GetMutexBlocked();
				numberofprioritychanges	+= cpulogfile->GetStatistics().GetNumberOfPriorityChanges();
				if ( maxpriorityceiling < cpulogfile->GetStatistics().GetMaxPriorityCeiling() )
					maxpriorityceiling = cpulogfile->GetStatistics().GetMaxPriorityCeiling();
			}
		}

		for ( i=0; i<envlogfile.GetTaskSet().GetCount() ; i++)
		{
			if ( envlogfile.GetTaskSet().At(i) )
			{
				YTaskLogFileStatistics	&statistic=((YLogFileTask*) envlogfile.GetTaskSet().At(i))->GetStatistics();
				activations			+= statistic.GetActivations();
				starts				+= statistic.GetStarts();
				suspended			+= statistic.GetSuspendedThreads();
				preemptions			+= statistic.GetPreemptions();
				cpulosses			+= statistic.GetCPULosses();
				exclusions			+= statistic.GetExclusions();
				skippedthreads		+= statistic.GetSkippedThreads();
				emergencythreads	+= statistic.GetEmergencyThreads();
			}
		}


		if ( cpulogfile )
		{
			((YIntStatisticEntry*)		properties[NUMBER_OF_CPUS])					->SetContent( envlogfile.GetCount() );
			((YIntStatisticEntry*)		properties[SCHEDULER_CALLS])				->SetContent( schedulercalls );
			((YIntStatisticEntry*)		properties[TASK_SWITCHES])					->SetContent( taskswitches);
			((YIntStatisticEntry*)		properties[TASK_CPU_SWITCHES])				->SetContent( taskcpuswitches);
			((YIntStatisticEntry*)		properties[MATCHED_DEADLINES])				->SetContent( matcheddeadlines );
			((YIntStatisticEntry*)		properties[MISSED_DEADLINES])				->SetContent( misseddeadlines );
			((YIntStatisticEntry*)		properties[DETECTED_DEADLINE_MISSES])		->SetContent( detecteddeadlinesmisses );
			((YIntStatisticEntry*)		properties[MISSED_TOLERANCE_DEADLINES])		->SetContent( deadlinetolerancemisses );
			((YIntStatisticEntry*)		properties[SIZE_OF_LOGBUFFER])				->SetContent( logbuffersize );
			((YIntStatisticEntry*)		properties[USAGE_OF_LOGBUFFER])				->SetContent( usedlogbuffersize );
			((YYasaTimeStatisticEntry*)	properties[IDLE])							->SetContent( idletime );
			((YYasaTimeStatisticEntry*)	properties[TIME_OF_SCHEDULING])				->SetContent( schedulingtime );
			((YIntStatisticEntry*)		properties[MUTEX_LOCKED])					->SetContent( lockedmutexes );
			((YIntStatisticEntry*)		properties[MUTEX_RELOCKED])					->SetContent( relockedmutexes );
			((YIntStatisticEntry*)		properties[MUTEX_UNLOCKED])					->SetContent( unlockedmutexes );
			((YIntStatisticEntry*)		properties[MUTEX_BLOCKED])					->SetContent( blockedmutexes );
			((YPriorityStatisticEntry*)	properties[MUTEX_MAX_PRIORITY_CEILING])		->SetContent( maxpriorityceiling );
			((YIntStatisticEntry*)		properties[MUTEX_NUMBER_OF_PRIORITY_CHANGES])->SetContent(numberofprioritychanges );

			((YIntStatisticEntry*)		properties[ACTIVATIONS])					->SetContent( activations );
			((YIntStatisticEntry*)		properties[STARTS])							->SetContent( starts );
			((YIntStatisticEntry*)		properties[THREADS_SUSPENDED])				->SetContent( suspended );
			((YIntStatisticEntry*)		properties[PREEMPTIONS])					->SetContent( preemptions );
			((YIntStatisticEntry*)		properties[CPU_LOSSES])						->SetContent( cpulosses );
			((YIntStatisticEntry*)		properties[EXCLUSIONS])						->SetContent( exclusions );
			((YIntStatisticEntry*)		properties[SKIPPED_THREADS])				->SetContent( skippedthreads );
			((YIntStatisticEntry*)		properties[EMERGENCY_THREADS])				->SetContent( emergencythreads );

			((YPercentStatisticEntry*)properties[UTILIZATION_OF_CPUS_PERCENT])->CalcContent( envlogfile.GetCount()*100, utilization );
			((YPercentStatisticEntry*)properties[USAGE_OF_LOGBUFFER_PERCENT])->CalcContent( logbuffersize, usedlogbuffersize );
			((YPercentStatisticEntry*)properties[IDLE_PERCENT])				 ->CalcContent( cpulogfile->GetStatistics().GetExecutionDuration(), idletime );
			((YPercentStatisticEntry*)properties[TIME_OF_SCHEDULING_PERCENT])->CalcContent( cpulogfile->GetStatistics().GetExecutionDuration(), schedulingtime );
		}
	}
	else
	{
		properties[EXECUTIVE_NAME]->Invalid();
		properties[EXECUTIVE_VERSION]->Invalid();
		properties[YASA_VERSION]->Invalid();
		properties[NUMBER_OF_CPUS]->Invalid();
		properties[SCHEDULER_CALLS]->Invalid();
		properties[TASK_SWITCHES]->Invalid();
		properties[TASK_CPU_SWITCHES]->Invalid();
		properties[MATCHED_DEADLINES]->Invalid();
		properties[MISSED_DEADLINES]->Invalid();
		properties[DETECTED_DEADLINE_MISSES]->Invalid();
		properties[MISSED_TOLERANCE_DEADLINES]->Invalid();
		properties[SIZE_OF_LOGBUFFER]->Invalid();
		properties[USAGE_OF_LOGBUFFER]->Invalid();
		properties[IDLE]->Invalid();
		properties[TIME_OF_SCHEDULING]->Invalid();
		properties[MUTEX_LOCKED]->Invalid();
		properties[MUTEX_RELOCKED]->Invalid();
		properties[MUTEX_UNLOCKED]->Invalid();
		properties[MUTEX_BLOCKED]->Invalid();
		properties[MUTEX_MAX_PRIORITY_CEILING]->Invalid();
		properties[MUTEX_NUMBER_OF_PRIORITY_CHANGES]->Invalid();
		properties[ACTIVATIONS]->Invalid();
		properties[STARTS]->Invalid();
		properties[THREADS_SUSPENDED]->Invalid();
		properties[PREEMPTIONS]->Invalid();
		properties[CPU_LOSSES]->Invalid();
		properties[EXCLUSIONS]->Invalid();
		properties[SKIPPED_THREADS]->Invalid();
		properties[EMERGENCY_THREADS]->Invalid();
		properties[UTILIZATION_OF_CPUS_PERCENT]->Invalid();
		properties[USAGE_OF_LOGBUFFER_PERCENT]->Invalid();
		properties[IDLE_PERCENT]->Invalid();
		properties[TIME_OF_SCHEDULING_PERCENT]->Invalid();
	}

	return 0;
}
