//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Main Window code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/mainwindow.cpp,v 1.3 2003/01/23 09:18:36 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qapplication.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qmainwindow.h>
#include <qworkspace.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qmessagebox.h>
#include <yasagui/yasagui.h>
#include "translations.h"
#include "config.h"
#include "mainwindow.h"
#include "application.h"
#include "actionhandler.h"
#include "workspacemanager.h"
#include "workspacetreeview.h"
#include "output.h"
#include "helpwindow.h"
#include "actions.h"
#include "settings.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YMainWindow::~YMainWindow()
{
	if (filemenu)			delete filemenu;
	if (environmentmenu)	delete environmentmenu;
	if (windowmenu)			delete windowmenu;
	if (helpmenu)			delete helpmenu;
	if (workspacetoolbar)	delete workspacetoolbar;
	if (environmenttoolbar)	delete environmenttoolbar;

	if (workspacetreeview)	delete workspacetreeview;
	if (outputview)			delete outputview;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Initialize main window and all child windows
int YMainWindow::InitInstance(YApplication *app)
{
	int					retvalue;
	int					a;

	application=app;
	setCaption(MSGP_YASA);
	splittv				= new QSplitter( QSplitter::Vertical, this);
    splitth				= new QSplitter( QSplitter::Horizontal, splittv);
	outputview			= new YOutputView( splittv );
	workspacetreeview	= new YWorkSpaceTreeView( 0, splitth );
	workspace			= new QWorkspace( splitth );
	workspace->setBackgroundMode( PaletteDark );
	splitth->setResizeMode( workspace, QSplitter::Stretch );
	//splittv->setResizeMode( 0, QSplitter::Stretch );


	// correct size of horizontal splitter
	QValueList<int>	widthlist;
	QSize t=splitth->size();										// get current size
	widthlist.append( t.width()*1/5 );
	widthlist.append( t.width()*4/5 );
	splitth->setSizes( widthlist );	//resize( 300, t.height() );//t.width()/5, 0 );

	// correct size of vertical splitter
	widthlist=splittv->sizes();
	a=widthlist[0]+widthlist[1];
	widthlist[0]=a*4/5;
	widthlist[1]=a*1/5;
	splittv->setSizes( widthlist );	//resize( 300, t.height() );//t.width()/5, 0 );


	retvalue=CreateActions();
	if ( retvalue )
		return retvalue;

	retvalue=CreateMenu();
	if ( retvalue )
		return retvalue;

	retvalue=CreateToolBar();
	if ( retvalue )
		return retvalue;

	retvalue=CreateStatusBar();
	if ( retvalue )
		return retvalue;

	retvalue=workspacetreeview->InitInstance();
	if ( retvalue )
		return retvalue;

	retvalue=outputview->InitInstance( application->GetOutput() );
	if ( retvalue )
		return retvalue;

	splitth->setResizeMode( workspacetreeview, QSplitter::Stretch);
	splittv->setResizeMode( outputview, QSplitter::Stretch);

	setCentralWidget( splittv );
	YApplicationFigures::SetIcon(*this, YApplicationFigures::ICON_APPLICATION );

	connect( app->GetWorkSpaceManager(), SIGNAL( WorkSpaceDestroyed(YWorkSpace *) ), workspacetreeview, SLOT( ClearListView(YWorkSpace *) ) );
	connect( app->GetWorkSpaceManager(), SIGNAL( NewWorkSpaceCreated(YWorkSpace *)   ), workspacetreeview, SLOT( UpdateListView(YWorkSpace *) ) );
	connect( app->GetWorkSpaceManager(), SIGNAL( WorkSpaceDestroyed(YWorkSpace *) ), this, SLOT( WorkSpaceActionsDisabled(YWorkSpace *) ) );
	connect( app->GetWorkSpaceManager(), SIGNAL( NewWorkSpaceCreated(YWorkSpace *)   ), this, SLOT( WorkSpaceActionsEnabled(YWorkSpace *) ) );

	emit StatusMessage( MSGP_WELCOME_STATUS, 0 );
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMainWindow::CreateActions()
{
	int count;
	for ( count=0; count<MAX_ACTIONS ; count++ )
		actions[count]=0;

	actions[NEW_WORKSPACE]		= new QAction( MSGP_NEW_WORKSPACE	, MSGP_NEW_WORKSPACE_MENU		, CTRL+Key_N	, this );
	actions[OPEN_WORKSPACE]		= new QAction( MSGP_OPEN_WORKSPACE	, MSGP_OPEN_WORKSPACE_MENU		, CTRL+Key_O	, this );
	actions[SAVE_WORKSPACE]		= new QAction( MSGP_SAVE_WORKSPACE	, MSGP_SAVE_WORKSPACE_MENU		, CTRL+Key_S	, this );
	actions[SAVE_WORKSPACE_AS]	= new QAction( MSGP_SAVE_WORKSPACE_AS,MSGP_SAVE_WORKSPACE_AS_MENU	, 0				, this );
	actions[CLOSE_WORKSPACE]	= new QAction( MSGP_CLOSE_WORKSPACE	, MSGP_CLOSE_WORKSPACE_MENU		, CTRL+Key_F4	, this );
	actions[EXIT]				= new QAction( MSGP_EXIT			, MSGP_EXIT_MENU				, ALT+Key_F4	, this );
	actions[EXECUTE]			= new QAction( MSGP_EXECUTE			, MSGP_EXECUTE_MENU				, CTRL+Key_F5	, this );
//	actions[ZOOM_IN]			= new QAction( MSGP_ZOOM_IN			, MSGP_ZOOM_IN_MENU				, Key_Plus		, this );
//	actions[ZOOM_OUT]			= new QAction( MSGP_ZOOM_OUT		, MSGP_ZOOM_OUT_MENU			, Key_Minus		, this );
	actions[ZOOM_IN]			= new QAction( MSGP_ZOOM_IN			, MSGP_ZOOM_IN_MENU				, CTRL+Key_Plus	, this );
	actions[ZOOM_OUT]			= new QAction( MSGP_ZOOM_OUT		, MSGP_ZOOM_OUT_MENU			, CTRL+Key_Minus, this );
	actions[CLOSE_WINDOW]		= new QAction( MSGP_CLOSE_WINDOW	, MSGP_CLOSE_WINDOW_MENU		, Key_Escape	, this );
	actions[NEW_WINDOW]			= new QAction( MSGP_NEW_WINDOW		, MSGP_NEW_WINDOW				, 0				, this );
	actions[NEW_FULLSCREEN_WINDOW]= new QAction( MSGP_NEW_FULLSCREEN_WINDOW	, MSGP_NEW_FULLSCREEN_WINDOW, CTRL+Key_F, this );
	actions[CLOSE_DOUBLE_WINDOWS]=new QAction( MSGP_CLOSE_DOUBLE_WINDOWS,MSGP_CLOSE_DOUBLE_WINDOWS_MENU	, 0			, this );
	actions[CLOSE_ALL_WINDOWS]	= new QAction( MSGP_CLOSE_ALL_WINDOWS,MSGP_CLOSE_ALL_WINDOWS_MENU	, 0				, this );
	actions[CASCADE_WINDOWS]	= new QAction( MSGP_CASCADE_WINDOWS	, MSGP_CASCADE_WINDOWS_MENU		, 0				, this );
	actions[TILE_WINDOWS]		= new QAction( MSGP_TILE_WINDOWS	, MSGP_TILE_WINDOWS_MENU		, SHIFT+Key_F4	, this );
	actions[ABOUT_QT]			= new QAction( MSGP_ABOUT_QT		, MSGP_ABOUT_QT_MENU			, 0				, this );
	actions[ABOUT_APP]			= new QAction( MSGP_ABOUT_TITLE		, MSGP_ABOUT_TITLE				, 0				, this );
	for ( count=0; count<MAX_ACTIONS ; count++ )
	{	if ( actions[count]==0 )
		{
			qWarning("YMainWindow::CreateActions(): Did you forget to add new actions?");
			return YASAE_OUT_OF_MEMORY;
		}
	}
	connect( actions[NEW_WORKSPACE]		, SIGNAL( activated() ), application->GetWorkSpaceManager(), SLOT( NewWorkSpace() ) );
	connect( actions[OPEN_WORKSPACE]	, SIGNAL( activated() ), application->GetWorkSpaceManager(), SLOT( OpenWorkSpace() ) );
	connect( actions[SAVE_WORKSPACE]	, SIGNAL( activated() ), application->GetWorkSpaceManager(), SLOT( SaveWorkSpace() ) );
	connect( actions[SAVE_WORKSPACE_AS]	, SIGNAL( activated() ), application->GetWorkSpaceManager(), SLOT( SaveWorkSpaceAs() ) );
	connect( actions[CLOSE_WORKSPACE]	, SIGNAL( activated() ), application->GetWorkSpaceManager(), SLOT( CloseWorkSpace() ) );
	connect( actions[EXIT]				, SIGNAL( activated() ), application, SLOT( closeAllWindows() ) );
	connect( actions[EXECUTE]			, SIGNAL( activated() ), this, SLOT( ExecuteCurrentEnvironment() ) );
	connect( actions[ZOOM_IN]			, SIGNAL( activated() ), this, SLOT( ZoomIn() ) );
	connect( actions[ZOOM_OUT]			, SIGNAL( activated() ), this, SLOT( ZoomOut() ) );
	connect( actions[CLOSE_WINDOW]		, SIGNAL( activated() ), this, SLOT( CloseWindow() ) );
	connect( actions[NEW_WINDOW]		, SIGNAL( activated() ), this, SLOT( NewWindow() ) );
	connect( actions[NEW_FULLSCREEN_WINDOW],SIGNAL( activated() ), this, SLOT( NewFullScreenWindow() ) );
	connect( actions[CLOSE_DOUBLE_WINDOWS], SIGNAL( activated() ), this, SLOT( CloseDoubleWindows() ) );
	connect( actions[CLOSE_ALL_WINDOWS]	, SIGNAL( activated() ), this, SLOT( CloseAllWindows() ) );
	connect( actions[CASCADE_WINDOWS]	, SIGNAL( activated() ), workspace, SLOT( cascade() ) );
	connect( actions[TILE_WINDOWS]		, SIGNAL( activated() ), workspace, SLOT( tile() ) );
	connect( actions[ABOUT_QT]			, SIGNAL( activated() ), this, SLOT( OpenQTAbout() ) );
	connect( actions[ABOUT_APP]			, SIGNAL( activated() ), this, SLOT( OpenHelpWindow() ) );
	YApplicationFigures::SetIconSet( *actions[NEW_WORKSPACE]	, YApplicationFigures::WORKSPACE_NEW );
	YApplicationFigures::SetIconSet( *actions[OPEN_WORKSPACE]	, YApplicationFigures::WORKSPACE_OPEN );
	YApplicationFigures::SetIconSet( *actions[SAVE_WORKSPACE]	, YApplicationFigures::WORKSPACE_SAVE );
	YApplicationFigures::SetIconSet( *actions[EXECUTE]			, YApplicationFigures::EXECUTE );
	YApplicationFigures::SetIconSet( *actions[ZOOM_IN]			, YApplicationFigures::ZOOM_IN );
	YApplicationFigures::SetIconSet( *actions[ZOOM_OUT]			, YApplicationFigures::ZOOM_OUT );
	WorkSpaceActionsDisabled(0);
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::WorkSpaceActionsEnabled(YWorkSpace *workspace)
{
	if ( workspace )
	{
		actions[SAVE_WORKSPACE]->setEnabled(true);
		actions[SAVE_WORKSPACE_AS]->setEnabled(true);
		actions[CLOSE_WORKSPACE]->setEnabled(true);
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::WorkSpaceActionsDisabled(YWorkSpace *workspace)
{
	actions[SAVE_WORKSPACE]->setEnabled(false);
	actions[SAVE_WORKSPACE_AS]->setEnabled(false);
	actions[CLOSE_WORKSPACE]->setEnabled(false);
	Q_UNUSED(workspace);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMainWindow::CreateMenu()
{
	bool	success=true;
	filemenu = new QPopupMenu( this );
	if ( ! filemenu )
		return YASAE_OUT_OF_MEMORY;

	menuBar()->insertItem( MSGP_FILE_MENU, filemenu );
	success&=actions[NEW_WORKSPACE]->addTo( filemenu );
	success&=actions[OPEN_WORKSPACE]->addTo( filemenu );
	success&=actions[CLOSE_WORKSPACE]->addTo( filemenu );
	filemenu->insertSeparator();
	success&=actions[SAVE_WORKSPACE]->addTo( filemenu );
	success&=actions[SAVE_WORKSPACE_AS]->addTo( filemenu );
	filemenu->insertSeparator();
	success&=actions[EXIT]->addTo( filemenu );
	if ( ! success )
		return YASAE_OUT_OF_MEMORY;

	environmentmenu = new QPopupMenu( this );
	if ( ! environmentmenu )
		return YASAE_OUT_OF_MEMORY;
	menuBar()->insertItem( MSGP_ENVIRONMENT_MENU, environmentmenu );
	success&=actions[EXECUTE]->addTo( environmentmenu );
	environmentmenu->insertSeparator();
	success&=actions[ZOOM_IN]->addTo( environmentmenu );
	success&=actions[ZOOM_OUT]->addTo( environmentmenu );
	if ( ! success )
		return YASAE_OUT_OF_MEMORY;


	windowmenu = new QPopupMenu( this );
	if ( ! windowmenu )
		return YASAE_OUT_OF_MEMORY;
	menuBar()->insertItem( MSGP_WINDOWS_MENU, windowmenu );
	connect(windowmenu, SIGNAL(aboutToShow()), this, SLOT(UpdateWindowsMenu()));

	helpmenu = new QPopupMenu( this );
	if ( ! helpmenu )
		return YASAE_OUT_OF_MEMORY;
	menuBar()->insertItem( MSGP_HELP_MENU, helpmenu );
	success&=actions[ABOUT_QT]->addTo( helpmenu );
	success&=actions[ABOUT_APP]->addTo( helpmenu );
	if ( ! success )
		return YASAE_OUT_OF_MEMORY;

	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMainWindow::CreateToolBar()
{
	bool	success=true;
	workspacetoolbar=new QToolBar( this , MSGP_FILE );
	if ( workspacetoolbar )
	{
		success&=actions[NEW_WORKSPACE]->addTo( workspacetoolbar );
		success&=actions[OPEN_WORKSPACE]->addTo( workspacetoolbar );
		success&=actions[SAVE_WORKSPACE]->addTo( workspacetoolbar );
	}

	environmenttoolbar=new QToolBar( this , MSGP_ENVIRONMENT );
	if (environmenttoolbar)
	{
		success&=actions[EXECUTE]->addTo( environmenttoolbar );
		environmenttoolbar->addSeparator();
		success&=actions[ZOOM_IN]->addTo( environmenttoolbar );
		success&=actions[ZOOM_OUT]->addTo( environmenttoolbar );
		if ( success )
			return 0;
	}



	return YASAE_OUT_OF_MEMORY;
}

#include <qlabel.h>
#include <qstatusbar.h>
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMainWindow::CreateStatusBar()
{
/*	QWidget	*w=new QLabel( statusBar(), "huhu");
	statusBar()->addWidget( w, 1, true );
	w=new QLabel( statusBar(), "huhu");
	w->setMaximumWidth(100);
	w->setMinimumWidth(100);
*/	//w->setText("jkjkjk");
	connect( this, SIGNAL( StatusMessage(const QString&, int) ), (const class QObject*)statusBar(), SLOT( message(const QString&, int )) );
	return 0;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::CloseAllWindows()
{
	QWidgetList		windowlist;
	unsigned int	i;

	windowlist = workspace->windowList();
	for (i=0; i<windowlist.count(); i++ )
	{
		windowlist.at(i)->close();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::UpdateWindowsMenu()
{
	QWidgetList		windowlist;
	unsigned int	i;
	int				id;

	windowmenu->clear();
	actions[NEW_WINDOW]->addTo(windowmenu);
	actions[NEW_FULLSCREEN_WINDOW]->addTo(windowmenu);
	windowmenu->insertSeparator();
	actions[CLOSE_WINDOW]->addTo(windowmenu);
	actions[CLOSE_DOUBLE_WINDOWS]->addTo(windowmenu);
	actions[CLOSE_ALL_WINDOWS]->addTo(windowmenu);
	windowmenu->insertSeparator();
	actions[CASCADE_WINDOWS]->addTo(windowmenu);
	actions[TILE_WINDOWS]->addTo(windowmenu);

	if ( workspace->windowList().count()>1 )
	{
		actions[CASCADE_WINDOWS]->setEnabled(true);
		actions[TILE_WINDOWS]->setEnabled(true);
	}
	else
	{
		actions[CASCADE_WINDOWS]->setEnabled(false);
		actions[TILE_WINDOWS]->setEnabled(false);
	}

	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		actions[CLOSE_WINDOW]->setEnabled(true);
		actions[NEW_WINDOW]->setEnabled(true);
		actions[NEW_FULLSCREEN_WINDOW]->setEnabled(true);
	}
	else
	{
		actions[CLOSE_WINDOW]->setEnabled(false);
		actions[NEW_WINDOW]->setEnabled(false);
		actions[NEW_FULLSCREEN_WINDOW]->setEnabled(false);
	}



	if ( workspace->windowList().isEmpty() )
	{
		actions[CLOSE_DOUBLE_WINDOWS]->setEnabled(false);
		actions[CLOSE_ALL_WINDOWS]->setEnabled(false);
	}
	else
	{
		actions[CLOSE_DOUBLE_WINDOWS]->setEnabled(true);
		actions[CLOSE_ALL_WINDOWS]->setEnabled(true);
		windowmenu->insertSeparator();
		windowlist = workspace->windowList();
		for (i=0; i<windowlist.count(); i++ )
		{
			id=windowmenu->insertItem(QString("&%1 ").arg(i+1)+windowlist.at(i)->caption(), this, SLOT( WindowMenuActivated(int) ) );
			windowmenu->setItemParameter( id, i );
			windowmenu->setItemChecked(id, workspace->activeWindow() == windowlist.at(i) );
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Is called if a window was selected in the popup menu
void YMainWindow::WindowMenuActivated(int id )
{
	QWidget* window = workspace->windowList().at(id);
	if (window)
		window->setFocus();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::OpenQTAbout()
{
	QMessageBox::aboutQt(this);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::OpenHelpWindow()
{
	YHelpFrame::OpenHelpWindow();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function will be called after InitInstance() by the application
int YMainWindow::Show()
{
	if ( MAINWINDOW_OPEN_MAXIMIZED )
		showMaximized();
    else
		show();
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// After resizing the main window this function will be called
void YMainWindow::resizeEvent(QResizeEvent *)
{
	int a;
	// correct size of horizontal splitter
	QValueList<int>	widthlist;
	QSize t=splitth->size();										// get current size
	widthlist.append( t.width()*1/5 );
	widthlist.append( t.width()*4/5 );
	splitth->setSizes( widthlist );	//resize( 300, t.height() );//t.width()/5, 0 );

	// correct size of vertical splitter
	widthlist=splittv->sizes();
	a=widthlist[0]+widthlist[1];
	widthlist[0]=a*4/5;
	widthlist[1]=a*1/5;
	splittv->setSizes( widthlist );	//resize( 300, t.height() );//t.width()/5, 0 );

//	splittv->resize( e->size() );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::CloseWindow()
{
	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		window->close();
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::NewWindow()
{
	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		YMDIWindow::OpenNewWindow( &window->GetWindowArgs(), true);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::NewFullScreenWindow()
{
	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		YMDIWindow::OpenNewWindow( &window->GetWindowArgs(), true, true);
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::ZoomIn()
{
	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		YObject *object=window->GetWindowArgs().GetObject();
		if (object && object->GetActions() )
		{
			object->GetActions()->DoWindowCommand(object, YActions::ZOOM_IN, window);
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::ZoomOut()
{
	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		YObject *object;
		object=window->GetWindowArgs().GetObject();
		if (object && object->GetActions() )
		{
			object->GetActions()->DoWindowCommand(object, YActions::ZOOM_OUT, window );
		}
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMainWindow::ExecuteCurrentEnvironment()
{
	YMDIWindow	*window=(YMDIWindow*)workspace->activeWindow();
	if ( window )
	{
		YObject *object;
		object=application->GetActionHandler()->GetCurrentObject();
		if (object && object->GetActions() )
		{
			object->GetActions()->DoTreeCommand(object, YActions::EXECUTE );
		}
	}
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Find window in window list.
YMDIWindow* YMainWindow::FindWindow(const QString &title, YObject *object)
{
	QWidgetList	windows = YApplication::GetMainWindow()->GetWindowList();
	YMDIWindow	*mdiwindow;
	int			i;

	//if ( title )
	{
		for (i=0; i<int(windows.count()); i++ )
		{
			mdiwindow=(YMDIWindow*)windows.at(i);
			//if ( mdiwindow->caption()==title )
			if ( mdiwindow->CheckTitle(title) && (mdiwindow->GetWindowArgs().GetObject()==object)  )
				return mdiwindow;
		}
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Find window in window list.
//bool YMainWindow::ForegroundWindow(const YObject *object, const QString *title)
bool YMainWindow::ForegroundWindow(const QString &title, YObject *object)
{
	YMDIWindow *window;
	window=YMainWindow::FindWindow(title, object);
	if ( window )
	{
		window->setFocus();
		window->setActiveWindow();
		return true;
	}
	else
		return false;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Find window in window list.
void YMainWindow::CloseDoubleWindows()
{
	QWidgetList	windows;
	YMDIWindow *window;
	int			i,k;

repeat:
	windows = YApplication::GetMainWindow()->GetWindowList();
	for ( k=0; k<int(windows.count()); k++)
	{
		window=(YMDIWindow*)windows.at(k);

		//if ( window->GetObject() )
		{
			for ( i=k+1; i<int(windows.count()); i++ )
			{
				if ( ((YMDIWindow*)windows.at(i))->GetTitle() == window->GetTitle() )
				{
					((YMDIWindow*)windows.at(i))->close(true);		// window list may change
					goto repeat;
				}
			}
		}
	}
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
QWidgetList	YMainWindow::GetWindowList()
{
	QWidgetList	list;
	if (this)
	{
		list=workspace->windowList();
	}
	else
		qWarning("YMainWindow::GetWindowList() called but this is zero!");
	return list;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YMDIWindow::~YMDIWindow()
{
	if (mainframe)
		delete mainframe;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMDIWindow::Show()
{
	if ( !mainwindow || mainwindow->GetMDIWorkSpace()->windowList().isEmpty() && MDI_OPEN_MAXIMIZED )
		showMaximized();
	else
		show();
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Open the window associated with the object. If it already exist and multiple is false
//! it will be pushed in the foreground (default)
int YMDIWindow::OpenNewWindow(YFrameArgs *args, bool multiple, bool fullscreen)
{
	YMDIWindow	*window=0;
	int retvalue=YASAE_ERROR_IN_ARGUMENTS;

	// check if window is already present
	if ( args && args->GetTitle())
	{
		if (  multiple || ! YMainWindow::ForegroundWindow( args->GetTitle(), args->GetObject() ) )
		{
			if ( fullscreen )
				window=new YMDIWindow( 0, args->GetTitle() );
			else
				window=new YMDIWindow(YApplication::GetMainWindow(), args->GetTitle() );
			if ( window )
			{
				retvalue=window->InitInstance(*args);
			}
		}
	}
	else
		qWarning("YMDIWindow::OpenNewWindow() called with invalid arguments!");
	if ( retvalue )
	{
		if (window)
			delete window;
		else
		{
			if (args)
				delete args;
		}
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMDIWindow::InitInstance(YFrameArgs &windowargs)
{
	int retvalue=YASAE_ERROR_IN_ARGUMENTS;

	mainframe=windowargs.CreateFrame(this);
	if ( mainframe )
	{
		connect( mainframe, SIGNAL( CloseFrame() ), this, SLOT(close() ) );
		connect( mainframe, SIGNAL( TitleChanged(const QString &, YObject *) ), this, SLOT(SetCaption(const QString &, YObject *) ) );
//		connect( mainframe, SIGNAL( StatusMessage(const QString&, int) ), (const class QObject*)mainwindow->statusBar(), SLOT( message(const QString&, int )) );
		retvalue = mainframe->InitInstance();
		if ( ! retvalue )
		{
			retvalue=YASAE_ERROR_IN_ARGUMENTS;
			if ( mainframe->GetMainWidget() )
			{
				setCentralWidget( mainframe->GetMainWidget() );
				setBaseSize( QSize( MDI_WIDTH, MDI_HEIGHT ) );
				SetCaption( windowargs.GetTitle(), windowargs.GetObject() );
				YApplicationFigures::SetIcon(*this, YApplicationFigures::ICON_MDIWINDOW );
				resize( MDI_WIDTH, MDI_HEIGHT );
				mainframe->PostCreateWindow(this);
				retvalue=Show();
			}
		}
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMDIWindow::SetCaption(const QString &newname, YObject *object)
{
	// compare only real window name ala "About yasa" and not "About yasa:1"
	//if (newname!=caption().left( GetTitle().length() ) )
	if ( ! CheckTitle( newname ) )
	{
		int		count=0;
		QString	captionstr;
		while (1)
		{
			if (count)
			{
				captionstr=newname+":%1";
				captionstr=captionstr.arg(count+1);
			}
			else
				captionstr=newname;
			if ( ! YMainWindow::FindWindow(captionstr, object) )
				break;
			count++;
		}
		setCaption(captionstr);
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YMDIWindow::keyPressEvent(QKeyEvent *event)
{
	if ( event->key() == Key_Escape )
	{
		close();
	}
	else
		event->ignore();
		//QWidget::keyPressEvent(event);
}



