//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA application main window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/mainwindow.h,v 1.4 2003/01/24 15:47:41 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about the whole window (project manager, output, toolbars etc)
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_MAINWINDOW_INCLUDE
#define	YASAGUI_MAINWINDOW_INCLUDE

#include <qapplication.h>
#include <qmainwindow.h>
#include <qworkspace.h>
#include <qaction.h>
#include <qtoolbar.h>
#include <yasagui/yasagui.h>
#include <qsplitter.h>
#include "translations.h"
#include "config.h"
#include "frame.h"
//#include "settings.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YOutputView;
class YApplication;
class YMDIWindow;
class YWorkSpaceTreeView;
class YWorkSpace;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YMainWindow : public QMainWindow
{
	enum{	NEW_WORKSPACE=0		, OPEN_WORKSPACE	, SAVE_WORKSPACE		, SAVE_WORKSPACE_AS		, CLOSE_WORKSPACE, EXIT	,
			CLOSE_WINDOW		, NEW_WINDOW		, NEW_FULLSCREEN_WINDOW	, CLOSE_DOUBLE_WINDOWS	,
			CLOSE_ALL_WINDOWS	, CASCADE_WINDOWS	, TILE_WINDOWS			, ABOUT_QT				, ABOUT_APP				,
			EXECUTE				, ZOOM_IN			, ZOOM_OUT				, MAX_ACTIONS };
	Q_OBJECT
	QSplitter			*splittv;
    QSplitter			*splitth;
	QWorkspace			*workspace;
	YApplication		*application;
	YWorkSpaceTreeView	*workspacetreeview;
	YOutputView			*outputview;
	QArray<QAction*>	actions;
	QPopupMenu			*filemenu;
	QPopupMenu			*environmentmenu;
	QPopupMenu			*windowmenu;
	QPopupMenu			*helpmenu;
	QToolBar			*workspacetoolbar;
	QToolBar			*environmenttoolbar;

	int					CreateActions();
	int					CreateMenu();
	int					CreateToolBar();
	int					CreateStatusBar();
public:
						YMainWindow( QWidget* parent = 0, const char* name = 0, WFlags f = WType_TopLevel ) : QMainWindow( parent, name, f ),
						splittv(0),
						splitth(0),
						workspace(0),
						application(0),
						workspacetreeview(0),
						outputview(0),
						actions(MAX_ACTIONS),
						filemenu(0),
						environmentmenu(0),
						windowmenu(0),
						helpmenu(0),
						workspacetoolbar(0),
						environmenttoolbar(0)
						{ }
						~YMainWindow();
	int					InitInstance(YApplication *app);
	int					Show();
	static YMDIWindow*	FindWindow(const QString &title, YObject *object);
	static bool			ForegroundWindow(const QString &title, YObject *object);
	QWidgetList			GetWindowList();
	QWorkspace*			GetMDIWorkSpace()				{ return this ? workspace :0;		}

public slots:
	void				CloseWindow();
	void				NewWindow();
	void				NewFullScreenWindow();
	void				UpdateWindowsMenu();									//! update window menu with current window list
	void				OpenQTAbout();
	void				OpenHelpWindow();
	void				CloseDoubleWindows();
	void				CloseAllWindows();
	void				WindowMenuActivated(int id );
	void				WorkSpaceActionsEnabled(YWorkSpace *workspace);
	void				WorkSpaceActionsDisabled(YWorkSpace *workspace);
	void				ZoomIn();
	void				ZoomOut();
	//void				ExecuteEnvironment();
	void				ExecuteCurrentEnvironment();
protected:
	virtual void		resizeEvent(QResizeEvent *e);
signals:
	void				StatusMessage(const QString&, int );					// print given text to Statusbar
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class is a base for every kind of child windows
class YMDIWindow : public QMainWindow
{	Q_OBJECT
protected:
	YMainWindow			*mainwindow;											//! This value contains the ptr the main window
	YFrame				*mainframe;
	virtual void		keyPressEvent (QKeyEvent *event);
public:					YMDIWindow(YMainWindow *parent, const QString title) :
						QMainWindow( parent ? parent->GetMDIWorkSpace() : 0, title.latin1(), WDestructiveClose),
						mainwindow(parent),
						mainframe(0)
						{ }
						~YMDIWindow();
	virtual int			InitInstance(YFrameArgs &args);
	virtual int			Show();
	const QString&		GetTitle() const				{	return GetWindowArgs().GetTitle();		}
	YFrameArgs&			GetWindowArgs()	const			{	return mainframe->GetFrameArgs();		}
	static int			OpenNewWindow(YFrameArgs *args, bool multiple=false, bool fullscreen=false);
	//! checks if "About Yasa"		== "About Yasa:1"
	//! checks if "Simulator ELLF"	== "Simulator ELLF - Timing diagram"
	bool				CheckTitle(const QString &name) const
						{
							if ( caption().length() > GetTitle().length() )
								if ( caption().at( GetTitle().length() ) != QChar(':') )
									return false;
							return name==caption().left( GetTitle().length() );
						}
	YFrame*				GetMainFrame() const			{	return mainframe;						}
public slots:
	void				SetCaption(const QString &newname, YObject *object);
signals:
	void				StatusMessage(const QString&, int );				//! print given text to Statusbar
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_MAINWINDOW_INCLUDE
