//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project, mutex support
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/mutex.cpp,v 1.3 2003/01/23 09:18:36 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "mutex.h"
#include "actionhandler.h"
#include "parser.h"
#include "project.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YMutexSet::YMutexSet() :
YSet(YPC_MUTEX, MSGP_DEFAULT_MUTEXNAME, YObjectArgs( MSGP_CAPTION_MUTEXES, YActionHandler::GetActions(YActionHandler::MUTEXSET_ACTIONS)))
{
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YMutex::YMutex(const YObjectArgs &args) :
YObject(args)
//priorityceiling(0),
//preemptceiling(0)
{
	connect( &protocol			, SIGNAL( UpdateObject() ), this, SLOT( ObjectUpdated() ) );
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read mutex specific configuration data from instance and set to stream
int YMutex::SetConfig(YInputParser &parser)
{
	QString name;
	if ( parser.CheckSectionStart(YPC_PROTOCOL) )
		return parser.ReadContentObject(protocol );
/*	if ( parser.CheckSectionStart(YPC_PRIORITYCEILING) )
		return parser.ReadContentObject(priorityceiling );
	if ( parser.CheckSectionStart(YPC_PREEMPTCEILING) )
		return parser.ReadContentObject(preemptceiling );
*/	return YObject::SetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Read mutex specific configuration data from stream and set to instance
int YMutex::GetConfig(YOutputParser &parser)
{
	int	retvalue;
	retvalue=parser.WriteContentObject(YPC_PROTOCOL, protocol);
		if ( retvalue )	return retvalue;
/*	retvalue=parser.WriteContentObject(YPC_PRIORITYCEILING, priorityceiling);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_PREEMPTCEILING, preemptceiling);
		if ( retvalue )	return retvalue;
*/	return YObject::GetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
void YMutex::SetProtocol(const YASA_MUTEXPROTOCOL &p)
{
	if ( p!=protocol )
	{
		protocol=p;
		ObjectUpdated();
	}
}
*/


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YASA_PRIORITY YMutex::GetPriorityCeiling() const
//YASA_TIME YMutex::GetPriorityCeiling() const
{
	YASA_PRIORITY	prio;
	YTask			*task;
	unsigned int	i,j;

	for( i=0; i<GetTaskSet().GetCount(); i++)
	{
		task=((YTask*)GetTaskSet().At(i));

		// does the task use this mutex ?
		for ( j=0; j<task->GetResourceActionSet().GetCount(); j++)
		{
			if ( this == (YMutex*) ((YResourceAction*)task->GetResourceActionSet().At(j))->GetMutex() )
			{
				// has task priority higher than "priorityceiling"
				if ( task->GetPriority() > prio )
					prio=((YTask*)GetTaskSet().At(i))->GetPriority();
				break;
			}
		}
	}
//	return prio.GetContent();
	return prio;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool YMutex::IsUsed() const
{
	unsigned int	i,j;
	YTask			*task;
	for( i=0; i<GetTaskSet().GetCount(); i++)
	{
		task=((YTask*)GetTaskSet().At(i));

		// does the task use this mutex ?
		for ( j=0; j<task->GetResourceActionSet().GetCount(); j++)
		{
			if ( this == (YMutex*) ((YResourceAction*)task->GetResourceActionSet().At(j))->GetMutex() )
			{
				return true;
			}
		}
	}
	return false;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YTaskSet& YMutex::GetTaskSet() const
{
	return ((YProject*)GetSet()->GetParent())->GetTaskSet();
}
