//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA mutex window code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/mutexwindow.cpp,v 1.3 2003/01/23 09:18:36 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include "mutexwindow.h"


#include "taskwindow.h"
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMutexFrame::InitInstance()
{
	YMutex				*mutex=(YMutex*)GetObject();
	QFrame				*scrollframe	= new QFrame( GetMainWidget() );
	QVBoxLayout			*vboxlayout		= new QVBoxLayout(scrollframe, MDI_SCROLLVIEW_BORDER );


	QGroupBox	*group1		= new QGroupBox( 5, Horizontal, MSGP_WIDGET_PROPERTIES	, scrollframe );
	QGroupBox	*group2		= new QGroupBox( 1, Horizontal, MSGP_MUTEX_CEILINGS		, scrollframe );
	QGroupBox	*group3		= new QGroupBox( 1, Horizontal, MSGP_WIDGET_DESCRIPTION	, scrollframe );

	vboxlayout->addWidget( group1 );
	vboxlayout->addWidget( group2 );
	vboxlayout->addWidget( group3 );
	group1->setAlignment(AlignHCenter);
	group2->setAlignment(AlignHCenter);
	group3->setAlignment(AlignHCenter);

	// upper group
	QVBox *labels =	new QVBox( group1 );
	QVBox *widgets=	new QVBox( group1 );

	new	QLabel(MSGP_WIDGET_NAME				, labels );
	new	QLabel(MSGP_MUTEX_PROTOCOL			, labels );

	new YNameWidget(				mutex	, widgets );
	new YProtocolWidget(			mutex	, widgets );

	// middle group
	YTaskSet	*set=0;
	if ( mutex->GetSet() && mutex->GetSet()->GetParent() && &((YProject*)mutex->GetSet()->GetParent())->GetTaskSet() )
		set=&((YProject*)mutex->GetSet()->GetParent())->GetTaskSet();
	if ( set )
	{
		YMutexTimesListView	*mutexview=new YMutexTimesListView( set, group2, mutex );
		if ( mutexview )
			mutexview->InitInstance();
	}


	// lower group
	new YDescriptionWidget( mutex, group3 );

	scrollview.addChild(scrollframe);
	scrollview.setResizePolicy(QScrollView::AutoOneFit);

	return YObjectFrame::InitInstance();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMutexTimesListView::InitInstance()
{
	addColumn(MSGP_MUTEX_PRIORITY);
	addColumn(MSGP_MUTEX_PERIOD);
	addColumn(MSGP_MUTEX_DEADLINE);
	addColumn(MSGP_MUTEX_LAXITY);
	return YListView::InitInstance();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMutexTimesListView::CreateSetItems()
{
	YTask				*task;

	clear();
	if ( set )													// check taskset of project
	{
		for (unsigned int i=0; i<set->GetCount(); i++)
		{
			task=(YTask*)set->At(i);
			if ( task )
			{
				NewChildObject(task);
			}
		}
	}
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! object is ptr to YTask
void YMutexTimesListView::NewChildObject(YObject *object)
{
	connect ( object, SIGNAL( UpdateObject(YObject *) ), this, SLOT( UpdateTaskProperties(YObject *) ) );
	UpdateTaskProperties(object);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Find listviewitem of given task
YMutexTimeListViewItem* YMutexTimesListView::_FindItem(YTask *task)
{
	YMutexTimeListViewItem	*item=(YMutexTimeListViewItem*)firstChild();
	while( item )										// parse mutex listview and search task entry
	{
		if ( item->GetObject() == task )				// task already exists
			break;
		item=(YMutexTimeListViewItem*)item->itemBelow();
	}
	return item;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is used more often than the name explains because we have to  object has type:
//! YResourceAction and is new created or should be updated
void YMutexTimesListView::UpdateTaskProperties(YObject *object)
{
	YTask	*task=(YTask*)object;
	if ( task )
	{
		// Create listentry for this task if there this no entry and the current mutex is used by this task
		YMutexTimeListViewItem *item=_FindItem(task);

		// determine if the task is this mutex or not
		YResourceActionSet	*actionset=&task->GetResourceActionSet();
		YResourceAction		*action;
		bool				mutexused=false;

		for (unsigned int i=0; i<actionset->GetCount(); i++)
		{
			action=(YResourceAction*)actionset->At(i);
			if ( action->GetMutex() == mutex )			// mutex found in resource action list
			{
				mutexused=true;
				break;
			}
		}

		// allocate listview item only if task contains resource actions for this mutex
		if ( ! item && mutexused)								// task not found -> allocate if needed
		{
			item=CreateNewItem(0, task, item);
			connect ( &task->GetResourceActionSet(), SIGNAL( ItemRemoved(YObject *) ), this, SLOT( ResourceActionRemoved(YObject *) ) );
		}

		if ( item )
		{
			if ( mutexused )
				item->UpdateObject(task);
			else
				delete item;
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A resource action was removed
void YMutexTimesListView::ResourceActionRemoved(YObject *resourceaction)
{
	if ( resourceaction && resourceaction->GetSet() && resourceaction->GetSet()->GetParent() )
	{
		YTask				*task=(YTask*) resourceaction->GetSet()->GetParent();
		YResourceAction		*action;
		YResourceActionSet	*actionset=&task->GetResourceActionSet();
		bool				mutexused=false;

		// determine if the task is using the mutex anymore
		for (unsigned int i=0; i<actionset->GetCount(); i++)
		{
			action= ((YResourceAction*)actionset->At(i));
			if ( action!=resourceaction && action->GetMutex() == mutex )			// mutex found in resource action list
			{
				mutexused=true;
				break;
			}
		}

		// remove task entry from listview if this task does not use this mutex anymore
		if ( ! mutexused )
		{
			YMutexTimeListViewItem *item=_FindItem( task );
			if (item)
				delete item;
		}
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YMutexTimeListViewItem::InitInstance()
{
	int	retvalue;
	retvalue=YListViewItem::InitInstance();
	DisableInlineRename();
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Update the contents of this item
void YMutexTimeListViewItem::UpdateObject(YObject *o)
{
	if ( QString::compare( o->GetName(), text(0) ) )					// task name
		setText(0, object->GetName() );
	if ( text(1) != ((YTask*)object)->GetPriority().GetVisibleContentString() ) //priority!=((YTask*)object)->GetPriority() )			// priority
	{
		setText(1, ((YTask*)object)->GetPriority().GetVisibleContentString() );
	}
	if ( text(2) != ((YTask*)object)->GetPeriod().GetVisibleContentString() ) //period != ((YTask*)object)->GetPeriod() )
	{
		setText(2, ((YTask*)object)->GetPeriod().GetVisibleContentString() );
	}
	if ( text(3) != ((YTask*)object)->GetDeadline().GetVisibleContentString() )
	{
		setText(3, ((YTask*)object)->GetDeadline().GetVisibleContentString() );
	}
	if ( text(4) != ((YTask*)object)->GetLaxity().GetVisibleContentString() )
	{
		setText(4, ((YTask*)object)->GetLaxity().GetVisibleContentString() );
	}
}
