//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA output
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/output.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_OUTPUT_INCLUDE
#define	YASAGUI_OUTPUT_INCLUDE

#include <qobject.h>
#include <qlistview.h>
//#if QT_VERSION<300
 #include <qtextview.h>
//#else
// #include <qtextedit.h>
//#endif
#include <stdio.h>
#include "translations.h"
#include "mainwindow.h"



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Warning levels
#define YWARNING_LOW			-1
#define YWARNING_MEDIUM			0
#define YWARNING_HIGH			1
#define YWARNING_DEFAULT		YWARNING_MEDIUM


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YOutput : public QObject
{	Q_OBJECT
public:
	int				InitInstance()										{	return 0;					}
	void			NewLine()											{	emit NewMessage("\n");		}
	void			Important(const QString &text=QString::null)		{	emit NewImportant(text);	}
	void			Command(const QString &text=QString::null)			{	emit NewCommand(text);		}
	void			Comment(const QString &text=QString::null)			{	emit NewComment(text);		}
	void			Message(const QString &text=QString::null)			{	emit NewMessage(text);		}
	void			Warning(const QString &text=QString::null, int level=YWARNING_MEDIUM) {	emit NewWarning(text,level);}
	void			Error(const QString &text=QString::null)			{	emit NewError(text);		}
signals:
	void			NewImportant(const QString &text);
	void			NewComment(const QString &text);
	void			NewCommand(const QString &text);
	void			NewMessage(const QString &text);
	void			NewWarning(const QString &text, int level);
	void			NewError(const QString &text);
};




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class provides support for output to scrollview widget
//class YOutputView : public QTextView
class YOutputView : public QTextView
{	Q_OBJECT
	YOutput			*output;
	QString			buffer;
	//! Minimum warning level. All warning messages with a lower level will not be printed
	int				minlevel;
public:				YOutputView(QWidget *widget) :
					QTextView(MSGP_OUTPUTVIEW_WELCOME, MSGP_OUTPUT_TITLE, widget),
					output(0),
					minlevel(YWARNING_MEDIUM)
					{
						// The event filter is needed to receive events occured by pressing the right button of the mouse
						widget->installEventFilter( this );
					}
	int				InitInstance(YOutput *output);
	void			_print(const QString &str);
protected:
	bool			eventFilter(QObject *o, QEvent *e);
	virtual QPopupMenu *
					createPopupMenu( const QPoint & pos );
public slots:
	void			PrintImportant(const QString &text)					{	_print( QString("<font size=5 color=\"#e00000\">%1</font>").arg(text) );	}
	void			PrintCommand(const QString &text)					{	_print( text );																}
	void			PrintComment(const QString &text)					{	_print( QString("<font color=\"#005000\">%1</font>").arg(text) );			}
	void			PrintMessage(const QString &text)					{	_print( text );																}
	void			PrintWarning(const QString &text, int level)		{	if (level>=minlevel)
																				_print( QString("<font color=\"#0000e0\">Warning: %1</font>").arg(text) );	}
	void			PrintError(const QString &text)						{	_print( QString("<font color=\"#e00000\">Error: %1</font>").arg(text) );	}
	void			ClearView()											{	setText(QString::null);
																			buffer=QString::null;														}
	void			PopupMenuActivated(int index);
	YOutput*		GetOutput() const									{	return output;	}
	const QString &	GetText();
};





//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YOutputWindowFrameArgs : public YFrameArgs
{	YOutputView			*oritextview;
public:					YOutputWindowFrameArgs( YOutputView *oritextview );
	YOutputView*		GetOutputView() const							{	return oritextview; }
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing a the output messages in a full screen window
class YOutputWindowFrame : public YFrame
{	Q_OBJECT
protected:
	YOutputView			outputview;
public:					YOutputWindowFrame(QWidget *parent, YFrameArgs &args) :
						YFrame(args),
						outputview(parent)
						{}
	virtual int			InitInstance()							{	outputview.InitInstance( ((YOutputWindowFrameArgs&)GetFrameArgs()).GetOutputView()->GetOutput() );
																	outputview.setText( MSGP_OUTPUTVIEW_WELCOME );
																	outputview.append( ((YOutputWindowFrameArgs&)GetFrameArgs()).GetOutputView()->GetText() );
																	return YFrame::InitInstance();	}
	virtual QWidget*	GetMainWidget()								{ return &outputview;	}
	static YFrame*		CreateOutputWindowFrame(QWidget *parent, YFrameArgs &args)
						{
							return new YOutputWindowFrame(parent, args );
						}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class provides console output support
class YConsoleOutput : public QObject
{	Q_OBJECT
public:				YConsoleOutput(YOutput *output)
					{
						connect( output, SIGNAL( NewMessage(const QString &) ), this, SLOT( PrintMessage(const QString &) ) );
						connect( output, SIGNAL( NewWarning(const QString &) ), this, SLOT( PrintWarning(const QString &) ) );
						connect( output, SIGNAL( NewError(const QString &) ), this, SLOT( PrintError(const QString &) ) );
					}
	int				InitInstance();
public slots:
	void			PrintMessage(const QString &text)
					{
						if ( text.length() )
							printf( text.latin1() );
						else
							printf("\n");
					}
	void			PrintWarning(const QString &text)
					{
						if ( text.length() )
							printf( text.latin1() );
						else
							printf("\n");
					}
	void			PrintError(const QString &text)
					{
						if ( text.length() )
							printf( text.latin1() );
						else
							printf("\n");
					}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_OUTPUT_INCLUDE
