//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA plugin's
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/plugins.cpp,v 1.3 2003/01/23 09:18:37 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qdir.h>
#include "plugins.h"
#include "parser.h"
#include "application.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Scan sub directory for plugin's
int YPlugInManager::InitInstance()
{
	int retvalue;

	// Initialize scheduler plugin's
	retvalue=schedulerplugins.InitInstance();
	if ( retvalue )
		return retvalue;

	// Initialize executive plugin's
	retvalue=executiveplugins.InitInstance();
	if ( retvalue )
		return retvalue;
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Scan sub directory for plugin's
int YPlugInList::InitInstance()
{
	int					retvalue=0;
	QDir				dir(subdirectory);
	const QFileInfoList	*list;
	const QFileInfo		*finfo;
	QString				tmp;

	list=dir.entryInfoList(QDir::Name | QDir::Dirs);
	if ( list )
	{
		QListIterator<QFileInfo>	iterator(*list);
		for ( ; iterator.current(); ++iterator)
		{
			finfo=iterator.current();
			tmp=QDir::cleanDirPath(finfo->fileName() );
			if ( tmp.length() && (tmp[0]!='.') )
			{
				YPlugIn	*object = (YPlugIn*) CreateNewItem( tmp );
				if ( ! object )
					return YASAE_OUT_OF_MEMORY;
				object->SetShortName(tmp);
				retvalue=object->InitInstance( finfo->filePath(), doctype );
				if ( retvalue && retvalue!=YASAE_COULD_NOT_OPEN_FILE )
					return retvalue;
				else
					retvalue=0;
			}
		}
	}
	return retvalue;
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YPlugIn::InitInstance(const QString &p, const QString &dt)
{
	int						retvalue;
	YXMLInputParser			parser(dt);
	QString					filename;
	QFileInfo				finfo(p, GetName() );

	path=p;
	filename=finfo.filePath();
	filename+=YASA_POSTFIX;
	retvalue=parser.Parse( filename, *this);
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YPlugIn::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_SUPPORT_CALC_PREEMPTIONTIME) )
		return parser.ReadContentState(support_calcpreemptiontime );
	if ( parser.CheckSectionStart(YPC_SUPPORT_TIMER_TICK) )
		return parser.ReadContentState(support_timertick);
	if ( parser.CheckSectionStart(YPC_MAX_CPU) )
		return parser.ReadContentObject(maxcpu);
	if ( parser.CheckSectionStart(YPC_MIN_TIMER_TICK) )
		return parser.ReadContentObject(min_timertick);
	return YObject::SetConfig(parser);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YPlugIn::GetConfig(YOutputParser &parser)
{
	int retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_CALC_PREEMPTIONTIME, support_calcpreemptiontime);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_TIMER_TICK, support_timertick);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_MAX_CPU, maxcpu);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_MIN_TIMER_TICK, min_timertick);
		if ( retvalue )	return retvalue;
	return YObject::GetConfig(parser);
}






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSchedulerPlugIn::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_SCHEDULER_TYPE) )
		return parser.ReadContentObject(type);
	return YPlugIn::SetConfig(parser);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YSchedulerPlugIn::GetConfig(YOutputParser &parser)
{
	int retvalue;
	retvalue=parser.WriteContentObject(YPC_SCHEDULER_TYPE, type);
		if ( retvalue )	return retvalue;
	return YPlugIn::GetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YExecutivePlugIn::SetConfig(YInputParser &parser)
{
	if ( parser.CheckSectionStart(YPC_GCC_COMPILER_OPTIONS) )
		return parser.ReadContentObject(gcccompileroptions);
	if ( parser.CheckSectionStart(YPC_SUPPORT_SOURCE_CODES) )
		return parser.ReadContentState(sourcecodesupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_UNLIMITED_CPUS) )
		return parser.ReadContentState(unlimitedcpusupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_CHANGEABLE_CPUS) )
		return parser.ReadContentState(changeablecpusupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_RESUME_TIMES) )
		return parser.ReadContentState(resumetimesupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_RESOURCE_ACTIONS) )
		return parser.ReadContentState(resourceactionsupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_EMERGENCY_THREADS) )
		return parser.ReadContentState(emergencythreadsupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_CANCEL_THREADS) )
		return parser.ReadContentState(cancelthreadsupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_SCHEDULING_TIME) )
		return parser.ReadContentState(schedulingtimesupport);
	if ( parser.CheckSectionStart(YPC_SUPPORT_SYNCHRONIZATION_OFFSET) )
		return parser.ReadContentState(synchronizationoffset);
	if ( parser.CheckSectionStart(YPC_MAP_IDLE_NAME) )
		return parser.ReadContentObject(idlename);
	return YPlugIn::SetConfig(parser);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YExecutivePlugIn::GetConfig(YOutputParser &parser)
{
	int retvalue;
	retvalue=parser.WriteContentState(YPC_GCC_COMPILER_OPTIONS, gcccompileroptions);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_SOURCE_CODES, sourcecodesupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_UNLIMITED_CPUS, unlimitedcpusupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_CHANGEABLE_CPUS, changeablecpusupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_RESUME_TIMES, resumetimesupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_RESOURCE_ACTIONS, resourceactionsupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_EMERGENCY_THREADS, emergencythreadsupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_CANCEL_THREADS, cancelthreadsupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_SCHEDULING_TIME, schedulingtimesupport);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentState(YPC_SUPPORT_SYNCHRONIZATION_OFFSET, synchronizationoffset);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentObject(YPC_MAP_IDLE_NAME, idlename );
		if ( retvalue )	return retvalue;
	return YPlugIn::GetConfig(parser);
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Find the Executive plugin list within the application
YSchedulerPlugIns& YSchedulerPlugIns::GetSchedulerPlugIns()
{
	return YApplication::GetPlugInManager()->GetSchedulerPlugIns();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Find the Executive plugin list within the application
YExecutivePlugIns& YExecutivePlugIns::GetExecutivePlugIns()
{
	return YApplication::GetPlugInManager()->GetExecutivePlugIns();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function generates a feature list of the execute
QStringList YExecutivePlugIn::GetFeatureList() const
{
	QStringList	featurelist;
	QString		feature;

	// Support of own source codes
	if ( GetSourceCodeSupport() )
		featurelist+=MSGP_EP_SUPPORT_SOURCES.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_SOURCES.arg(MSGP_ERROR_NO);

	// Support of unlimited number of cpu's
	if ( GetEmergencyThreadSupport() )
		featurelist+=MSGP_EP_SUPPORT_UNLIMITED_CPUS.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_UNLIMITED_CPUS.arg(MSGP_ERROR_NO);

	// Support of changeable cpu's
	if ( GetChangeableCPUSupport() )
		featurelist+=MSGP_EP_SUPPORT_CHANGEABLE_CPUS.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_CHANGEABLE_CPUS.arg(MSGP_ERROR_NO);

	// Support of resource actions
	if ( GetResourceActionsSupport() )
		featurelist+=MSGP_EP_SUPPORT_RESOURCE_ACTIONS.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_RESOURCE_ACTIONS.arg(MSGP_ERROR_NO);

	// Support of resume times
	if ( GetResumeTimeSupport() )
		featurelist+=MSGP_EP_SUPPORT_RESUME_TIMES.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_RESUME_TIMES.arg(MSGP_ERROR_NO);

	// Support of emergency threads
	if ( GetEmergencyThreadSupport() )
		featurelist+=MSGP_EP_SUPPORT_EMERGENCY_THREADS.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_EMERGENCY_THREADS.arg(MSGP_ERROR_NO);

	// Support of cancel thread
	if ( GetCancelThreadSupport() )
		featurelist+=MSGP_EP_SUPPORT_CANCEL_THREADS.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_CANCEL_THREADS.arg(MSGP_ERROR_NO);

	// Support of scheduling times
	if ( GetSchedulingTimeSupport() )
		featurelist+=MSGP_EP_SUPPORT_SCHEDULING_TIME.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_SCHEDULING_TIME.arg(MSGP_ERROR_NO);

	// Support of synchronization offset
	if ( GetSchedulingTimeSupport() )
		featurelist+=MSGP_EP_SUPPORT_SYNCHRONIZATION_OFFSET.arg(MSGP_ERROR_YES);
	else
		featurelist+=MSGP_EP_SUPPORT_SYNCHRONIZATION_OFFSET.arg(MSGP_ERROR_NO);

	return featurelist;
}
