//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project manager
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/project.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_PROJECT_INCLUDE
#define	YASAGUI_PROJECT_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasagui/yasagui.h>
#include "translations.h"
#include "config.h"
#include "task.h"
#include "mutex.h"
#include "environment.h"
#include "sources.h"
#include "results.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProject : public YObject
{
	Q_OBJECT
	YTaskSet				taskset;
	YMutexSet				mutexset;
	YEnvironmentSet			environmentset;
	YProjectLogFile			projectlogfile;
	YSourceSet				sourceset;
	YHeaderSet				headerset;
	YResults				results;
	YString					projectdir;							//! relative to workspace if possible

	YASA_MAXCPU				maxcpu;								//! number of cpu in project
	YASA_CALCPREEMPTIONTIME	calcpreemptiontime;					//! settings for property calcpreemption time
	YASA_TICK				timertick;							//! tick value in this object
	YTIME					endtime;
public:						YProject(const YObjectArgs &args);
	YTaskSet &				GetTaskSet()						{ return taskset;			}
	YMutexSet &				GetMutexSet()						{ return mutexset;			}
	YEnvironmentSet &		GetEnvironmentSet()					{ return environmentset;	}
	YProjectLogFile &		GetProjectLogFile()					{ return projectlogfile;	}

	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
	const YString &			GetProjectDir()						{ return projectdir;		}
	YSourceSet &			GetSourceSet()						{ return sourceset;			}
	YWorkSpace *			GetWorkSpace()						{ return ((YWorkSpace*)GetSet()->GetParent()); }
	QString					CopyProjectDir(bool absdir=false);
//	QString					GetProjectDir(bool absdir=false);
	const YASA_CPU &		GetMaxCPU() const					{ return maxcpu;			}
	const YASA_TICK &		GetTimerTick() const				{ return timertick;			}
	const YTIME &			GetEndTime() const					{ return endtime;			}
	const YASA_CALCPREEMPTIONTIME &
							GetCalcPreemptionTime() const		{ return calcpreemptiontime;}
	void					SetProjectDir(const QString &pd);
	void					SetMaxCPU(int c)					{ maxcpu=c;					}
	void					SetTimerTick(YASA_TIME t)			{ timertick=t;				}
	void					SetEndTime(YASA_TIME t)				{ endtime=t;				}
	void					SetCalcPreemptionTime(bool c)		{ calcpreemptiontime=c;		}
	void					CleanupProject();
	void					RemoveProjectDirectory();
	int						EvaluateLogFiles();
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProjectSet : public YSet
{
public: 					YProjectSet();
	void					CleanupProjects();
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)	{ return new YProject(args);	}
	virtual YActions*		GetItemActions() const
							{ return YActionHandler::GetActions(YActionHandler::PROJECT_ACTIONS);	}
};










//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_PROJECT_INCLUDE
