//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA result window declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/03/18
// $Header: /sources/yasa/yasagui/resultwindows.h,v 1.4 2003/01/24 15:47:42 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_RESULT_WINDOWS_INCLUDE
#define	YASAGUI_RESULT_WINDOWS_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qsplitter.h>
#include "frame.h"
#include "environment.h"
#include "project.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResultHandler;

class YTimingFrameObjectArgs : public YFrameObjectArgs
{
public:							YTimingFrameObjectArgs(CreateYFrameCall call, YTimingResults *timingresult);
};
class YStatisticFrameObjectArgs : public YFrameObjectArgs
{
public:							YStatisticFrameObjectArgs(CreateYFrameCall call, YStatisticResults *statisticresult);
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a base class providing functionality to show different types of results in different form of windows
//! e.g. in a splitter window
class YResultFrame : public YObjectFrame
{	Q_OBJECT
	QSplitter					splitter;
	YResultHandler				*resulthandler;
public:							YResultFrame(QWidget *parent, YFrameArgs &args) :
								YObjectFrame(args),
								splitter(parent),
								resulthandler(0)
								{ }
	virtual						~YResultFrame();
	virtual int					InitInstance();
	virtual QWidget*			GetMainWidget()							{	return &splitter;		}
	virtual void				PostCreateWindow(YMDIWindow *)			{	PostCreateTimingFrame();}
	virtual void				PostCreateTimingFrame()					{	return;					}
	YResultHandler *			GetResultHandler() const				{	return resulthandler;	}
	static YFrame*				CreateTimingFrame(QWidget *parent, YFrameArgs &args);
	static YFrame*				CreateStatisticFrame(QWidget *parent, YFrameArgs &args);
protected:
	//! A timing handler can handle the different types of diagrams. Mostly used are "DiagramNameDiagrams"
	//! This is a splitter window. On the left side you can see the names of the timings and on the right
	//! side there is the diming diagram.
	virtual YResultHandler*		CreateResultHandler(QSplitter *parent) const =0;
	virtual int					InsertResults(YResultHandler *resulthandler) const=0;
public slots:
	virtual void				UpdateResults();
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the timing results of the simulation of the current environment
//! YObjectFrame::GetObject() is a type of YEnvironment*
class YEnvironmentResultFrame : public YResultFrame
{	Q_OBJECT
public:							YEnvironmentResultFrame(QWidget *parent, YFrameArgs &args) :
								YResultFrame(parent, args)
								{ }
	virtual void				PostCreateTimingFrame();
	virtual int					InitInstance()
								{
									YEnvironment	*environment = GetEnvironment();
									int				retvalue=environment->EvaluateLogFile();
									if ( ! retvalue )
									{
										retvalue = YResultFrame::InitInstance();
									}
									return retvalue;
								}
protected:	YEnvironment *		GetEnvironment() const
								{
									return (YEnvironment*) ((YTimingResults*)GetObject())->GetParent()->GetParent();
								}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the timing results of the simulation of the current project
//! YObjectFrame::GetObject() is a type of YProject*
class YProjectResultFrame : public YResultFrame
{	Q_OBJECT
public:							YProjectResultFrame(QWidget *parent, YFrameArgs &args) :
								YResultFrame(parent, args)
								{ }
	virtual void				PostCreateTimingFrame();
	virtual int					InitInstance()
								{
									YProject	*project= GetProject();
									int			retvalue=project->EvaluateLogFiles();
									if ( ! retvalue )
									{
										retvalue = YResultFrame::InitInstance();
									}
									return retvalue;
								}
protected:
	YProject *					GetProject() const
								{
									return (YProject*) ((YTimingResults*)GetObject())->GetParent()->GetParent();
								}
public slots:
	virtual void				NewEnvironment(YObject *);
	virtual void				RemoveEnvironment(YObject *object);
};






/*
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is a base class providing functionality to show different types of timings diagrams in a splitter window
class YTimingFrame : public YResultFrame
{	Q_OBJECT
public:							YTimingFrame(QWidget *parent, YFrameArgs &args) :
								YResultFrame(parent, args)
								{ }
	virtual void				PostCreateWindow(YMDIWindow *)			{	PostCreateTimingFrame();	}
	virtual void				PostCreateTimingFrame()					{	return;						}
	static YFrame*				CreateTimingFrame(QWidget *parent, YFrameArgs &args);
protected:
	//! A timing handler can handle the different types of diagrams. Mostly used are "DiagramNameDiagrams"
	//! This is a splitter window. On the left side you can see the names of the timings and on the right
	//! side there is the diming diagram.
	virtual YResultHandler*		CreateResultHandler(QSplitter *parent) const;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the timing results of the simulation of the current environment
//! YObjectFrame::GetObject() is a type of YEnvironment*
class YEnvironmentTimingFrame : public YTimingFrame
{	Q_OBJECT
public:							YEnvironmentTimingFrame(QWidget *parent, YFrameArgs &args) :
								YTimingFrame(parent, args)
								{ }
	virtual void				PostCreateTimingFrame();
	virtual int					InitInstance()
								{
									YEnvironment	*environment = GetEnvironment();
									int				retvalue=environment->EvaluateLogFile();
									if ( ! retvalue )
									{
										retvalue = YTimingFrame::InitInstance();
									}
									return retvalue;
								}
protected:
	virtual int					InsertResults(YResultHandler *resulthandler) const;
	YEnvironment *				GetEnvironment() const
								{
									return (YEnvironment*) ((YTimingResults*)GetObject())->GetParent()->GetParent();
								}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This frame is showing the timing results of the simulation of the current project
//! YObjectFrame::GetObject() is a type of YProject*
class YProjectTimingFrame : public YTimingFrame
{	Q_OBJECT
public:							YProjectTimingFrame(QWidget *parent, YFrameArgs &args) :
								YTimingFrame(parent, args)
								{ }
	virtual void				PostCreateTimingFrame();
	virtual int					InitInstance()
								{
									YProject	*project= GetProject();
									int			retvalue=project->EvaluateLogFiles();
									if ( ! retvalue )
									{
										retvalue = YTimingFrame::InitInstance();
									}
									return retvalue;
								}
protected:
	virtual int					InsertResults(YResultHandler *timinghandler) const;
	YProject *					GetProject() const
								{
									return (YProject*) ((YTimingResults*)GetObject())->GetParent()->GetParent();
								}
public slots:
	virtual void				NewEnvironment(YObject *);
	virtual void				RemoveEnvironment(YObject *object);
};
*/






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_RESULT_WINDOWS_INCLUDE
