//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA source generator
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/sourcegenerator.h,v 1.6 2003/01/30 09:08:08 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_SOURCE_GENERATOR_INCLUDE
#define	YASAGUI_SOURCE_GENERATOR_INCLUDE

#include <sys/stat.h>
#include <qdatetime.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YProject;
class YOutput;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! source generator file names
#define SOURCE_PROJECTCONFIG_NAME			"yasa_projectconfig.h"
#define SOURCE_EXECUTIVE_NAME				"yasa_projectexecutive.h"
#define SOURCE_SCHEDULERS_NAME				"yasa_projectschedulers.h"
#define SOURCE_TASKSET_HEADER_NAME			"yasa_projecttaskset.h"
#define SOURCE_TASKSET_SOURCE_NAME			"yasa_projecttaskset.c"
#define SOURCE_FILES						"files"
#define SOURCE_MAKEFILE_NAME				"Makefile"
#define SOURCE_MAKEFILE_PROJECT_NAME		"Makefile.project"
#define SOURCE_MAKEFILE_SCHEDULERS_NAME		"Makefile.schedulers"
#define SOURCE_RUNIT_NAME					"runit"
#define SOURCE_CLEANUP_NAME					"cleanup"
#define SOURCE_UNINSTALL_NAME				"uninstall"
#define SOURCE_RUNPROJECT_NAME				"runproject"
#define SOURCE_CVSIGNORE_NAME				"cvsignore"
#define SOURCE_CVSIGNORE_FINAL_NAME			".cvsignore"



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! common strings to write source and header files
#define SG_COMMENT_LINE						"//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////"
#define SG_SCRIPT_STARTUP					"#!/bin/sh"
#define SG_SCRIPT_RUNPROJECT				"./runproject"
#define SG_SCRIPT_SHOW_LOGFILE				"cat \"%1\" && echo \"Press Ctrl+d to exit\" && cat"
#define SG_MAKE_COMMAND_SIMULATE			"simulate"
#define SG_MAKE_COMMAND_CLEANUP				"clean"
#define SG_MAKE_COMMAND_UNINSTALL			"uninstall"
#define SG_COMMENT_START					"// "
#define SG_IMPORTANT_COMMENT				"/* %1 */"
#define SG_BRACE_OPENED						"{"
#define SG_BRACE_CLOSED						"}"
#define SG_TAB								"\t"
#define SG_SEMICOLON						";"
#define SG_OBJECT_EXTENSION					"o"						// "hello.o"
#define SG_LOGGING_SIZE						100000
#define SG_DEFAULT_ENVIRONMENTATTR_NAME		"environmentattr"
#define SG_DEFAULT_SCHEDULERATTR_NAME		"schedulerattr"
#define SG_DEFAULT_THREADATTR_NAME			"threadattr"
#define SG_DEFAULT_RESOURCEACTION_NAME		"resourceactions"
#define SG_DEFAULT_RESUMETIME_NAME			"resumetimes"
#define SG_DEFAULT_MUTEXATTR_NAME			"mutexattr"
#define SG_DEFAULT_MUTEXID_NAME				"MUTEXID"
#define SG_FUNCTION_INIT					"extern void* %1(void* param, void **userdata);"
#define SG_FUNCTION_EXECUTE					"extern void* %1(void **userdata);"
#define SG_FUNCTION_CLEANUP					"extern void  %1(void **userdata);"

#define SG_SHELLVAR_YASA					"export YASA=\"%1\""
#define SG_SHELLVAR_PROJECTDIR				"export YASA_PROJECTDIR=\"%1\""
#define SG_SHELLVAR_PROJECTFILENAME			"export YASA_PROJECTFILENAME=\"%1\""
#define SG_SHELLVAR_WORKDIR					"export YASA_WORKDIR=\"%1\""
#define SG_SHELLVAR_EXECUTIVE				"export YASA_EXECUTIVE=\"%1\""
#define SG_SHELLVAR_EXECUTIVEDIR			"export YASA_EXECUTIVEDIR=\"%1\""	// $1/executives/$4
#define SG_SHELLVAR_SCHEDULERS				"export YASA_SCHEDULERS="
#define SG_SHELLVAR_LOGFILE					"export YASA_LOGFILE=\"%1\""
#define SG_SHELLVAR_CFLAGS					"export YASA_CFLAGS=\"-D__YASA__ %1\""
#define SG_SHELLVAR_CFILES					"export YASA_CFILES="
#define SG_SHELLVAR_OFILES					"export YASA_OFILES="




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YSourceGenerator
{
	YEnvironment	*environment;
	YOutput			*output;
public:				YSourceGenerator(YEnvironment *e, YOutput *o) :
					environment(e),
					output(o)
					{
						if (!o)
							qWarning("YSourceGenerator::YSourceGenerator() called with illegal arguments!");
					}
	int				PrepareWorkProject();
	int				WriteHeader(YSourceFile &file, const QString &description);
	int				WriteProjectConfig(YSourceFile &file);
	int				WriteExecutive(YSourceFile &file);
	int				WriteSchedulers(YSourceFile &file);
	int				WriteTaskSetHeader(YSourceFile &file);
	int				WriteTaskSetSource(YSourceFile &file);
	int				CreateProjectSources();
	void			GetMutexProtocolInfo(int protocol, QString &name, QString &comment);
	void			GetResourceActionInfo(int action, QString &param);
	QString&		ConvertToValidLabelName(QString &name);
	QString			ConvertToValidYasaName(const QString &name);
	QString			GetTaskTypeInfo(YTask *task);
	QString			GetDeadlineBehaviour(YTask *task);
	int				CreateScriptFile(const QString &projectdir, const QString &workdir, const QString &command, const QString &filename);
	int				CopyProjectFile(const QString &filename);
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif															// ifndef YASAGUI_SOURCE_GENERATOR_INCLUDE
