//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA statistic declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/03/18
// $Header: /sources/yasa/yasagui/statistics.h,v 1.4 2003/01/24 15:47:43 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_STATISTICS_INCLUDE
#define	YASAGUI_STATISTICS_INCLUDE


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qtable.h>
#include <qvbox.h>
#include <qlabel.h>
#include <yasagui/yasagui.h>
#include <qstringlist.h>
#include <qfont.h>
#include <qfontmetrics.h>
#include "datatypes.h"
#include "environment.h"
#include "project.h"
#include "statistichandler.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YLogAction;
class YStatistics;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStatisticTableItem : public QTableItem
{public:				YStatisticTableItem(QTable *table, const QString &text=YStatisticBaseEntry::GetUnknownString() ) :
						QTableItem(table, QTableItem::Never, text )
						{}
	virtual int			alignment()	const								{	return AlignVCenter | AlignRight;						}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YStatisticLogFileTableItem : public YStatisticTableItem
{	const YStatisticBaseEntry	&statisticentry;
public:					YStatisticLogFileTableItem(QTable *table, const YStatisticBaseEntry &sbe) :
						YStatisticTableItem(table, sbe.GetVisibleString() ),
						statisticentry(sbe)
						{}
	virtual QString		key() const										{	return statisticentry.GetKey();			}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define MIN_COLUMN_WIDTH					85
#define COLUMN_WIDTH_OFFSET					5			// offset added after calculating max width
#define ROW_HEIGHT_OFFSET					7			// offset added after calculating max width
#define MIN_HORIZONTAL_HEADER_WIDTH			85
#define STATISTICTABLE_XOFFSET				4			// used to add width of lines in header
#define STATISTICTABLE_YOFFSET				4

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This is the table class providing a statistic table with horizontal sorting
class YStatisticTable : public QTable
{	Q_OBJECT
	YStatisticHandler	*statistichandler;
	int					horizontalheaderwidth;
	int					columnwidth;
	bool				drawinglocked;
	bool				sortdirection;	// true=lower text is left
protected:
public:					YStatisticTable(int r, int c, QWidget *parent, YStatisticHandler *handler) :
						QTable(r, c, parent),
						statistichandler(handler),
						horizontalheaderwidth(0),
						columnwidth(0),
						drawinglocked(false),
						sortdirection(true)
						{
							connect( verticalHeader(), SIGNAL( clicked(int) ), this, SLOT( rowClicked(int) ) );
							setSorting(true);
						}
	YStatisticHandler*	GetStatisticHandler() const					{	return statistichandler;		}
	virtual void		drawContents(QPainter * p, int cx, int cy, int cw, int ch )
						{
							if ( ! drawinglocked )
								QTable::drawContents(p,cx,cy,cw,ch);
						}
	virtual void		sortColumn ( int col, bool ascending = TRUE, bool wholeRows = FALSE )
						{
							QTable::sortColumn(col, ascending, true);
							clearSelection( true );
							Q_UNUSED(wholeRows);
						}
	virtual void		swapRows( int row1, int row2 )
						{
							QTable::swapRows(row1, row2);
							QString	str=verticalHeader()->label(row1);
							int		w=verticalHeader()->sectionSize(row1);
							verticalHeader()->setLabel(row1, verticalHeader()->label(row2) );
							verticalHeader()->resizeSection(row1, verticalHeader()->sectionSize(row2) );
							verticalHeader()->setLabel(row2, str );
							verticalHeader()->resizeSection(row2, w);
						}
	virtual void		swapColumns( int col1, int col2 )
						{
							QTable::swapColumns(col1, col2);
							QString	str=horizontalHeader()->label(col1);
							int		w=horizontalHeader()->sectionSize(col1);
							horizontalHeader()->setLabel(col1, horizontalHeader()->label(col2) );
							horizontalHeader()->resizeSection(col1, horizontalHeader()->sectionSize(col2) );
							horizontalHeader()->setLabel(col2, str );
							horizontalHeader()->resizeSection(col2, w);
						}
	void				SetVerticalNames(const QStringList &list)
						{
							for ( unsigned int i=0 ; i<list.count(); i++ )
								verticalHeader()->setLabel(i, list[i] );
						}
	void				SetHorizontalNames(const QStringList &list)
						{
							for ( unsigned int i=0 ; i<list.count(); i++ )
								horizontalHeader()->setLabel(i, list[i] );
						}
	//! Enlarge table so that it has no scrollbars left
	void				EnlargeTable()
						{
							int				i,j,k;
							QFont			font;
							QFontMetrics	fontmetrics(font);
							font.setBold(true);
							QFontMetrics	fontmetricsbold(font);

							// determine min. width of all items in the current table, adjustColumn() will not work here
							columnwidth=MIN_COLUMN_WIDTH;
							for ( i=0; i<numCols(); i++)
							{
								k=STATISTICTABLE_XOFFSET + fontmetricsbold.width( horizontalHeader()->label(i) );
								if ( k > columnwidth )
									columnwidth=k;
								for ( j=0; j<numRows(); j++)
								{
									k=fontmetrics.width( text(j,i) );
									if ( k > columnwidth )
										columnwidth=k;
								}
							}
							columnwidth+=COLUMN_WIDTH_OFFSET;
							for ( i=0; i<numCols(); i++)			// adjust all columns to the same size
							{
								horizontalHeader()->resizeSection( i, columnwidth );
							}
							/* does not work here
							for ( i=0; i<numRows(); i++)
							{
								adjustRow(i);
							}*/
							// resize vertical header
							for ( horizontalheaderwidth=MIN_HORIZONTAL_HEADER_WIDTH, i=0; i<numRows(); i++)
							{
								k=fontmetricsbold.width( verticalHeader()->label(i) );
								if ( horizontalheaderwidth < k )
									horizontalheaderwidth=k;
							}
							setLeftMargin(horizontalheaderwidth);
							// adjust horizontal header
							j=horizontalHeader()->height()+STATISTICTABLE_YOFFSET;	// width of horizontal header
							for ( i=0; i<numRows(); i++)
							{
								j += rowHeight(i);
								//j += rowHeight(i) > fontmetricsbold.height()+ROW_HEIGHT_OFFSET ? rowHeight(i) : fontmetricsbold.height()+ROW_HEIGHT_OFFSET ;
							}
							setMinimumHeight(j);
							setMaximumHeight(j);

							// calculate minimum table size
							//j=verticalHeader()->width()+STATISTICTABLE_XOFFSET;		// width of vertical header
							j=0;
							for ( i=0; i<numCols(); i++)
							{
								//setColumnStretchable(i,true);
								j += columnWidth(i);
							}
							setMinimumWidth( j+STATISTICTABLE_XOFFSET );
							resize( width(), height() );
						}
	int					GetMaxVerticalHeaderWidth() const				{	return horizontalheaderwidth;	}
	int					GetMaxColumnWidth() const						{	return columnwidth;				}
	void				SetVerticalHeaderWidth(int w)
						{
							if ( w )
								setLeftMargin(w);
							setMinimumWidth( width() + (w-horizontalheaderwidth+w) );
							resize( width(), height() );
						}
	void				SetColumnWidth(int w)
						{
							for ( int i=0; i<numCols(); i++)			// adjust all columns to the same size
							{
								horizontalHeader()->resizeSection(i,w);
							}
							setMinimumWidth( width() + (w-columnwidth)*numCols() );
							resize( width(), height() );
						}
public slots:
	virtual void		rowClicked ( int row )
						{
							QTableItem	*item1,*item2;
							QString		str1, str2;

							bool	repeat=true;
							if ( numCols() >1 )
							{
								drawinglocked=true;
								horizontalHeader()->setUpdatesEnabled( FALSE );
 								while (repeat)
								{
									repeat=false;
									for ( int i=0; i<numCols()-1; i++)
									{
										item1=item( row, i );
										str1=str2=QString::null;
										if ( item1 )
											str1=item1->key();
										item2=item( row, i+1 );
										if ( item2 )
											str2=item2->key();
										//repeat=str1>str2;
										if ( sortdirection&&(str1>str2) || (!sortdirection&&(str1<str2)) )
										{
											swapColumns(i, i+1);
											repeat=true;
											break;
										}
									}
								};
								drawinglocked=false;
								horizontalHeader()->setUpdatesEnabled(TRUE);
 							}

							// refresh all contents. update() / repaint() does not work :-( Here is the hack :-)
							for ( int i=0; i<numCols(); i++ )
							{
								for ( int j=0; j<numRows(); j++ )
								{
									updateCell( j,i );
								}
							}
							clearSelection( true );
							horizontalHeader()->update();
							sortdirection=!sortdirection;
						}
/*public slots:
	virtual void		NewStatisticInserted(YStatistics *st);
*/
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class is an abstract base class providing an interface for different statisti tables
//! It acts like a widget. It must be inserted with InsertStatistic()
class YStatistics
{	QVBox				*vbox;
	QLabel				*label;
	YStatisticTable		*table;
public:					YStatistics() : vbox(0), label(0), table(0)
						{}
	virtual				~YStatistics()						{													}
	YStatisticTable *	GetTable() const					{	return table;									}
	QLabel *			GetLabel() const					{	return label;									}
	QWidget *			GetMainWidget()						{	return vbox;									}
	virtual int			InitInstance(YStatisticHandler *handler, QWidget *p);
	void				FillTable();
	//! This function is called after initialization the whole class. It is used to resize the tables
	virtual void		PostCreate()						{	table->EnlargeTable();							}
	//! Returns the number of entries (columns) of the statistic table
	virtual unsigned int GetMaxEntries() const=0;
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const=0;
	//! Returns a list of property names which should be drawn in each row
	virtual QStringList	GetPropertyNames() const=0;
	//! Returns a list of entry names which should be drawn in each row
	virtual QStringList	GetEntryNames() const=0;
	//! Return title of statistic table
	virtual QString		GetTableTitle() const=0;
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number) =0;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all statistics of one YEnvironment of the project
class YEnvironmentStatistics : public YStatistics
{protected:
	YEnvironmentLogFileSet &environmentlogfile;
public:					YEnvironmentStatistics(YEnvironmentLogFileSet &e) : environmentlogfile(e)			{ }
	//! Returns the title of statistic table
	virtual QString		GetTableTitle() const
						{
							QString	str=MSGP_SR_TITLE_ENVIRONMENT;
							return str  .arg( environmentlogfile.GetParent()->GetSet()->GetParent()->GetName() )
										.arg( environmentlogfile.GetParent()->GetName() )
										.arg( GetSubTitleType() );
						}
	//! Return sub title of statistic table e.g. "task" or "mutex"
	virtual QString		GetSubTitleType() const=0;
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all entries of the following type:
//!
//! This frame shows the task statistic results of the given environment
//!
//! Statistically results of tasks in environment "simulator"
//!                  | Task 1 | Task 2 | Task 3
//! Utilization      |   64%  |   20%  |  10%
//! missed deadlines |    1   |    5   |   0
//! jitter           |   10   |    0   |   5
//!       +--------------------------------------------------------
class YTaskEnvironmentStatistics : public YEnvironmentStatistics
{public:				YTaskEnvironmentStatistics(YEnvironmentLogFileSet &e) : YEnvironmentStatistics(e)			{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const			{	return YTaskLogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const			{	return YTaskLogFileStatistics::GetMaxProperties();	}
	//! Returns the number of entries (columns) of the statistic table
	virtual unsigned int GetMaxEntries() const				{	return environmentlogfile.GetTaskSet().GetCount();	}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const				{	return MSGP_TI_SUBTITLE_TASK;						}
	//! Returns a list of entry names which should be drawn in each row
	virtual QStringList	GetEntryNames() const
						{	QStringList	list;
							for ( unsigned int i=0; i<GetMaxEntries() ; i++ )
								list.append( environmentlogfile.GetTaskSet().At(i)->GetName() );
							return list;
						}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)	{	return &((YLogFileTask*)(environmentlogfile.GetTaskSet().At(number)))->GetStatistics();	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all entries of the following type:
//!
//! This frame shows the usage of mutexes: The printed numbers are number of the using task
//!
//!         |  Mutex 1 | Mutex 2 | Mutex 3
//! --------+----------+---------+---------
//! Locked  |    10    |   10    |    5
//! Blocked |    4     |    4    |    3
class YMutexEnvironmentStatistics : public YEnvironmentStatistics
{public:				YMutexEnvironmentStatistics(YEnvironmentLogFileSet &e) : YEnvironmentStatistics(e)				{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const			{	return YMutexLogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const			{	return YMutexLogFileStatistics::GetMaxProperties();	}
	//! Returns the number of entries (columns) of the statistic table
	virtual unsigned int GetMaxEntries() const				{	return environmentlogfile.GetMutexSet().GetCount();		}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const				{	return MSGP_TI_SUBTITLE_MUTEX;						}
	//! Returns a list of entry names which should be drawn in each row
	virtual QStringList	GetEntryNames() const
						{	QStringList	list;
							for ( unsigned int i=0; i<GetMaxEntries(); i++ )
								list.append( environmentlogfile.GetMutexSet().At(i)->GetName() );
							return list;
						}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)	{	return &((YLogFileMutex*)(environmentlogfile.GetMutexSet().At(number)))->GetStatistics();	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all entries of the following type:
//!
//! This frame shows the CPU timing results of the given environment
//!
//!            | CPU 1 | CPU 2 | CPU 3
//! -----------+-------+-------+--------
//! Switches   |   1   |   0   |   3
//! Idle times |  100  | 2000  | 3000
class YCPUEnvironmentStatistics : public YEnvironmentStatistics
{ public:				YCPUEnvironmentStatistics(YEnvironmentLogFileSet &e) : YEnvironmentStatistics(e)					{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const			{	return YCPULogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const			{	return YCPULogFileStatistics::GetMaxProperties();	}
	//! Returns the number of entries (columns) of the statistic table
	virtual unsigned int GetMaxEntries() const				{	return environmentlogfile.GetCount();				}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const				{	return MSGP_TI_SUBTITLE_CPU;						}
	//! Returns a list of entry names which should be drawn in each row
	virtual QStringList	GetEntryNames() const
						{	QStringList	list;
							for ( unsigned int i=0; i<GetMaxEntries(); i++ )
								list.append( environmentlogfile.At(i)->GetName() );
							return list;
						}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)	{	return &((YCPULogFile*)(environmentlogfile.At(number)))->GetStatistics();	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Summary statistically result of the selected environment - environment order
//!
//! Global        | Simulator
//! --------------+-----------
//! Task switches |   10
//! Idle times    |  1000
class YSummaryEnvironmentStatistics : public YEnvironmentStatistics
{public:				YSummaryEnvironmentStatistics(YEnvironmentLogFileSet &e) : YEnvironmentStatistics(e)		{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const					{	return YEnvironmentLogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const					{	return YEnvironmentLogFileStatistics::GetMaxProperties();	}
	//! Returns the number of entries (columns) of the statistic table
	virtual unsigned int GetMaxEntries() const						{	return 1;									}
	//! Returns a list of entry names which should be drawn in each row
	virtual QStringList	GetEntryNames() const						{	return QStringList( ((YEnvironment*)environmentlogfile.GetParent())->GetName() );	}
	//! Returns the title of statistic table
	virtual QString		GetTableTitle() const
						{	QString	str=MSGP_SR_TITLE_SUMMARY;
							return str.arg( ((YEnvironment*)environmentlogfile.GetParent())->GetProject()->GetName() );
						}
	//! Returns the mutex name
	virtual QString		GetSubTitleName() const						{	return QString("Global");					}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const						{	return MSGP_TI_SUBTITLE_ENVIRONMENT;		}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int)
	{
		return &environmentlogfile.GetStatistics();
	}
};















//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all environments of the whole projects
class YProjectStatistics : public YStatistics
{	protected:
	YProjectLogFile&	projectlogfile;
public:					YProjectStatistics(YProjectLogFile &lfs) : projectlogfile(lfs)		{ }
	//! Returns the title of statistic table
	virtual QString		GetTableTitle() const
						{
							QString	str=MSGP_SR_TITLE_PROJECT;
							return str  .arg( GetSubTitleType() )
										.arg( GetSubTitleName() )
										.arg( projectlogfile.GetProject()->GetName() );
						}
	//! Returns the property names
	virtual QString		GetSubTitleName() const =0;
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const =0;
	//! Returns the number of entries (columns) of the statistic table
	virtual unsigned int GetMaxEntries() const						{	return projectlogfile.GetMaxEntries();		}
	//! Returns a list of entry names which should be drawn in each row
	virtual QStringList	GetEntryNames() const					{	return projectlogfile.GetEnvironmentNames();}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all entries of the following type:
//!
//! Statistically results of Task X - environment order
//!
//! Task X           | Simulator 1 | Simulator 2 | RT-Linux
//! -----------------+-------------+-------------+----------
//! Activations      |     10      |     10      |    10
//! Missed deadlines |     2       |     1       |     3
class YTaskProjectStatistics : public YProjectStatistics
{	int					taskid;
	QString				taskname;
public:					YTaskProjectStatistics(const QString &tname, int id, YProjectLogFile &p) :
						YProjectStatistics(p), taskid(id), taskname(tname)											{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const					{	return YTaskLogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const					{	return YTaskLogFileStatistics::GetMaxProperties();	}
	//! Returns the task name
	virtual QString		GetSubTitleName() const						{	return taskname;							}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const						{	return MSGP_TI_SUBTITLE_TASK;				}
	int					GetTaskID() const							{	return taskid;								}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)
	{
		YLogFileTaskSet	&taskset=((YEnvironment*)projectlogfile.GetProject()->GetEnvironmentSet().At(number) )
				->GetEnvironmentLogFileSet().GetTaskSet();
		for (unsigned int i=0; i<taskset.GetCount(); i++)
		{
			if ( ((YLogFileTask*)taskset.At(i))->GetId() == taskid )
			{
				return &((YLogFileTask*)taskset.At(i))->GetStatistics();
			}
		}
		return 0;
	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Statistically result of Mutex 1 - environment order
//!
//! Mutex 1         | Simulator 1 | Simulator 2 | RT-Linux
//! ----------------+-------------+-------------+----------
//! Locked          |     10      |     10      |    10
//! Blocked threads |     2       |     1       |     3
class YMutexProjectStatistics : public YProjectStatistics
{	int					mutexid;
	QString				mutexname;
public:					YMutexProjectStatistics(const QString &mname, int id, YProjectLogFile &p) : YProjectStatistics(p),
						mutexid(id), mutexname(mname)																{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const					{	return YMutexLogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const					{	return YMutexLogFileStatistics::GetMaxProperties();	}
	//! Returns the mutex name
	virtual QString		GetSubTitleName() const						{	return mutexname;							}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const						{	return MSGP_TI_SUBTITLE_MUTEX;				}
	int					GetMutexID() const							{	return mutexid;								}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)
	{
		YLogFileMutexSet	&mutexset=((YEnvironment*)projectlogfile.GetProject()->GetEnvironmentSet().At(number) )
				->GetEnvironmentLogFileSet().GetMutexSet();
		for (unsigned int i=0; i<mutexset.GetCount(); i++)
		{
			if ( ((YLogFileMutex*)mutexset.At(i))->GetId() == mutexid )
			{
				return &((YLogFileMutex*)mutexset.At(i))->GetStatistics();
			}
		}
		return 0;
	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class can handle all entries of the following type:
//!
//! Statistically results of CPU 0 - environment order
//!
//! CPU 0         | Simulator | RT-Linux
//! --------------+-----------+------------
//! Task switches |   10      |    5
//! Idle times    |  1000     |    0
class YCPUProjectStatistics : public YProjectStatistics
{	int					cpu;
public:					YCPUProjectStatistics(int c, YProjectLogFile &p) : YProjectStatistics(p), cpu (c)			{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const					{	return YCPULogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const					{	return YCPULogFileStatistics::GetMaxProperties();	}
	//! Returns the mutex name
	virtual QString		GetSubTitleName() const						{	return QString::number(cpu);				}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const						{	return MSGP_TI_SUBTITLE_CPU;				}
	unsigned int		GetCPU() const								{	return cpu;									}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)
	{
		YEnvironmentLogFileSet	&envlogfile=((YEnvironment*)projectlogfile.GetProject()->GetEnvironmentSet().At(number) )->GetEnvironmentLogFileSet();

		//YEnvironmentLogFileSet	*environmentlogfile;
		for (unsigned int i=0 ; i<envlogfile.GetCount() ; i++)
		{
			if ( ((YCPULogFile*)envlogfile.At(i))->GetCPU() == cpu )
			{
				return &((YCPULogFile*)envlogfile.At(i))->GetStatistics();
			}
		}
		return 0;
	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Summary statistically result of all environments - environment order
//!
//! Global        | Simulator | RT-Linux
//! --------------+-----------+------------
//! Task switches |   10      |    5
//! Idle times    |  1000     |    0
class YSummaryProjectStatistics : public YProjectStatistics
{public:				YSummaryProjectStatistics(YProjectLogFile &p) : YProjectStatistics(p)					{ }
	//! Returns the property names
	virtual QStringList	GetPropertyNames() const					{	return YEnvironmentLogFileStatistics::CreatePropertyNames();}
	//! Returns the number of properties
	virtual unsigned int GetMaxProperties() const					{	return YEnvironmentLogFileStatistics::GetMaxProperties();	}
	//! Returns the title of statistic table
	virtual QString		GetTableTitle() const
						{	QString	str=MSGP_SR_TITLE_SUMMARY;
							return str.arg( projectlogfile.GetProject()->GetName() );
						}
	//! Returns the mutex name
	virtual QString		GetSubTitleName() const						{	return QString("Global");					}
	//! Returns the statistic type "Task" or "Mutex"
	virtual QString		GetSubTitleType() const						{	return MSGP_TI_SUBTITLE_ENVIRONMENT;		}
	//! Returns the ptr to YLogFileStatistics of the given column
	virtual YLogFileStatistics* GetEntryLogFileStatistics(int number)
	{
		if ( number < (int)projectlogfile.GetProject()->GetEnvironmentSet().GetCount() )
			return & ((YEnvironment*)projectlogfile.GetProject()->GetEnvironmentSet().At(number) )->GetEnvironmentLogFileSet().GetStatistics();
		return 0;
	}
};












//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_STATISTICS_INCLUDE
