//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Statistic windows code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/04/18
// $Header: /sources/yasa/yasagui/statisticwindows.cpp,v 1.3 2003/01/23 09:18:38 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "resultwindows.h"
#include "statisticwindows.h"
#include "statistichandler.h"
#include "statistics.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YFrame* YResultFrame::CreateStatisticFrame(QWidget *parent, YFrameArgs &args)
{
	YStatisticResults	*statisticresult = (YStatisticResults*) args.GetObject();
	// first parent points to YResult
	// second parent points to YProject or YEnvironment depending on YResult::type
	if ( statisticresult && statisticresult->GetParent() && statisticresult->GetParent()->GetParent() )
	{
		switch ( statisticresult->GetParent()->GetResultArgs().GetResultType() )
		{
		case YResultArgs::PROJECT_RESULTS:		return new YStatisticProjectResultFrame(parent, args);
		case YResultArgs::ENVIRONMENT_RESULTS:	return new YStatisticEnvironmentResultFrame(parent, args);
		default:								break;
		}
	}
	qWarning("YStatisticFrame::CreateStatisticFrame() failed!");
	return 0;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YResultHandler* YStatisticEnvironmentResultFrame::CreateResultHandler(QSplitter *parent) const
{
	return new YStatisticTableHandler(parent);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YStatisticEnvironmentResultFrame::InsertResults(YResultHandler *resulthandler) const
{
	YEnvironment		*environment	= GetEnvironment();
	int					retvalue=YASAE_ERROR_IN_ARGUMENTS;;
	YStatisticHandler	*statistichandler=(YStatisticHandler*)resulthandler;

	if ( statistichandler && environment)
	{
		if ( environment->GetEnvironmentLogFileSet().GetTaskSet().GetCount() )
		{
			retvalue=statistichandler->InsertStatistics( new YTaskEnvironmentStatistics(environment->GetEnvironmentLogFileSet() ) );
			if (retvalue)
				return retvalue;
		}
		if ( environment->GetEnvironmentLogFileSet().GetMutexSet().GetCount() )
		{
			retvalue=statistichandler->InsertStatistics( new YMutexEnvironmentStatistics(environment->GetEnvironmentLogFileSet() ) );
			if (retvalue)
				return retvalue;
		}
		if ( environment->GetEnvironmentLogFileSet().GetCount() )
		{
			retvalue=statistichandler->InsertStatistics( new YCPUEnvironmentStatistics(environment->GetEnvironmentLogFileSet() ) );
			if (retvalue)
				return retvalue;
		}
		//if ( environment->GetEnvironmentLogFileSet().GetCount() >1 )
		retvalue=statistichandler->InsertStatistics( new YSummaryEnvironmentStatistics(environment->GetEnvironmentLogFileSet() ) );
		if (retvalue)
			return retvalue;

		retvalue=statistichandler->PostCreate();
	}
	return retvalue;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YResultHandler* YStatisticProjectResultFrame::CreateResultHandler(QSplitter *parent) const
{
	return new YStatisticTableHandler(parent);
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YStatisticProjectResultFrame::InsertResults(YResultHandler *resulthandler) const
{
	YProject				*project= GetProject();
	YEnvironmentLogFileSet	*logfileset;
	YProjectStatistics		*projectstatistics;
	int						id,retvalue=YASAE_ERROR_IN_ARGUMENTS;;
	unsigned int			index, index2, index3;
	YStatisticHandler		*statistichandler=(YStatisticHandler*)resulthandler;

	if ( statistichandler )
	{
		// create task timings, parse each environment
		for ( index2=0; index2<project->GetEnvironmentSet().GetCount() ; index2++ )
		{
			// check tasks of the current environment
			logfileset=(YEnvironmentLogFileSet*) &((YEnvironment*)project->GetEnvironmentSet().At(index2) )->GetEnvironmentLogFileSet();
			for (index=0; index<logfileset->GetTaskSet().GetCount(); index++)
			{
				// is this task id already inserted ?
				id			= ( (YLogFileTask*) logfileset->GetTaskSet().At(index) )->GetId();
				for( index3=0; index3< statistichandler->GetStatisticsList().count(); index3++)
				{
					projectstatistics=(YProjectStatistics*)statistichandler->GetStatisticsList().at(index3)->GetStatistics();
					if ( projectstatistics->GetSubTitleType() == MSGP_TI_SUBTITLE_TASK )
					{
						if ( ((YTaskProjectStatistics*) projectstatistics )->GetTaskID() == id )
							break;
					}
				}
				// if index3==count() -> task not found
				if ( index3 == statistichandler->GetStatisticsList().count() )
				{
					retvalue=statistichandler->InsertStatistics(
						new YTaskProjectStatistics( ((YLogFileTask*) logfileset->GetTaskSet().At(index))->GetName() ,
												 ((YLogFileTask*) logfileset->GetTaskSet().At(index))->GetId(),
												 project->GetProjectLogFile() ) );
				}
			}
		}


		// create mutex timings, parse each environment
		for ( index2=0; index2<project->GetEnvironmentSet().GetCount() ; index2++ )
		{
			// check mutex' of the current environment
			logfileset=(YEnvironmentLogFileSet*) &((YEnvironment*)project->GetEnvironmentSet().At(index2) )->GetEnvironmentLogFileSet();
			for (index=0; index<logfileset->GetMutexSet().GetCount(); index++)
			{
				// is this mutex id already inserted ?
				id			= ( (YLogFileMutex*) logfileset->GetMutexSet().At(index) )->GetId();
				for( index3=0; index3< statistichandler->GetStatisticsList().count(); index3++)
				{
					projectstatistics=(YProjectStatistics*)statistichandler->GetStatisticsList().at(index3)->GetStatistics();
					if ( projectstatistics->GetSubTitleType() == MSGP_TI_SUBTITLE_MUTEX )
					{
						if ( ((YMutexProjectStatistics*) projectstatistics )->GetMutexID() == id )
							break;
					}
				}
				// if index3==count() -> mutex not found
				if ( index3 == statistichandler->GetStatisticsList().count() )
				{
					retvalue=statistichandler->InsertStatistics(
						new YMutexProjectStatistics( ((YLogFileMutex*) logfileset->GetMutexSet().At(index))->GetName() ,
												 ((YLogFileMutex*) logfileset->GetMutexSet().At(index))->GetId(),
												 project->GetProjectLogFile() ) );
				}
			}
		}

		// create cpu timings, parse each environment
		for ( index=0; index<project->GetEnvironmentSet().GetCount() ; index++ )
		{
			logfileset=&((YEnvironment*)project->GetEnvironmentSet().At(index))->GetEnvironmentLogFileSet();
			for ( index3=0; index3<logfileset->GetCount() ; index3++ )
			{
				// is this task id already inserted ?
				for( index2=0; index2< statistichandler->GetStatisticsList().count(); index2++)
				{
					projectstatistics=(YProjectStatistics*) statistichandler->GetStatisticsList().at(index2)->GetStatistics();
					if ( projectstatistics->GetSubTitleType() == MSGP_TI_SUBTITLE_CPU )
					{
						if ( ((YCPUProjectStatistics*) projectstatistics )->GetCPU() == index3 )
							break;
					}
				}
				// if index2 != count() -> cpu not found
				if ( index2 == statistichandler->GetStatisticsList().count() )
				{
					retvalue=statistichandler->InsertStatistics(
						new YCPUProjectStatistics( index3, project->GetProjectLogFile() ) );
				}
			}
		}

		// insert one global table with the summary of all environments
		retvalue=statistichandler->InsertStatistics(
			new YSummaryProjectStatistics( project->GetProjectLogFile() ) );

		retvalue=statistichandler->PostCreate();
	}
	return retvalue;
}




