/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA project, task support
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/task.h,v 1.6 2003/02/26 14:13:24 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_TASK_INCLUDE
#define	YASAGUI_TASK_INCLUDE

#include <yasagui/yasagui.h>
#include <qobject.h>
#include "mutex.h"
#include <qsortedlist.h>

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTaskSet;
class YProject;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains information when should which action start or end
class YResumeTime : public YObject
{	Q_OBJECT
	YTIME					time;											// time of action
public:
							YResumeTime(const YObjectArgs &args) : YObject(args),
							time(YASA_TIME_INFINITY)
							{
								connect(&time, SIGNAL(UpdateObject()), this, SLOT( UpdateName() ) );
							}
	int						InitInstance(YASA_TIME t)
							{
								time=t;
								return 0;
							}
	const YTIME &			GetTime() const									{ return time;				}
	void					SetTime(YASA_TIME t);
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
public slots:
	void					UpdateName();
/*							{
								if ( GetName() != time.GetVisibleContentString() )
									SetName(time.GetVisibleContentString());
							}*/
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResumeTimeSet : public YSet
{
public:						YResumeTimeSet() : YSet(YPC_RESUME_TIME, MSGP_ZERO, MSGP_DEFAULT_RESUMENAME)
							{
							}
	void					Sort()									{	sort();								}
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)			{ return new YResumeTime(args);	}
	virtual void			AppendItem(YObject *object)				{ if (object)
																		QList<YObject>::inSort(object);		}
	virtual int				compareItems( QCollection::Item item1, QCollection::Item item2 )
	{
		if ( ((YResumeTime*)item1)->GetTime() < ((YResumeTime*)item2)->GetTime() )
			return -1;
		if ( ((YResumeTime*)item1)->GetTime() > ((YResumeTime*)item2)->GetTime() )
			return 1;
		return 0;
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
inline void YResumeTime::UpdateName()
{
	if ( GetName() != time.GetVisibleContentString() )
		SetName(time.GetVisibleContentString());
	if ( GetSet() )
		((YResumeTimeSet*)GetSet())->Sort();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This class contains information when should which action start or end
class YResourceAction : public YObject
{	Q_OBJECT
	YTIME					time;						//! time of action
	QString					mutexname;					//! used during config load
	YObject					*mutex;						//! ptr to mutex object (if 0 mutexname is used!)
	YASA_RESOURCEACTION		action;						//! see yasa_types.h, e.g. lock_mutex
public:
							YResourceAction(const YObjectArgs &args) :
							YObject(args),
							time(YASA_TIME_INFINITY),
							mutex(0),
							action(YASAA_NO_MORE_ACTION)
							{
								connect(&action, SIGNAL(UpdateObject()), this, SLOT( ObjectUpdated() ) );
								connect(&time, SIGNAL(UpdateObject()), this, SLOT( UpdateName() ) );
							}
	const YTIME &			GetTime() const						{	return time;						}
	YObject*				GetMutex();
	const YASA_RESOURCEACTION&	GetAction()	const				{	return action;						}
	void					SetTime(YASA_TIME t)				{	time.SetContent(t);					}
	void					SetMutex(YObject *m);
	void					SetAction(int id)					{	action.SetContent(id);				}
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);
public slots:
	void					MutexUpdated(YObject *);
	void					MutexDestroyed();
	void					UpdateName();
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YResourceActionSet : public YSet
{
public:						YResourceActionSet() : YSet(YPC_RESOURCE_TIME, MSGP_ZERO, MSGP_DEFAULT_RESOURCENAME)
							{ }
	static YMutexSet*		GetMutexSet(YSet *set);
	void					Sort()									{	sort();								}
protected:
	virtual YObject*		AllocNewItem(const YObjectArgs &args)	{ return new YResourceAction(args);		}
	virtual void			AppendItem(YObject *object)				{ if (object)
																		QList<YObject>::inSort(object);		}
	virtual int				compareItems( QCollection::Item item1, QCollection::Item item2 )
	{
		if ( ((YResourceAction*)item1)->GetTime() < ((YResourceAction*)item2)->GetTime() )
			return -1;
		if ( ((YResourceAction*)item1)->GetTime() > ((YResourceAction*)item2)->GetTime() )
			return 1;
		return 0;
	}
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
inline void YResourceAction::UpdateName()
{
	if ( GetName() != time.GetVisibleContentString() )
		SetName(time.GetVisibleContentString());
	if ( GetSet() )
		((YResourceActionSet*)GetSet())->Sort();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YTask;
class YTaskSet : public YSet
{	YTask					*parenttask;
public:						YTaskSet(YTask *p);
protected:
	inline virtual YObject*	AllocNewItem(const YObjectArgs &args);
	virtual YActions*		GetItemActions() const
							{ return YActionHandler::GetActions(YActionHandler::TASK_ACTIONS);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This class contains information about one defined task including "mutex times"
class YTask : public YObject
{
	Q_OBJECT
	YTaskSet				*parenttaskset;
	YASA_PERIOD				period;					//! period of task or zero
	YASA_OFFSET				offset;					//! start offset
	YASA_REQUIREDTIME		requiredtime;			//! required time of task
	YASA_LAXITY				laxity;
	YASA_DEADLINE			deadline;				//! deadline of task
	YASA_DEADLINETOLERANCE	deadlinetolerance;		//! deadline tolerance
	YASA_CPU				cpu;					//! cpu where task should run (zero=not defined)
	YASA_TASKTYPE			tasktype;				//! periodically, aperiodically, server
	YASA_SERVERTYPE			servertype;
	YASA_PRIORITY			priority;
	YASA_DLBEHAVIOUR		deadlinebehaviour;		//! behaviour if deadline missed
	YTaskSet				subtaskset;				//! if this is a server, all aperiodically tasks are subtasks
	YResourceActionSet		resourceactionset;		//! resource action times
	YResumeTimeSet			resumetimeset;			//! start times of asynchronous threads
	YString					function_init;			//! init function of task
	YString					function_execute;		//! execute function of task
	YString					function_cleanup;		//! cleanup function of task
public:						YTask(const YObjectArgs &args);
	int						InitInstance();
	YTask*					CreateSubTask(const YObjectArgs &i );
	const YTaskSet*			CreateParentTaskSet()				{ return parenttaskset;			}
//	const QList<YTask> &	GetSubTaskSet()						{ return subtaskset;			}
//	YTaskSet &				GetSubTaskSet()						{ return taskset;				}
	YResumeTimeSet &		GetResumeTimeSet()					{ return resumetimeset;			}
	YResourceActionSet &	GetResourceActionSet()				{ return resourceactionset;		}
	virtual int				SetConfig(YInputParser &parser);
	virtual int				GetConfig(YOutputParser &parser);

	YASA_CPU &				GetCPU()							{ return cpu;					}
	const YASA_PRIORITY	&	GetPriority() const					{ return priority;				}
	//const YASA_PERIOD &		GetPeriod() const					{ return period;				}
	YASA_PERIOD &		GetPeriod()					{ return period;				}
	const YASA_OFFSET &		GetOffset() const					{ return offset;				}
	const YASA_REQUIREDTIME & GetRequiredTime() const			{ return requiredtime;			}
	const YASA_LAXITY &		GetLaxity() const					{ return laxity;				}
	const YASA_DEADLINE	&	GetDeadline() const					{ return deadline;				}
	const YASA_DEADLINETOLERANCE & GetDeadlineTolerance() const	{ return deadlinetolerance;		}
	const YASA_DLBEHAVIOUR & GetDeadlineBehaviour() const		{ return deadlinebehaviour;		}
	const YASA_TASKTYPE	&	GetTaskType() const					{ return tasktype;				}
	const YASA_SERVERTYPE &	GetServerType()	const				{ return servertype;			}
	const YString&			GetInitFunction() const				{ return function_init;			}
	const YString&			GetExecuteFunction() const			{ return function_execute;		}
	const YString&			GetCleanupFunction() const			{ return function_cleanup;		}
	YProject*				GetProject() const					{ return ((YProject*)GetSet()->GetParent()); }
	YTask*					GetEmergencyThread() const			{ return 0;						}

	void					SetCPU(int c)						{ cpu=c;						}
	void					SetPriority(int p)					{ priority=p;					}
	void					SetPeriod(YASA_TIME p)				{ period=p;						}
	void					SetOffset(YASA_TIME o)				{ offset=o;						}
	void					SetRequiredTime(YASA_TIME r)		{ requiredtime=r;				}
	void					SetDeadline(YASA_DEADLINE d)		{ deadline=d;					}
	void					SetDeadlineTolerance(YASA_TIME d)	{ deadlinetolerance=d;			}
	void					SetDeadlineBehaviour(int b)			{ deadlinebehaviour=b;			}
	void					SetTaskType(int t)					{ tasktype=t;					}
	void					SetServerType(int t)				{ servertype=t;					}
	void					SetInitFunction(const QString &f)	{ function_init=f;				}
	void					SetExecuteFunction(const QString &f){ function_execute=f;			}
	void					SetCleanupFunction(const QString &f){ function_cleanup=f;			}
public slots:
	void					SubTaskChanged(YObject *object);
	void					ResumeTimeChanged(YObject *object);
	void					ResourceActionChanged(YObject *object);


	void					CalculateLaxity()				{	laxity.operator =(deadline-requiredtime);	}
};


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																	// ifndef YASAGUI_TASK_INCLUDE
