//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA task window code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/taskwindow.cpp,v 1.3 2003/01/23 09:18:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <qlayout.h>
#include <qvbox.h>
#include <qhbox.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include "taskwindow.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTaskFrame::InitInstance()
{
	YTask				*task=(YTask*)GetObject();

	QFrame				*scrollframe	= new QFrame( GetMainWidget() );
	QVBoxLayout			*vboxlayout		= new QVBoxLayout(scrollframe, MDI_SCROLLVIEW_BORDER );


	QGroupBox	*group1		= new QGroupBox( 5, Horizontal, MSGP_WIDGET_PROPERTIES	, scrollframe );
	QHBox		*middleframe= new QHBox ( scrollframe );
	QGroupBox	*group2		= new QGroupBox( 1, Horizontal, MSGP_WIDGET_DESCRIPTION	, scrollframe );

	vboxlayout->addWidget( group1 );
	vboxlayout->addWidget( middleframe );
	vboxlayout->addWidget( group2 );
	group1->setAlignment(AlignHCenter);
	group2->setAlignment(AlignHCenter);


	// upper group
	QFrame	*upper[]=
	{
		new QVBox( group1 ),
		new QVBox( group1 ),
		new QFrame(group1 ),
		new QVBox( group1 ),
		new QVBox( group1 ),
	};
	upper[2]->setMinimumWidth(MDI_SCROLLVIEW_BORDER);
	new	QLabel(MSGP_WIDGET_NAME				, upper[0] );
	new QLabel(MSGP_TASK_PERIOD				, upper[0] );
	new QLabel(MSGP_TASK_REQUIRED_TIME		, upper[0] );
	new QLabel(MSGP_TASK_OFFSET				, upper[0] );
	new QLabel(MSGP_TASK_DEADLINE			, upper[0] );
	new QLabel(MSGP_TASK_DEADLINE_TOLERANCE	, upper[0] );
	new QLabel(MSGP_TASK_DEADLINE_BEHAVIOUR	, upper[0] );

	new YNameWidget(				task	, upper[1] );
	new YPeriodWidget(				task	, upper[1] );
	new YRequiredTimeWidget(		task	, upper[1] );
	new YOffsetWidget(				task	, upper[1] );
	new YDeadlineWidget(			task	, upper[1] );
	new YDeadlineToleranceWidget(	task	, upper[1] );
	new YDeadlineBehaviourWidget(	task	, upper[1] );

	new QLabel(MSGP_TASK_TYPE				, upper[3] );
	new QLabel(MSGP_TASK_PRIORITY			, upper[3] );
	new QLabel(MSGP_TASK_CPU				, upper[3] );
	new QLabel(MSGP_TASK_INIT_FUNCTION		, upper[3] );
	new QLabel(MSGP_TASK_EXECUTE_FUNCTION	, upper[3] );
	new QLabel(MSGP_TASK_CLEANUP_FUNCTION	, upper[3] );

	new YTaskTypeWidget(			task	, upper[4] );
	new YPriorityWidget(			task	, upper[4] );
	new YCPUTaskWidget(				task	, upper[4] );
	new YInitFunctionWidget(		task	, upper[4] );
	new YExecuteFunctionWidget(		task	, upper[4] );
	new YCleanupFunctionWidget(		task	, upper[4] );

	// middle frame
	YListViewGroup	*list1			=	new YResourceActionWidget( &task->GetResourceActionSet(), middleframe );
	QFrame			*middleframe2	=	new QFrame(middleframe);
	YListViewGroup	*list2			=	new YResumeTimeWidget( &task->GetResumeTimeSet(), middleframe );
	middleframe2->setMinimumWidth(MDI_SCROLLVIEW_BORDER);
	list1->InitInstance();
	list2->InitInstance();

	// lower group
	new YDescriptionWidget( task, group2 );


	scrollview.addChild(scrollframe);
	scrollview.setResizePolicy(QScrollView::AutoOneFit);

	return YObjectFrame::InitInstance();
}







//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YListViewGroup::InsertListView(YListView *l)
{
	listview=l;
	//connect( listview, SIGNAL(currentChanged(QListViewItem *)), this, SLOT(UpdateWidgets(QListViewItem *) ) );
	connect( listview, SIGNAL(selectionChanged(QListViewItem *)), this, SLOT(UpdateWidgets(QListViewItem *) ) );
	connect( listview, SIGNAL(ListViewEmpty()), this, SLOT(ListViewEmpty() ) );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YListViewGroup::CreateWidgets(YSet *set, QWidget *parent)
{
	if ( parent )
	{
		hbox		= new QHBox( parent );
		add			= new QPushButton( MSGP_WIDGET_ADD, hbox );
		del			= new QPushButton( MSGP_WIDGET_DELETE, hbox );
		deleteall	= new QPushButton( MSGP_WIDGET_DELETE_ALL, hbox );
		connect( add, SIGNAL(released()), this, SLOT(AddEntry() ) );
		connect( del, SIGNAL(released()), this, SLOT(DeleteEntry() ) );
		connect( deleteall, SIGNAL(released()), this, SLOT(RemoveAll() ) );
	}
	UpdateWidgets(0);

	return 0;
	Q_UNUSED(set);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Button "Add" pressed
YObject* YListViewGroup::AddEntry()
{
	if ( listview && listview->GetSet() )
		return listview->GetSet()->CreateNewItem();
	return 0;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YListViewGroup::DeleteEntry()
{
	if ( listview && listview->selectedItem() )
		listview->GetSet()->RemoveItem( ((YListViewItem*) listview->selectedItem() ) -> GetObject() );
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YListViewGroup::RemoveAll()
{
	if ( listview && listview->GetSet() )
		listview->GetSet()->ClearSet();
	UpdateWidgets(0);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YListViewGroup::UpdateWidgets(QListViewItem *currentitem)
{
	del->setEnabled( currentitem );
	if ( listview && listview->GetSet() )
		deleteall->setEnabled( listview->GetSet()->GetCount()  );
	else
		deleteall->setEnabled(false);
}




//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YResumeTimeWidget::InitInstance()
{
	int	retvalue=YASAE_OUT_OF_MEMORY;
	InsertListView( new YResumeTimeListView( (YResumeTimeSet*)YWidget::object, this, MSGP_TASK_RESUME_TIME) );
	if ( listview )
	{
		retvalue=listview->InitInstance();
		if ( !retvalue)
		{
			edittime  = new QSpinBox(this);
			edittime->setMinValue(YTIME::YASA_TIME_MIN);
			edittime->setMaxValue(YTIME::YASA_TIME_INT_MAX);
			connect( edittime, SIGNAL(valueChanged(int)), this, SLOT(UpdateTime(int) ) );
			CreateWidgets((YSet*)YWidget::object, this);
			retvalue=YListViewGroup::InitInstance();
		}
	}
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YResumeTimeWidget::AddEntry()
{
	YObject *object;
	object=YListViewGroup::AddEntry();
	if ( object )
	{
		((YResumeTime*)object)->SetTime(0);
		edittime->setFocus();
	}
	return object;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResumeTimeWidget::UpdateTime(int t)
{
	YResumeTimeListViewItem	*item;
	item=(YResumeTimeListViewItem*)listview->currentItem();
	if ( item )
		((YResumeTime*)item->GetObject())->SetTime(t);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResumeTimeWidget::UpdateWidgets(QListViewItem *item)
{
	edittime->setEnabled( item );
	if ( item )
		edittime->setValue( ((YResumeTime*)((YResumeTimeListViewItem*)item)->GetObject())->GetTime() );
	//else
	//	edittime->setValue(0);
	YListViewGroup::UpdateWidgets(item);
}






//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YResourceActionWidget::InitInstance()
{
	int	retvalue=YASAE_OUT_OF_MEMORY;
	InsertListView( new YResourceActionListView((YResourceActionSet*)YWidget::object, this, MSGP_TASK_RESOURCE_TIME) );
	if ( listview )
	{
		retvalue=listview->InitInstance();
		if ( ! retvalue )
		{
			hbox	  = new QHBox( this );
			edittime  = new QSpinBox(hbox);
			edittime->setMinValue(YTIME::YASA_TIME_MIN);
			edittime->setMaxValue(YTIME::YASA_TIME_INT_MAX);
			actioncombo = new QComboBox(hbox);
			actioncombo->insertItem(MSGP_TASK_ACTION_UNDEFINED);
			actioncombo->insertItem(MSGP_TASK_ACTION_LOCK_MUTEX);
			actioncombo->insertItem(MSGP_TASK_ACTION_UNLOCK_MUTEX);
			mutexstringlist= new YStringList( YResourceActionSet::GetMutexSet( (YSet*)object ), true);
			mutexcombo = new QComboBox(hbox);
			mutexstringlist->AddItem(0, MSGP_TASK_MUTEX_UNDEFINED, false);
			mutexcombo->insertStringList(*mutexstringlist);
			listview->addColumn(MSGP_TASK_ACTION);
			listview->addColumn(MSGP_TASK_MUTEX);
			connect( edittime, SIGNAL(valueChanged(int)), this, SLOT(TimeChanged(int) ) );
			connect( actioncombo, SIGNAL(activated(int)), this, SLOT(ActionChanged(int) ) );
			connect( mutexcombo, SIGNAL(activated(int)), this, SLOT(MutexChanged(int) ) );
			connect( mutexstringlist, SIGNAL(StringListChanged()), this, SLOT(MutexStringListChanged() ) );
			CreateWidgets((YSet*)YWidget::object, this);
			retvalue=YListViewGroup::InitInstance();
		}
	}
	return retvalue;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YObject* YResourceActionWidget::AddEntry()
{
	YObject *object;
	object=YListViewGroup::AddEntry();
	if ( object )
	{
		((YResourceAction*)object)->SetTime(0);
		edittime->setFocus();
	}
	return object;
}








//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YResourceActionListViewItem::UpdateObject(YObject *o)
{
	switch ( ((YResourceAction*)object)->GetAction().GetContent() )
	{
	case YASAA_MUTEX_LOCK:		setText(1, MSGP_TASK_ACTION_LOCK_MUTEX);
								break;
	case YASAA_MUTEX_UNLOCK:	setText(1, MSGP_TASK_ACTION_UNLOCK_MUTEX);
								break;
	default:
	case YASAA_NO_MORE_ACTION:	setText(1, MSGP_TASK_ACTION_UNDEFINED);
								break;
	}
	if ( ((YResourceAction*)object)->GetMutex() )
		setText( 2, ((YResourceAction*)object)->GetMutex()->GetName() );
	else
		setText( 2, MSGP_TASK_MUTEX_UNDEFINED);

	YListViewItem::UpdateObject(o);
}


//////////////////////////////////////////////////////////
// RESOLVE MUTEX <-> RESOURCETIME dependencies after loading the config
//////////////////////////////////////////////////////////
