//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA timing painter declarations
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/timingpainter.h,v 1.5 2003/01/24 15:47:43 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This file contains all information about timing diagram frames
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#ifndef YASAGUI_TIMING_PAINTER_INCLUDE
#define	YASAGUI_TIMING_PAINTER_INCLUDE

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <yasagui/yasagui.h>
#include "datatypes.h"
#include <qpainter.h>
#include "timingsettings.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YLogFileFigures;
class YLogAction;


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
class YPainterLogFileArgs
{
public:
	YASA_TIME			cxtime;					// time value at position cx
	YASA_TIME			crtime;					// time value at position cw+cx
	int					xroot;					// y-position left lower edge in current timing widget diagram
	int					yroot;					// y-position left lower edge in current timing widget diagram
	bool				drawsystemrange;
	bool				drawtaskrange;
	bool				drawtaskexcludedrange;
	bool				drawtaskidlerange;
	bool				drawmutexrange;
	bool				drawjitterrange;
						YPainterLogFileArgs() :	cxtime(0), crtime(0), xroot(0), yroot(0), drawtaskrange(false),
						drawtaskexcludedrange(false), drawmutexrange(false), drawjitterrange(false)
						{ }
	void				ResetRangeDrawModes()						{	drawsystemrange			=false;
																		drawtaskrange			=false;
																		drawtaskidlerange		=false;
																		drawtaskexcludedrange	=false;
																		drawmutexrange			=false;
																		drawjitterrange			=false;		}

};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! A painter class is used to draw all scrollviews in one timing window. The interger values of cx, cy, cw, ch
//! contain the current invalid rectangle. This works since all updates should be done in a queue
class YPainter
{	QPainter			*painter;
	QPoint				root;
	YLogFileFigures	&	logfilepixmaps;
	// ptr to brushes stored here to improve speed and to avoid traversing pixmap list
	const QBrush &		windowbackground;		// ptr to brush of YApplicationFigures::FIGURE_BACKGROUND_TIMINGWINDOWS
	const QBrush &		systemtaskbrush;		// ptr to brush of YLogFileFigures::SYSTEM_TASK_BRUSH
	const QBrush &		taskbrush;				// ptr to brush of YLogFileFigures::TASK_BRUSH
	const QBrush &		taskidlebrush;			// ptr to brush of YLogFileFigures::TASK_IDLE_BRUSH
	const QBrush &		taskexcludedbrush;		// ptr to brush of YLogFileFigures::TASK_EXCLUDED_BRUSH
	const QBrush &		mutexbrush;				// ptr to brush of YApplicationFigures::FIGURE_MUTEX_BRUSH
	const QBrush &		jitterbrush;			// ptr to brush of YApplicationFigures::FIGURE_JITTER_BRUSH
public:
	YPainterLogFileArgs	painterlogfileargs;
	int					cx;
	int					cy;
	int					cw;
	int					ch;
	//fast hack: int					zoomfactorx;
	//fast hack: int					legendfactorx;			// every x times we draw a time message under the horizontal legend line
	YASA_TIME			zoomfactorx;
	YASA_TIME			legendfactorx;			// every x times we draw a time message under the horizontal legend line
public:					YPainter();
	void				DrawText(int x, int y, const QString &text)	{	painter->drawText( root.x()+x, root.y()+y, text );	}
	void				DrawLine(const QPoint &p1, const QPoint &p2){	painter->drawLine( p1.x()+root.x(), p1.y()+root.y(), p2.x()+root.x(), p2.y()+root.y() );	}
	void				DrawRect(const QRect &r, const QColor &color=QColor(240,222,208) )
																	{	painter->fillRect( r.x()+root.x(), r.y()+root.y(), r.width(), r.height(), color );	}
	void				DrawRect(int x,int y,int w,int h, const QColor &color=QColor(240,222,208) )
																	{	painter->fillRect( x+root.x(), y+root.y(), w, h, color );	}
	void				DrawPixmap(int x,int y, const QPixmap &p) const
																	{	painter->drawPixmap(x+root.x(),y+root.y(),p);	}
	void				DrawLoggingAction(YLogAction *action, int row);
	//void				SetZoomFactor(int z=0);
	void				SetZoomFactor(YASA_TIME maxtime, int z);
	int					GetZoomFactor() const						{	return zoomfactorx;			}
	int					GetLegendFactor() const						{	return legendfactorx;		}
	void				SetPainter(QPainter *p)						{	painter=p;					}
	void				SetRoot(const QPoint &r)					{	root=r;						}
	int					GetRootX() const							{	return root.x();			}
	int					GetRootY() const							{	return root.y();			}
	void				SetUpdateArea(int x, int y, int w, int h)	{	cx=x; cy=y; cw=w; ch=h;		}
	QPainter*			GetPainter() const							{	return painter;				}
	const QBrush &		GetWindowBackgroundBrush() const			{	return windowbackground;	}
};



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#endif																			// ifndef YASAGUI_TIMING_PAINTER_INCLUDE
