//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA Timing diagram window code
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/timings.cpp,v 1.3 2003/01/23 09:18:39 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "timingsettings.h"
#include "timings.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTimings::CalculateHeight()
{
	height=YTIMING_TOP_OFFSET;

	// font size of title
	height += GetTitleHeight();

	// calculate timing entries
	height += GetTimingsHeight();

	// grid
	height += YTIMING_GRID_LINE_HEIGHT;

	// legend of timing grid
	height += YTIMING_GRID_LEGEND_HEIGHT+YTIMING_SPACING_Y;
	height += fontmetrics->height();						// legend font
	height += YTIMING_SPACING_Y;							// between Font and edge

	return height;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int YTimings::CalculateWidth(int zoomfactorx)
{
	int				tmp=0;
	unsigned int	index;

	width=YTIMING_EDGE_SPACING_X*2;							// left and right space

	// max name width
	tmp = fontmetrics->width( GetTitle() );
	//width += YTIMING_NAME_WIDTH;
	//width += YTIMING_SPACING_X;

	// grid
	width += YTIMING_GRID_LINE_WIDTH;

	// duration of simulation
	if ( GetMaxDuration() / zoomfactorx > tmp)
		width += GetMaxDuration() / zoomfactorx ;
	else
		width += tmp;

	width += fontmetrics->width( ( YTIME::GetSourceString(YASA_TIME_INFINITY-1) ) )/2;

	// calculate min width of name splitter
	namewidth=0;
	for ( index=0; index < GetRowNames().count(); index++ )
	{
		tmp=fontmetrics->width( GetRowNames()[index] );
		if ( tmp>namewidth )
			namewidth=tmp;
	}
	namewidth+=YTIMING_SPACING_X*3;


	return width;
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! This function is used to draw one timing diagram. It is called by YTimingDiagramScrollView::drawContents
void YTimings::DrawTimingContents(YPainter &p) const
{
	YASA_TIME	time;
	YASA_TIME	endtime;
	int			lineoffset;							// offset from top of widget to horizontally line
	int			topoffset;							// top offset of diagram in timing widget
	int			fontsize;
	int			posx,posx2;
	int			legendoffset;
	QString		tmp;

	// subtract/adding YTIMING_FIGURE_WIDTH/2 is needed to redraw figures during scrolling
	if ( p.cx >= YTIMING_FIGURE_WIDTH/2 )
		p.cx -= YTIMING_FIGURE_WIDTH/2;
	p.cw += YTIMING_FIGURE_WIDTH;

	if ( p.cx <= YTIMING_EDGE_SPACING_X )
	{
		time	= 0;	// time represented by p.cx
		endtime	= (p.cx+p.cw <= YTIMING_EDGE_SPACING_X) ? 0 : (p.cx+p.cw+1-YTIMING_EDGE_SPACING_X) * p.GetZoomFactor();				// time represented by p.cx+p.cw
		p.painterlogfileargs.xroot=p.GetRootX()+YTIMING_EDGE_SPACING_X;
	}
	else
	{
		time	= (p.cx-YTIMING_EDGE_SPACING_X) * p.GetZoomFactor();	// time represented by p.cx
		endtime	= time+(p.cw+1) * p.GetZoomFactor();				// time represented by p.cx+p.cw
		p.painterlogfileargs.xroot=p.GetRootX()+p.cx;
	}
	//endtime		= (p.cx+p.cw <= YTIMING_EDGE_SPACING_X) ? 0 : time+(p.cw+1) * p.GetZoomFactor();				// time represented by p.cx+p.cw
	topoffset	= YTIMING_TOP_OFFSET+GetTitleHeight();			// top offset of diagram in timing widget
	lineoffset	= topoffset + GetTimingsHeight();				// y offset of horizontal line in diagram
	fontsize	= fontmetrics->height();							// current font height
	p.painterlogfileargs.cxtime=time;
	p.painterlogfileargs.crtime=endtime;
	//p.painterlogfileargs.xroot=p.GetRootX()+p.cx+YTIMING_EDGE_SPACING_X;
	p.painterlogfileargs.yroot=p.GetRootY()+topoffset+YTIMING_SPACING_Y;


	// draw title (if disabled since drawText moves in the widget )
	//if ( (p.cy <= YTIMING_TOP_OFFSET+YTIMING_SPACING_Y) && (p.cy+p.ch >= YTIMING_TOP_OFFSET+YTIMING_SPACING_Y) )
	p.DrawText( YTIMING_EDGE_SPACING_X+YTIMING_SPACING_X, YTIMING_TOP_OFFSET+YTIMING_SPACING_Y+fontsize, GetTitle() );

	// draw underline of title
	if ( (p.cy <= topoffset-YTIMING_SPACING_Y+1) && (p.cy+p.ch >= topoffset-YTIMING_SPACING_Y+1) )
		p.DrawLine( QPoint( 0									, topoffset-YTIMING_SPACING_Y+1),
					QPoint( GetTitleWidth()	+ YTIMING_EDGE_SPACING_X+YTIMING_SPACING_X	, topoffset-YTIMING_SPACING_Y+1 ) );
	// draw horizontally line of timing diagram
	posx=0;	//YTIMING_EDGE_SPACING_X;
	if ( p.cx > posx )
		posx=p.cx;
	posx2=YTIMING_EDGE_SPACING_X+YTIMING_GRID_LINE_WIDTH+GetMaxDuration()/p.zoomfactorx;
	if ( p.cx+p.cw < posx2 )
		posx2=p.cx+p.cw;
	if ( (p.cy <= lineoffset) && (p.cy+p.ch >= lineoffset) && posx<posx2)
		p.DrawLine( QPoint( posx	, lineoffset),		QPoint( posx2	, lineoffset ) );

	// draw vertically line of timing diagram
	if ( (p.cx <= YTIMING_EDGE_SPACING_X) && (p.cx+p.cw >= YTIMING_EDGE_SPACING_X) )
		p.DrawLine( QPoint( YTIMING_EDGE_SPACING_X				, YTIMING_TOP_OFFSET+YTIMING_SPACING_Y),	// topoffset
					QPoint( YTIMING_EDGE_SPACING_X				, lineoffset ) );


	// Draw grid. time and endtime are used to print the legend squares and the time values. The must
	// be rounded up(endtime) and down(time)
	time=time/p.legendfactorx*p.legendfactorx;						// round time to legendfactor
	endtime=(endtime/p.legendfactorx+1)*p.legendfactorx;			// round endtime to legendfactor
	if ( endtime > GetMaxDuration() )
		endtime = GetMaxDuration();
	legendoffset=lineoffset+YTIMING_GRID_LINE_HEIGHT + YTIMING_GRID_LEGEND_HEIGHT+YTIMING_SPACING_Y+fontsize;
	if ( (p.cy <= lineoffset)   && (p.cy+p.ch >= lineoffset)  ||
		 (p.cy >= lineoffset)   && (p.cy      <= legendoffset) )
	{
		for ( ; time<=endtime ; time+=p.legendfactorx )
		{
			posx=YTIMING_EDGE_SPACING_X+time/p.zoomfactorx;
			p.DrawRect( posx-YTIMING_GRID_LEGEND_WIDTH/2, lineoffset,
						YTIMING_GRID_LEGEND_WIDTH, YTIMING_GRID_LEGEND_HEIGHT,
						YTIMING_GRID_COLOR );
			tmp=YTIME::GetSourceString(time);
			p.DrawText( posx - fontmetrics->width(tmp)/2,
						legendoffset,
						tmp );
		}
	}

	// draw the timing diagram. During the parsing process the Function "YTimingParser::EvaluateLoggingEntry()"
	// is called to filter and draw the contents using YPainter::DrawLoggingAction()
	timingparser->SetTime(p.painterlogfileargs.cxtime, p.painterlogfileargs.crtime );
	timingparser->PrepareLogfilePainting();
	EvaluateTimings();										// start parsing

}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// This function is used to print the timing names. It is called by YTimingNameScrollView::drawContents
void YTimings::DrawNameContents(YPainter &p) const
{
	QRect	rect=p.GetPainter()->viewport();
	unsigned int	index;
	int topoffset=YTIMING_TOP_OFFSET+GetTitleHeight()-YTIMING_SPACING_Y;
	p.DrawText( YTIMING_SPACING_X*2,YTIMING_TOP_OFFSET+YTIMING_SPACING_Y+fontmetrics->height(), GetOrderTitle() );

	// title line
	p.DrawLine( QPoint( p.cx				, topoffset+1),
				QPoint( p.cx+rect.right()	, topoffset+1 ) );

	// legend line
	p.DrawLine( QPoint( p.cx				, topoffset+GetTimingsHeight()+YTIMING_SPACING_Y),
				QPoint( p.cx+rect.right()	, topoffset+GetTimingsHeight()+YTIMING_SPACING_Y ) );

	// vertical line
	p.DrawLine( QPoint( YTIMING_SPACING_X	, YTIMING_TOP_OFFSET+YTIMING_SPACING_Y),
				QPoint( YTIMING_SPACING_X	, topoffset+GetTimingsHeight()+YTIMING_GRID_LEGEND_HEIGHT+YTIMING_GRID_LEGEND_HEIGHT-1 ) );

	for ( index=0; index < GetRowNames().count(); index++ )
	{
		p.DrawText( YTIMING_SPACING_X*2,topoffset+(YTIMING_SPACING_Y+1)/2+GetTimingHeight()*(index+1), GetRowNames()[index] );
	}
}



