//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// YASA work space
//
// Project: Yasa 2
// Author : Jan Blumenthal
// Start  : 2002/02/28
// $Header: /sources/yasa/yasagui/workspace.cpp,v 1.3 2003/01/23 09:18:40 bj Exp $
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program (See the included file COPYING);
// if not, write to the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.
//
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include "actionhandler.h"
#include "workspace.h"
#include "filedialog.h"
#include "parser.h"


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
YWorkSpace::YWorkSpace(YApplication &app) :
YObject(YObjectArgs(MSGP_CAPTION_PROJECTS) ),
application(app),
version(YASAGUI_VERSION)
{
	ResetProjectManager();
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Output:
int	YWorkSpace::SetConfig(YInputParser &parser)
{
	QString		str;
	int			retvalue;

	if ( parser.CheckSectionStart(YPC_PROJECT ) )
		return parser.NewSectionSet(YPC_PROJECT, projectset );
	if ( parser.CheckSectionStart(YPC_AUTHOR) )
		return parser.ReadContentString(author);
	if ( parser.CheckSectionStart(YPC_VERSION) )
		return parser.ReadContentInt(version);
	if ( parser.CheckSectionStart(YPC_CREATION_DATE) )
	{
		retvalue=parser.ReadContentString(str);
		if ( retvalue )
			return retvalue;
#if QT_VERSION>=300
		creation_time=QDateTime::fromString(str);
#endif
		return 0;
	}
	if ( parser.CheckSectionStart(YPC_LASTCHANGED_DATE) )
	{
		retvalue=parser.ReadContentString(str);
		if ( retvalue )
			return retvalue;
#if QT_VERSION>=300
		lastchanged_time=QDateTime::fromString(str);
#endif
		return 0;
	}
	return YObject::SetConfig(parser);
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
int	YWorkSpace::GetConfig(YOutputParser &parser)
{
	QString	string;
	int retvalue;
	retvalue=parser.NewSectionSet(YPC_PROJECT, projectset);
		if ( retvalue )	return retvalue;
	retvalue=parser.WriteContentString(YPC_AUTHOR, author);
		if ( retvalue )	return retvalue;
	string=creation_time.toString();
	retvalue=parser.WriteContentString(YPC_CREATION_DATE, string);
		if ( retvalue )	return retvalue;
	string=lastchanged_time.toString();
	retvalue=parser.WriteContentString(YPC_LASTCHANGED_DATE, string);
		if ( retvalue )	return retvalue;
	return YObject::GetConfig(parser);
}



//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpace::SetAuthor(const QString &a)
{
	if ( author!=a )
	{
		author=a;
		ObjectUpdated();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpace::SetCreationTime(const QDateTime &d)
{
	if ( creation_time != d )
	{
		creation_time=d;
		ObjectUpdated();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void YWorkSpace::SetLastChangedTime(const QDateTime &d)
{
	if ( lastchanged_time != d )
	{
		lastchanged_time=d;
		ObjectUpdated();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//! Input: worlspace file path (e.g. ".\projects\example.yasa"
void YWorkSpace::SetFilePath(const QString &path)
{
	QFileInfo	fileinfo(path);
	if ( fileinfo.dirPath(true) != filedir || fileinfo.fileName() != filename )
	{
		filename=fileinfo.fileName();
		filedir=fileinfo.dirPath(true);
		ObjectUpdated();
	}
}

